/// <field type="Object" static="true">
/// A wizard dialog window, where each step of the wizard is an HTML template.
/// </field>
DellClarity.Controls.AdvancedWizard = (function () {
    var retVal = {
        __wizardTemplateUrl: 'templates/advancedwizardtemplate.html',
        __wizardTemplateName: 'templates/advancedwizardtemplate.html',
        __idIncrementer: 0,

        create: function (options) {
            /// <summary>
            /// Creates a wizard using the specified options.
            /// </summary>
            /// <param name="options" type="AdvancedWizardOptions" mayBeNull="true" optional="true">
            /// Determines how the wizard appears and behaves.
            /// </param>
            /// <returns type="AdvancedWizard" />

            /// <var type="AdvancedWizardOptions">Options for the AdvancedWizard control</var>
            var defaultOptions = {
                /// <field type="String">The text that is displayed in the wizard's title bar</field>
                title: '',
                /// <field type="String">The ID that will be used to build a unique ID for the wizard</field>
                id: '',
                /// <field type="Object" mayBeNull="true">The model object to which the wizard is bound</field>
                model: null,
                /// <field type="Array" elementType="AdvancedWizardStepOptions">
                /// The steps of the wizard. This is a shortcut for the AdvancedWizard.createStep function.
                /// </field>
                steps: [],
                /// <field type="Boolean">
                /// Indicates whether a new Modal control will be created automatically to contain the wizard.
                /// If set to false, then you must create your own Modal and insert the wizard into it.
                /// </field>
                createModal: true,
                /// <field type="String" mayBeNull="true">The URL of the help page for the wizard</field>
                helpUrl: '',
                /// <field type="Number" integer="true" mayBeNull="true">
                /// The desired width of the wizard. Leave null to use the default width.
                /// </field>
                width: '865px',
                /// <field type="Number" integer="true" mayBeNull="true">
                /// The desired height of the wizard. Leave null to use the default height.
                /// </field>
                height: '550px',
                /// <field type="Function">The function to be invoked when the wizard is completed</field>
                finishAction: function () {
                    wizard.close();
                },
                /// <field type="Function">The function to be invoked when the wizard is canceled</field>
                cancelAction: function () {
                    wizard.close();
                },
                /// <field type="Function">Deprecated. DO NOT USE</field>
                historyMarkingAction: null,
                /// <field type="Function">When creating the modal set = false to turn off Close button</field>
                allowClose: true,

                closeAction: null,
            };

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            // Increment the ID counter to ensure a unique ID for the wizard
            var idIncrementer = DellClarity.Controls.AdvancedWizard.__idIncrementer;
            DellClarity.Controls.AdvancedWizard.idIncrementer = DellClarity.Controls.AdvancedWizard.__idIncrementer + 1;

            var initialized = false;

            /// <var type="AdvancedWizard">
            /// Provides properties and methods for controlling the wizard.
            /// </var>
            var wizard = {
                /// <field type="String">
                /// The unique ID of the wizard. You can use this ID to find the wizard
                /// in the DOM.
                /// </field>
                id: options.id + '_wizard_' + idIncrementer,

                /// <field type="jQuery">
                /// A jQuery selector that is scoped to the wizard.
                /// You can use wizard.$(selector) to find elements within the scope of this wizard.
                /// </field>
                $: function (selector) {
                    return $('#' + this.id).find(selector);
                },

                /// <field type="Object">Deprecated.  DO NOT USE</field>
                wizardInfoMessage: ko.observable(),

                /// <field type="Array" value="[]" elementType="AdvancedWizardStep">The steps of the wizard</field>
                steps: ko.observableArray(),

                /// <field type="Array" elementType="AdvancedWizardStep">
                /// The steps and substeps of the wizard, all flattened into a single array
                /// </field>
                denormalizedSteps: ko.observableArray(),

                /// <field type="Boolean">Deprecated.  DO NOT USE</field>
                isScrollable: ko.observable(true),

                /// <field type="Boolean">
                /// Indicates whether the wizard is being used to edit existing data (rather than creating new data)
                /// </field>
                editMode: ko.observable(false),

                /// <field type="Boolean">Deprecated.  DO NOT USE</field>
                propertyPageMode: ko.observable(false),

                /// <field type="Object" mayBeNull="true">The model object to which the wizard is bound</field>
                model: options.model,

                /// <field type="Object" mayBeNull="true">Set to false to turn off modal close</field>
                allowClose: options.allowClose,

                /// <field type="Modal" mayBeNull="true">The Modal control that contains the wizard</field>
                modal: null,

                /// <field type="AdvancedWizardStep" mayBeNull="true">The current step of the wizard</field>
                activeStep: ko.observable(),

                /// <field type="Number" integer="true" mayBeNull="true">The index of the current step</field>
                activeStepIndex: null,

                /// <field type="Boolean">Flag to check if confirmation modal is open</field>
                confirmationModalOpen: ko.observable(false),

                ///<field type="Boolean">Flag to check wizard not closed on Esc
                //closeFlag: ko.observable(false),

                addSteps: function (steps) {
                    /// <summary>
                    /// Adds an array of steps to the wizard.
                    /// </summary>
                    /// <param name="steps" type="Array" elementType="AdvancedWizardStep">
                    /// The steps to add to the wizard.  Create these using the AdvancedWizard.createStep function.
                    /// </param>

                    var indexIncrementer = 0;

                    var denormalizedSteps = [];

                    $.each(steps, function (i, step) {
                        if (step.subSteps.length > 0) {
                            //is this a collection of substeps?
                            $.each(step.subSteps, function (subStepIndex, subStep) {
                                subStep.index = indexIncrementer++;
                                denormalizedSteps.push(subStep);
                            });
                        } else {
                            //it's not a collection of substeps
                            step.index = indexIncrementer++;
                            denormalizedSteps.push(step);
                        }
                    });

                    this.steps(steps);
                    this.denormalizedSteps(denormalizedSteps);
                },

                createStep: function (stepOptions) {
                    /// <summary>
                    /// Creates a wizard step using the specified options.
                    /// </summary>
                    /// <param name="stepOptions" type="AdvancedWizardStepOptions">
                    /// Determines the appearance and behavior of the step.
                    /// </param>
                    /// <returns type="AdvancedWizardStep" />

                    var self = this;

                    /// <var type="AdvancedWizardStepOptions">Options for creating a wizard step</var>
                    var defaultStepOptions = {
                        /// <field type="String">The title of the step</field>
                        title: null,

                        /// <field type="Boolean">Indicates whether the step has been completed</field>
                        isCompleted: false,

                        /// <field type="String">The ID of the HTML template that contains the step's contents</field>
                        templateName: null,

                        /// <field type="Function">
                        /// A function that returns a boolean indicating whether the Next button should be enabled.
                        /// </field>
                        isNextButtonActive: function () {
                            return true;
                        },

                        /// <field type="Function">
                        /// A function that returns a boolean indicating whether the Back button should be enabled.
                        /// </field>
                        isBackButtonActive: function () {
                            return true;
                        },

                        /// <field type="Function">
                        /// A function that returns a boolean indicating whether the Finish button should be enabled.
                        /// </field>
                        isFinishButtonActive: function () {
                            return false;
                        },

                        /// <field type="Function">
                        /// A function that will be invoked when the user clicks the Next button on this step.
                        /// </field>
                        nextButtonHandler: wizard.defaultNextOrFinishAction,

                        nextButtonErrorHandler: function () {
                        },

                        /// <field type="Function" value="onStepLoadHandler(true)">
                        /// A function that will be invoked whenever the user moves to this step of the wizard.
                        /// A boolean parameter indicates whether the step is being shown for the first time.
                        /// </field>
                        onStepLoadHandler: function (firstTime) {
                        },

                        /// <field type="String">The caption of the Next button for this step</field>
                        nextButtonString: null,

                        /// <field type="Function">
                        /// A function that returns the caption for the Next button for this step.
                        /// &#10; If set, this function overrides the nextButtonString property.
                        /// </field>
                        nextButtonText: function () {
                            // If the step has specified a custom string for the "Next" button, then use it
                            if (stepOptions.nextButtonString)
                                return stepOptions.nextButtonString;

                            if (DellClarity.Config.apiVersion >= 2.5) {
                                // As of DellClarity 2.5, the Next button will always say "Next", unless
                                // the step has a specific nextButtonString property (see above)
                                return DellClarity.Localization.Translations.wizard.next;
                            }
                            else {
                                // Prior to DellClarity 2.5, the Next button only said "Next" for 
                                // substeps (except the last one).  Otherwise, it said "Save and Continue".
                                // This is confusing and misleading.  If a step is going to save the user's
                                // progress, and wants to indicate that via the Next button, then it should
                                // set the nextButtonString property instead.
                                if (this.isSubStep() && !this.isLastSubStep())
                                    return DellClarity.Localization.Translations.wizard.next;
                                return DellClarity.Localization.Translations.wizard.saveAndContinue;
                            }
                        },

                        /// <field type="String">The caption of the Finish button for this step</field>
                        finishButtonString: null,

                        /// <field type="Function">
                        /// A function that returns the caption for the Finish button for this step.
                        /// &#10; If set, this function overrides the finishButtonString property.
                        /// </field>
                        finishButtonText: function () {
                            // If the step has specified a custom string for the "Next" button, then use it
                            if (stepOptions.finishButtonString)
                                return stepOptions.finishButtonString;

                            // The Finish button will always say "Finish", unless
                            // the step has a specific finishButtonString property (see above)
                            return DellClarity.Localization.Translations.wizard.finish;
                        },

                        /// <field type="Array" elementType="AdvancedWizardStepOptions">
                        /// An array of sub-step options.
                        /// </field>
                        subSteps: []
                    };

                    stepOptions = $.extend(defaultStepOptions, stepOptions);


                    /// <var type="AdvancedWizardStep">A single step of an AdvancedWizard control</var>
                    var theStep = {
                        /// <field type="AdvancedWizardStep" mayBeNull="true">The parent step, if any</field>
                        parentStep: null, // is set by the parent

                        /// <field type="Boolean" mayBeNull="true">Indicates if the user is currently on this step</field>
                        isActive: null,   // defined just below 

                        /// <field type="Boolean" mayBeNull="true">Indicates if the step is disabled</field>
                        isDisabled: null, // defined just below 

                        /// <field type="AdvancedWizard">The AdvancedWizard control that contains this step</field>
                        wizard: self,

                        /// <field type="String">The title of this step</field>
                        title: stepOptions.title,

                        /// <field type="Boolean">Indicates if this step has been completed</field>
                        isCompleted: ko.observable(stepOptions.isCompleted),

                        /// <field type="String">The ID of the HTML template that contains this step's contents</field>
                        templateName: stepOptions.templateName,

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether the Next button is active
                        /// </field>
                        isNextButtonActive: ko.dependentObservable(stepOptions.isNextButtonActive, options.model),

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether the Back button is active
                        /// </field>
                        isBackButtonActive: ko.dependentObservable(stepOptions.isBackButtonActive, options.model),

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether the Finish button is active
                        /// </field>
                        isFinishButtonActive: ko.dependentObservable(stepOptions.isFinishButtonActive, options.model),

                        /// <field type="Function">
                        /// A function that returns the caption for the Next button for this step.
                        /// &#10; If set, this function overrides the nextButtonString property.
                        /// </field>
                        nextButtonText: stepOptions.nextButtonText,

                        /// <field type="Function">
                        /// A function that returns the caption for the Finish button for this step.
                        /// &#10; If set, this function overrides the finishButtonString property.
                        /// </field>
                        finishButtonText: stepOptions.finishButtonText,

                        /// <field type="Function">
                        /// A function that will be invoked when the user clicks the Next button on this step.
                        /// </field>
                        nextButtonHandler: function () { return doIfValid(wizard, stepOptions.nextButtonHandler, stepOptions.nextButtonErrorHandler); },

                        /// <field type="Function" value="onStepLoadHandler(true)">
                        /// A function that will be invoked whenever the user moves to this step of the wizard.
                        /// A boolean parameter indicates whether the step is being shown for the first time.
                        /// </field>
                        onStepLoadHandler: stepOptions.onStepLoadHandler,

                        /// <field type="Array" elementType="AdvancedWizardStepOptions">
                        /// An array of sub-step options.
                        /// </field>
                        subSteps: stepOptions.subSteps,

                        /// <field type="Number" integer="true">The index of this step among other steps of the wizard</field>
                        index: ko.observable(),

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether this step has sub-steps
                        /// </field>
                        hasSubSteps: function () {
                            return theStep.subSteps.length > 0;
                        },

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether this step is the final step of the wizard
                        /// </field>
                        isLastStep: function () {
                            return theStep.index >= self.denormalizedSteps().length - 1;
                        },

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether this step is a sub-step
                        /// </field>
                        isSubStep: function () {
                            return this.parentStep !== null;
                        },

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether this step is the last sub-step within its parent step
                        /// </field>
                        isLastSubStep: function () {
                            if (this.parentStep === null)
                                return false;
                            var parentSubSteps = this.parentStep.subSteps;
                            return parentSubSteps[parentSubSteps.length - 1] === this;
                        },

                        /// <field type="Function">
                        /// A function that returns a boolean value indicating whether this step is the first step of the wizard
                        /// </field>
                        isFirstStep: function () {
                            return theStep.index === 0;
                        },

                        /// <field type="Function">
                        /// This is an internal DellClarity API member. DO NOT USE IT IN YOUR APPLICATION
                        /// </field>
                        makeActive: function () {
                            if (theStep.subSteps.length > 0)
                                self.activeStep(theStep.subSteps[0]);
                            else
                                self.activeStep(theStep);

                            //we went backwards, mark other steps as not completed...
                            if (!self.editMode()) {

                                var stepFound = false;

                                $.each(self.denormalizedSteps(), function () {
                                    if (this === self.activeStep())
                                        stepFound = true;

                                    if (stepFound) {
                                        this.isCompleted(false);
                                        if (this.parentStep) {
                                            this.parentStep.isCompleted(false);
                                        }
                                    }
                                });


                            }

                            wizard.setFocus();
                            setCurrentStepForm(wizard);
                        },

                        /// <field type="Function">
                        /// This is an internal DellClarity API member. DO NOT USE IT IN YOUR APPLICATION
                        /// </field>
                        click: function () {
                            if (theStep.isActive())
                                return;

                            if (theStep.isDisabled())
                                return;

                            this.makeActive();
                        }
                    };

                    theStep.isActive = ko.dependentObservable(function () {
                        if (self.activeStep() === theStep)
                            return true;

                        var subSteps = theStep.subSteps;
                        for (var i = 0; i < subSteps.length; i++) {
                            if (self.activeStep() === subSteps[i])
                                return true;
                        }

                        return false;
                    }, self);

                    theStep.isDisabled = ko.dependentObservable(function () {

                        //edit mode so unlock all steps
                        if (self.editMode && self.editMode())
                            return false;

                        if (self.activeStep === null)
                            return true;

                        if (self.activeStep() === null)
                            return true;

                        //we're on the active step so it's not disabled
                        if (self.activeStep() === theStep)
                            return false;

                        //we're on the active step so it's not disabled
                        if (theStep.isCompleted() === true)
                            return false;

                        var subSteps = theStep.subSteps;

                        if (theStep.hasSubSteps() === true) {
                            var substepscomplete = true;
                            for (var stepIndex = 0; stepIndex < subSteps.length; stepIndex++) {
                                if (!subSteps[stepIndex].isCompleted())
                                    substepscomplete = false;
                            }
                            if (substepscomplete) return false;
                        }

                        for (var i = 0; i < subSteps.length; i++) {
                            if (self.activeStep() === subSteps[i])
                                return false;
                        }

                        return true;

                    }, self);

                    $.each(theStep.subSteps, function (i, subStep) {
                        subStep.parentStep = theStep;
                    });

                    return theStep;
                },

                /// <field type="Function">Deprecated. DO NOT USE.</field>
                setFocus: function () {
                    try {
                        wizard.$('.currentStepContainer input').first().focus();
                    }
                    catch (e) {

                    }
                },

                goToNextStep: function () {
                    /// <summary>
                    /// Completes the current step and moves the wizard to the next step.
                    /// </summary>

                    wizard.activeStep().isCompleted(true);
                    if (wizard.activeStep().isLastSubStep())
                        wizard.activeStep().parentStep.isCompleted(true);

                    if (!wizard.propertyPageMode()) {
                        // Make sure there is a next step
                        var nextIndex = wizard.activeStepIndex() + 1;
                        if (nextIndex >= wizard.denormalizedSteps().length)
                            return;

                        // Load the next step
                        var nextStep = wizard.denormalizedSteps()[nextIndex];
                        wizard.activeStep(nextStep);
                        setCurrentStepForm(wizard);
                    } else {
                        wizard.wizardInfoMessage(DellClarity.Controls.MessageBox.createSuccessMessage(undefined, DellClarity.Localization.Translations.wizard.successmessage));
                    }

                    wizard.setFocus();
                },

                goToPreviousStep: function () {
                    /// <summary>
                    /// Moves the wizard to the previous step, without completing the current step.
                    /// </summary>

                    var previousIndex = wizard.activeStepIndex() - 1;

                    if (previousIndex < 0)
                        return;

                    var previousStep = wizard.denormalizedSteps()[previousIndex];

                    //allow going back if you're still in the same step...
                    //otherwise force back to parent
                    if (previousStep.parentStep && (!this.parentStep || this.parentStep.title !== previousStep.parentStep.title))
                        previousStep = previousStep.parentStep;

                    previousStep.makeActive();
                },

                goToStep: function (index) {
                    /// <summary>
                    /// Moves the wizard to the specified step, without completing the current
                    /// step or any steps in-between.
                    /// </summary>
                    /// <param type="Number" integer="true">The index of the step to move to</param>

                    var steps = this.denormalizedSteps();
                    if (index >= steps.length)
                        return;

                    steps[index].makeActive();
                },

                getCurrentStepIndex: function () {
                    /// <summary>
                    /// Returns the index of the current step.
                    /// </summary>
                    /// <returns type="Number" integer="true" />

                    return this.denormalizedSteps.indexOf(wizard.activeStep());
                },

                jumpToStep: function (index) {
                    /// <summary>
                    /// Moves the wizard to the specified step, and marks all steps in-between
                    /// as completed.
                    /// </summary>
                    /// <param type="Number" integer="true">The index of the step to jump to</param>

                    var steps = this.denormalizedSteps();
                    if (index >= steps.length)
                        return;

                    $.each(this.denormalizedSteps(), function (i, step) {
                        if (i < index)
                            step.isCompleted(true);
                    });

                    $.each(this.steps(), function (i, step) {
                        if (step.hasSubSteps()) {
                            var stepsCompleted = true;
                            $.each(step.subSteps, function (subStepIndex, subStep) {
                                if (!subStep.isCompleted())
                                    stepsCompleted = false;
                            });
                            step.isCompleted(stepsCompleted);
                        }
                    });

                    steps[index].makeActive();
                },

                show: function () {
                    /// <summary>
                    /// Shows the Modal dialog containing the wizard.
                    /// </summary>

                    this.initialize();
                    this.modal.show();
                },

                close: function () {
                    /// <summary>
                    /// Closes the Modal dialog containing the wizard.
                    /// </summary>
                    //console.log('closeFlag:::' + this.closeFlag);
                    //if (this.modal && this.closeFlag == false) {
                    if (this.modal) {
                        //console.log('its closing...');
                        this.modal.close();
                    }

                    // unbind Keyup event
                    $("body").unbind("keyup.coreModal" + options.id);
                },

                cancel: function () {
                    /// <summary>
                    /// Cancels the wizard, and invokes the cancelAction callback, if any.
                    /// </summary>
                    var self = this;
                    //self.closeFlag = false;
                    var modal = DellClarity.Controls.ConfirmationModal.create({
                        title: DellClarity.Localization.Translations.wizard.confirmcanceltitle,
                        contents: DellClarity.Localization.Translations.wizard.confirmcancel,
                        allowCancel: true,
                        closeAction: function () {
                            self.confirmationModalOpen(false);
                            if (modal.accepted()) {
                                if (options.cancelAction)
                                    options.cancelAction();
                            }
                        }
                    });

                    this.confirmationModalOpen(true);
                    modal.show();

                },

                setCancelAction: function (theCancelAction) {
                    /// <summary>Deprecated. DO NOT USE.</summary>
                    options.cancelAction = theCancelAction;
                },

                finish: function () {
                    /// <summary>
                    /// Finishes the wizard, and invokes the finishAction callback, if any.
                    /// </summary>

                    if (options.finishAction)
                        options.finishAction();

                    if (wizard.propertyPageMode()) {
                        var modal = DellClarity.Controls.ConfirmationModal.create({
                            title: DellClarity.Localization.Translations.wizard.successtitle,
                            contents: DellClarity.Localization.Translations.wizard.successmessage,
                            iconClass: DellClarity.Constants.LargeIcons.success
                        });
                        modal.show();
                    }
                },

                setFinishAction: function (theFinishAction) {
                    /// <summary>Deprecated. DO NOT USE.</summary>
                    options.finishAction = theFinishAction;
                },

                setHistoryMarkingAction: function (theHistoryMarkingAction) {
                    /// <summary>Deprecated. DO NOT USE.</summary>

                    // takes a function of the form: function(stepIndex) {}
                    // when each step is accessed, the given function will be called so that the wizard can 
                    // set any browser history in whatever way the particular page needs
                    options.historyMarkingAction = theHistoryMarkingAction;
                },

                initialize: function () {
                    /// <summary>
                    /// This is an internal DellClarity API function. DO NOT USE IT IN YOUR APPLICATION
                    /// </summary>

                    if (initialized) {
                        this.reset();
                    }

                    DellClarity.Resources.loadTemplate(DellClarity.Controls.AdvancedWizard.__wizardTemplateName, DellClarity.Controls.AdvancedWizard.__wizardTemplateUrl);

                    var self = this;

                    //self.closeFlag = false;

                    // bind Keyup event
                    $("body").bind("keyup.coreModal" + options.id, function (event) {

                        // 27 == "esc"
                        if (event.which === 27) {
                            event.preventDefault();
                            //self.closeFlag = true;
                            //console.log('it wont close');
                            return false;
                        }

                        // 13 == "Enter"
                        if (event.which === 13 && self.confirmationModalOpen() == false) {
                            var isWizardLastStep = self.activeStep().isLastStep();
                            var isWizardFinishButtonDisabled = $('#idAdvancedWizardFinishBtn').is(':disabled');
                            if (isWizardLastStep && isWizardFinishButtonDisabled)
                                return false;
                            else
                                self.activeStep().nextButtonHandler();
                        }
                    });

                    this.activeStepIndex = ko.dependentObservable(function () {
                        if (self.activeStep())
                            return self.activeStep().index;
                        else
                            return null;
                    }, self);

                    this.numberOfSteps = ko.dependentObservable(function () {
                        return wizard.denormalizedSteps().length;
                    }, this);

                    this.nextAvailableStep = ko.dependentObservable(function () {
                        var nextAvailableIndex = null;
                        $.each(self.denormalizedSteps(), function (i, step) {
                            if (!step.isCompleted() && (nextAvailableIndex === null)) {
                                nextAvailableIndex = step.index;
                            }
                        });

                        nextAvailableIndex = nextAvailableIndex !== null ? nextAvailableIndex : self.denormalizedSteps().length - 1;

                        return this.denormalizedSteps()[nextAvailableIndex];
                    }, this);

                    this.stepProgressText = ko.dependentObservable(function () {
                        var token = DellClarity.Localization.Translations.wizard.stepXofY;
                        var currentStepText = self.activeStepIndex() + 1;
                        var totalSteps = self.numberOfSteps();
                        var completeText = DellClarity.Localization.Translator.translate(token, { currentStep: currentStepText, totalSteps: totalSteps });
                        return completeText;
                    }, this);

                    this.activeStep.subscribe(function (newValue) {
                        if (newValue !== null && options.historyMarkingAction !== null) {
                            options.historyMarkingAction(newValue.index);
                        }

                        if (newValue !== null) {
                            newValue.onStepLoadHandler(newValue.__firstTime === undefined);
                            newValue.__firstTime = false;
                        }

                        self.wizardInfoMessage(null);
                    });

                    initialized = true;
                    this.reset();
                },

                reset: function () {
                    this.nextAvailableStep().makeActive();
                },

                defaultNextOrFinishAction: function () {
                    /// <summary>
                    /// Invokes the nextButtonHandler of the current step, or the finishAction of the wizard, 
                    /// depending on whether the step is the last step of the wizard.
                    /// </summary>

                    var doFinishHandler;

                    if (DellClarity.Config.apiVersion < 2.5) {
                        // In older versions of DellClarity, we ALWAYS do the Next action, 
                        // even when the Finish button is clicked. It's up to the Next action to call Finish()
                        doFinishHandler = false;
                    }
                    else {
                        // As of DellClarity 2.5, we call the Next action on all but the last step.
                        // On the last step, we call the Finish action instead.
                        doFinishHandler = this.activeStep().isLastStep();
                    }

                    if (doFinishHandler) {
                        this.finish();
                    }
                    else {
                        this.wizardInfoMessage(null);
                        if (!this.propertyPageMode()) {
                            this.goToNextStep();
                        } else {
                            this.wizardInfoMessage(DellClarity.Controls.MessageBox.createSuccessMessage(undefined, DellClarity.Localization.Translations.wizard.successmessage));
                        }
                    }
                },
            };



            // If steps were specified in the options parameter, then add them to the wizard
            if (options.steps.length > 0) {
                var wizardSteps = [];

                // Convert each step from a JSON object to a WizardStep
                for (var i = 0; i < options.steps.length; i++) {
                    var step = options.steps[i];

                    if (!isNullUndefinedOrEmpty(step.subSteps)) {
                        // First, convert each subStep from a JSON object to a WizardStep
                        for (var j = 0; j < step.subSteps.length; j++) {
                            var subStep = step.subSteps[j];
                            step.subSteps[j] = wizard.createStep(subStep);
                        }
                    }

                    // Add this step to the array of WizardSteps
                    wizardSteps.push(wizard.createStep(step));
                }

                // Add the WizardSteps to the wizard
                wizard.addSteps(wizardSteps);
            }

            // Create the modal for the wizard
            if (options.createModal) {
                var modal = DellClarity.Controls.Modal.create({
                    title: options.title,
                    id: options.id,
                    helpUrl: options.helpUrl,
                    width: options.width,
                    height: options.height,
                    templateName: DellClarity.Controls.AdvancedWizard.__wizardTemplateName,
                    model: wizard,
                    allowClose: options.allowClose,
                    closeAction: options.closeAction
                });

                wizard.modal = modal;
            }

            // Add legacy APIs for backward compatibility
            if (DellClarity.Config.apiVersion < 2.5) {
                wizard.gotoStep = wizard.goToStep;
                wizard.jumptoStep = wizard.jumpToStep;
                wizard.gotoNextStep = wizard.goToNextStep;
                wizard.gotoPreviousStep = wizard.goToPreviousStep;
            }

            return wizard;
        }
    };

    function getCurrentStepForm(wizard) {
        /// <summary>Returns the form element (if any) for the current step.</summary>
        /// <param name="wizard" type="Object">The wizard whose current form is returned.</param>

        return wizard.$('.stepContents.active form')[0];
    }

    function setCurrentStepForm(wizard) {
        /// <summary>Sets the current form element</summary>
        /// <param name="wizard" type="Object">The wizard whose current form is set</param>

        var form = getCurrentStepForm(wizard);
        if (form) {
            // If the form doesn't have an ID, then add one
            if (!form.id) {
                form.id = 'DellClarity_AdvancedWizard_Validation_' + Math.floor(Math.random() * 1000);
            }

            // Bind the wizard's Next and Finish buttons to the form
            wizard.$('button[type=submit]').attr('form', form.id);
        }
        else {
            // This step has no form, so remove the "form" attribute from the buttons
            wizard.$('button[type=submit]').removeAttr('form');
        }
    }

    function validateStep(wizard) {
        /// <summary>
        /// Performs HTML 5 form validation on the current step of the wizard.
        /// </summary>
        /// <returns>True if the step is valid; otherwise, false</returns>

        // Get the <form> element for the current step
        var form = getCurrentStepForm(wizard);
        if (form) {
            isValid = form.checkValidity();

            if (!isValid && !Modernizr.formattribute) {
                // This browser doesn't support the `form` attribute, so in order to trigger
                // the validation UI, we have to append a submit button and then "click" it
                $('<input type="submit" id="DellClarity_AdvancedWizard_ValidateStep" style="display: none">')
                    .appendTo(form).click().remove();
            }

            if (form.getElementsByTagName('input').length == 1) {
                // HTML 2.0 specification (Section 8.2): When there is only one single-line text input field in a form, 
                // the user agent should accept Enter in that field as a request to submit the form.
                // For this reason, in order to prevent this behavior not expected, an aditional hidden input field is added.
                var ul = form.getElementsByTagName('ul')[0];
                if (ul) {
                    var id = "hiddenInput" + Math.floor(Math.random() * 1000).toString();
                    $('<li class="visuallyhidden"> <label for="' + id + '"><u>Input Hidden</u></label><input id="' + id + '" type="text" value="" class="value"></li>')
                       .appendTo(ul);
                }
            }

            return isValid;
        }

        // There is no form, so everything's valid
        return true;
    }

    function doIfValid(wizard, successHandler, errorHandler) {
        /// <summary>
        /// Invokes the specified function, only if the current step of the wizard is valid.
        /// </summary>

        if (validateStep(wizard)) {
            // The step is valid, so invoke the function
            successHandler.call(wizard);

            // Tell Knockout to cancel the event
            // (prevent the form from posting)
            return false;
        }

        errorHandler.call(wizard);
        // Tell Knockout NOT to cancel the event
        // (allow the browser to show form validation errors)
        return true;
    }

    //#region Legacy API (backward compatibility)

    if (DellClarity.Config.apiVersion < 2.5) {
        // Prior to DellClarity 2.5, UI controls were in the root namespace rather than DellClarity.Controls
        DellClarity.AdvancedWizard = retVal;
        DellClarity.AdvancedWizard.create = retVal.create;
        DellClarity.AdvancedWizard.idIncrementer = retVal.__idIncrementer;
        DellClarity.AdvancedWizard.wizardTemplateName = retVal.__wizardTemplateName;
        DellClarity.AdvancedWizard.wizardTemplateUrl = retVal.__wizardTemplateUrl;

        retVal.CreateWizard = function (wizardModel) {
            /// <summary>
            /// This method is deprecated. Please use the create() method instead.
            /// </summary>
            return DellClarity.Controls.AdvancedWizard.create({
                model: wizardModel,
                createModal: false // This is for backward-compatibility with the old API, where you have to create your own modal
            });
        };
    }

    //#endregion

    return retVal;
})();
