/// <summary>
/// A popup message box, optionally with confirm/cancel prompts.
/// </summary>
DellClarity.Controls.ConfirmationModal = (function () {
    var retVal = {
        create: function (options) {
            /// <summary>
            /// Creates a popup confirmation dialog using the specified options.
            /// </summary>
            /// <param name="options" type="ConfirmationModalOptions" mayBeNull="true" optional="true">
            /// Determines how the modal appears and behaves.
            /// </param>
            /// <returns type="ConfirmationModal" />

            /// <var type="ConfirmationModalOptions">Options for creating a ConfirmationModal control</var>
            var defaultOptions = {
                title: DellClarity.Localization.Translations.core.confirm,
                contents: '',
                allowCancel: false,
                helpUrl: '',
                iconClass: DellClarity.Constants.LargeIcons.nonCritical,
                items: [],
                width: 470,
                height: 350,
                closeAction: null
            };

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            var initialized = false;

            /// <var type="ConfirmationModal">
            /// Provides properties and methods for interacting with a ConfirmationModal control
            /// </var>
            return {
                /// <field type="String">The title text of the modal</field>
                title: ko.observable(options.title),

                /// <field type="String">The HTML or plain-text contents of the modal</field>
                contents: ko.observable(options.contents),

                /// <field type="Boolean">Indicates whether the modal will show a cancel button</field>
                allowCancel: ko.observable(options.allowCancel),

                /// <field type="String">The URL of the help page for the confirmation modal</field>
                helpUrl: ko.observable(options.helpUrl),

                /// <field type="String">An icon CSS class.  See the DellClarity.Constants.LargeIcons enumeration for valid values.</field>
                iconClass: ko.observable(options.iconClass),

                /// <field type="Array" elementType="String">An array of HTML or plain-text string content that will be displayed in the modal</field>
                items: ko.observableArray(options.items),

                /// <field type="Number" integer="true">The width of the modal</field>
                width: ko.observable(options.width),

                /// <field type="Number" integer="true">The desired height of the dialog (if no unit is specified, then pixels is assumed)</field>
                height: ko.observable(options.height),

                /// <field type="String">This is an internal DellClarity API member. DO NOT USE IT IN YOUR APPLICATION</field>
                modalTemplateUrl: 'templates/confirmationmodaltemplate.html',

                /// <field type="String">The ID of the ConfirmationModal HTML template</field>
                modalTemplateName: 'templates/confirmationmodaltemplate.html',

                /// <field type="Boolean">Indicates whether the confirmation modal was closed by the user clicking the "OK" button</field>
                accepted: ko.observable(false),

                /// <field type="Boolean">Indicates whether the confirmation modal is currently visible</field>
                visible: ko.observable(false),

                /// <field type="Function">A function that is invoked when the user clicks the Help link in the modal</field>
                help: function () {
                    if (this.helpUrl()) {
                        DellClarity.Help.showHelp(this.helpUrl());
                    }
                },

                /// <field type="Function">A function that returns a boolean value indicating whether the Help link is shown in the modal</field>
                isHelpVisible: function () {
                    if (this.helpUrl()) return true;
                    return false;
                },

                show: function (width, height) {
                    /// <summary>
                    /// Shows the confirmation modal.
                    /// </summary>
                    /// <param name="width" type="Number" integer="true" mayBeNull="true" optional="true">
                    /// The width of the confirmation modal.  Leave null or undefined to use the default width.
                    /// </param>
                    /// <param name="height" type="Number" integer="true" mayBeNull="true" optional="true">
                    /// The desired height of the modal.  Leave null or undefined to use the default height.
                    /// </param>

                    this.initialize();

                    if (width)
                        this.width(width);
                    if (height)
                        this.height(height);

                    this.visible(true);
                },

                close: function () {
                    /// <summary>
                    /// Closes the confirmation modal and invokes the closeAction function, if any.
                    /// &#10; Calling this method instead of accept() will set the accepted property to false.
                    /// </summary>

                    this.visible(false);

                    var element = $("#" + this.getModalId());

                    if (element.length != 0) {
                        ko.cleanNode(element.get(0));
                        element.remove();
                    }

                    if (options.closeAction) {
                        options.closeAction();
                    }
                },

                accept: function () {
                    /// <summary>
                    /// Closes the confirmation modal and invokes the closeAction function, if any.
                    /// &#10; Calling this method instead of close() will set the accepted property to true.
                    /// </summary>

                    this.accepted(true);
                    this.close();
                },

                getModalId: function () {
                    /// <summary>
                    /// Returns the ID of the confimation modal in the HTML DOM
                    /// </summary>

                    return "confirmationModal";
                },

                initialize: function () {
                    /// <summary>
                    /// This is an internal DellClarity API member. DO NOT CALL IT IN YOUR APPLICATION
                    /// </summary>

                    var self = this;

                    self.accepted(false);

                    if (!initialized) {
                        $("body").bind("keyup.coreModal" + self.getModalId(), function (event) {
                            // 27 == "esc"
                            if (event.which == 27) {
                                event.preventDefault();
                                //self.close();
                                // unbind the event
                                $("body").unbind("keyup.coreModal" + self.getModalId());
                            }
                        });

                        // bind Keydown event
                        $("body").bind("keydown.coreModal" + options.id, function (event) {

                            var code;
                            if (!event) var event = window.event; // some browsers don't pass e, so get it from the window
                            if (event.keyCode) code = event.keyCode; // some browsers use e.keyCode
                            else if (event.which) code = event.which;  // others use e.which

                            var inputfield = $(event.target).is("input:not([readonly]):not([type=radio]):not([type=checkbox]), textarea");
                            // 8 == "backspace"
                            if (code === 8 && !inputfield) {
                                return false;
                            }

                            // 112 == "F1", 123 == "F12", 82 == R to block Ctrl+R
                            if ((code >= 112 && code <= 123) || (code === 82 && !inputfield)) {
                                return false;
                            }
                        });

                        initialized = true;
                    }

                    DellClarity.Resources.loadTemplate(this.modalTemplateName, this.modalTemplateUrl);

                    var existingElement = $("#" + this.getModalId());

                    if (existingElement.length == 0) {
                        var element = $('<div id="' + this.getModalId() + '" data-bind="template: {name: modalTemplateName, data: $data }" />');
                        $('body').append(element);
                        ko.cleanNode(element.get(0));
                        ko.applyBindings(this, element.get(0));
                    }
                }
            };
        }
    };

    //#region Legacy API (backward compatibility)

    if (DellClarity.Config.apiVersion < 2.5) {
        // Prior to DellClarity 2.5, UI controls were in the root namespace rather than DellClarity.Controls
        DellClarity.ConfirmationModal = retVal;
        DellClarity.ConfirmationModal.Create = retVal.create;

        retVal.CreateConfirmationModalAndShow = function (title, text, allowCancel, helpLink, closeAction, iconClass) {
            /// <summary>
            /// This method is deprecated. Please use the create() method instead.
            /// </summary>
            var modal = DellClarity.ConfirmationModal.CreateConfirmationModal(title, text, allowCancel, helpLink, closeAction, iconClass);
            modal.show();
            return modal;
        };

        retVal.CreateConfirmationModal = function (title, text, allowCancel, helpLink, closeAction, iconClass) {
            /// <summary>
            /// This method is deprecated. Please use the create() method instead.
            /// </summary>
            return DellClarity.ConfirmationModal.Create({
                title: title,
                contents: text,
                allowCancel: allowCancel,
                helpUrl: helpLink,
                iconClass: iconClass,
                closeAction: closeAction
            });
        };
    }

    //#endregion

    return retVal;
})();
