/// <summary>
/// Displays the DellClarity login form that allows the user to authenticate into the application.
/// </summary>
DellClarity.Controls.Login = (function () {
    // Public DellClarity.Login API
    var retVal = {
        loginTemplateUrl: 'templates/logintemplate.html',
        loginTemplateName: 'templates/logintemplate.html',

        create: function (options) {
            /// <summary>
            /// Creates a Login control using the specified options.
            /// </summary>
            /// <param name="options" type="LoginOptions" mayBeNull="true" optional="true">
            /// Determines how the Login control appears and behaves.
            /// </param>
            /// <returns type="Login" />

            /// <var type="LoginOptions">Options for creating a Login control</var>
            var defaultOptions = {
                loginUrl: '',
                success: undefined,  // Callback function on successful login
                failure: undefined,  // Callback function on failed login
                useformpost: true  // Callback function on successful login
            };

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            var loginResponse = function (response, login) {
                if (response.success) {
                    if (DellClarity.Config.apiVersion >= 2.5) {
                        // Call the success callback (if any)
                        if (typeof options.success === 'function') {
                            options.success();
                        }
                        $("#btnLogin").progressFinish();
                        // Navigate to the response URL
                        navigateTo(response.url, response.route);
                    }
                    else {
                        // In DellClarity 1.0, it was the responsibility of the page to navigate to the response URL
                        $("#btnLogin").progressFinish();
                        options.success(response.url, response.route);
                    }
                } else {
                    $("#btnLogin").progressFinish();
                    login.verifyingCredentials(false);
                    login.failedLogin(true);
                    $('.login-error-div').show();
					if(response.accountLockedMessage != ""){
						$('#accountLocked').text(response.accountLockedMessage).show();
						$('#invalidLogin').hide();
					}else{
						$('#accountLocked').hide();
						$('#invalidLogin').show()
					}
                    login.message(response.message);
                    $('#password').focus();

                    // Call the failure callback (if any)
                    if (typeof options.failure === 'function') {
                        options.failure(login.model);
                    }
                }
            };

            var loginAjaxRequest = null;

            /// <var type="Login">
            /// Provides properties and methods for interacting with the Login control
            /// </var>
            return {
                /// <field type="Object" mayBeNull="true">The model to which the Login control is bound</field>
                model: null,

                /// <field type="Boolean">
                /// Indicates whether the Login control should use form post instead of ajax post (for java swing security)
                /// </field>
                useformpost: ko.observable(options ? options.useformpost : true),

                /// <field type="Boolean">
                /// Indicates whether the Login control is currently verifying the user's credentials with the server
                /// </field>
                verifyingCredentials: ko.observable(false),

                /// <field type="Function">Deprecated. DO NOT USE.</field>
                sessionTimeout: ko.observable(),

                /// <field type="Boolean">Indicates whether the user's login attempt failed</field>
                failedLogin: ko.observable(false),

                /// <field type="String">The login failure message received from the server</field>
                message: ko.observable(),

                /// <field type="Array" elementType="AccountType">A list of account types that user can choose from</field>
                loginAccounts: ko.observable(options ? options.loginAccounts : 0),

                /// <field type="Boolean">Indicates whether the "Login Account" field should be shown</field>
                loginAccountsVisibility: ko.observable(options ? options.loginAccountsVisibility : false),

                initialize: function () {
                    /// <summary>
                    /// This is an internal DellClarity API member. DO NOT USE IT IN YOUR APPLICATION
                    /// </summary>

                    DellClarity.Resources.loadTemplate(DellClarity.Controls.Login.loginTemplateName, DellClarity.Controls.Login.loginTemplateUrl);

                    var self = this;

                    if (self.loginAccountsVisibility()) {
                        self.model = {
                            username: ko.observable(),
                            password: ko.observable(),
                            domain: ko.observable(),
                            type: ko.observable(0)
                        };
                    } else {
                        self.model = {
                            username: ko.observable(),
                            password: ko.observable()
                        };
                    }

                    self.loginDomainFieldVisibility = ko.dependentObservable(function () {
                        return self.loginAccountsVisibility() && self.model.type() && self.model.type().value == 2; //Windows Domain Account
                    }, self);

                    self.shouldNextButtonBeEnabled = ko.dependentObservable(function () {
                        var isUsernameEmpty = isNullOrEmpty(self.model.username());
                        var isPasswordEmpty = isNullOrEmpty(self.model.password());
                        return !isUsernameEmpty && !isPasswordEmpty;
                    }, self);

                    this.resetLogin();
                    this.eventWire();
                },

                eventWire: function () {
                    /// <summary>
                    /// This is an internal DellClarity API member. DO NOT USE IT IN YOUR APPLICATION
                    /// </summary>

                    var self = this;
                    $(document).on('click.DellClarity.Controls.Login.cancel', '#btnCancel', function () { self.resetLogin(); });
                    $(document).on('submit.DellClarity.Controls.Login.login', '.loginControl form',
                        function (evt) {
                            evt.preventDefault();
                            $('.login-error-div').hide();
                            self.doLogin();
                        });
                    //$(".loginControl form").submit(self.doLogin);

                },

                resetLogin: function () {
                    if (loginAjaxRequest)
                        loginAjaxRequest.abort();

                    $('.login-error-div').hide();

                    this.verifyingCredentials(false);
                    this.failedLogin(false);

                    // this.model.username(null);
                    $('#password').focus();
                    this.model.password(null);

                    if (this.loginAccountsVisibility()) {
                        this.model.domain(null);
                        this.model.type(0);
                    }
                },

                doLogin: function () {
                    $('#btnLogin').progressInitialize().progressStart();

                    if (loginAjaxRequest)
                        loginAjaxRequest.abort();

                    var self = this;

                    this.verifyingCredentials(true);

                    $('#btnLogin').progressSet(Math.floor((Math.random() * 100) + 1));
                    if (self.useformpost()) {
                        //Adam needs these properties set for Java.
                        self.model.j_username = self.model.username();
                        self.model.j_password = self.model.password();

                        loginAjaxRequest = DellClarity.Ajax.PostJSONFormStyle(options.loginUrl, ko.toJS(self.model), function (response) { loginResponse(response, self); },
                            function (response) {
                                $("#btnLogin").progressFinish();
                                self.verifyingCredentials(false);
                                self.failedLogin(true);
                                $('.login-error-div').show();
                                if (response && response.errorMessage)
                                    self.message(response.errorMessage);
                            });
                    } else {
                        loginAjaxRequest = DellClarity.Ajax.PostJSON(options.loginUrl, ko.toJS(self.model), function (response) { loginResponse(response, self); },
                            function (response) {
                                $("#btnLogin").progressFinish();
                                self.verifyingCredentials(false);
                                self.failedLogin(true);
                                $('.login-error-div').show();
                                if (response && response.errorMessage)
                                    self.message(response.errorMessage);
                            });
                    }
                }
            };
        },

        logOut: function (logoutUrl) {
            if (logoutUrl) {
                var successHandler = function (response) {
                    window.location = response.url + '#' + response.route;
                };

                var request = DellClarity.Ajax.PostJSON(logoutUrl, null, successHandler);
            }
        },

        validateUsername: function (objThis) {
            var self = this;

            if (objThis.model.username() !== undefined) {
                var username = $.trim(objThis.model.username());

                //Checking for username to be in the format domain\user
                $('#username').get(0).setCustomValidity((username.match(/^[a-zA-Z][a-zA-Z0-9\-\.]{0,61}[a-zA-Z0-9]\\\w[\w\.\- ]+$/)) ? '' : DellClarity.SCVMM.Translations.login.errorMessage.invalidUsername);
            }
        }
    };

    function navigateTo(url, route) {
        /// <summary>
        /// Navigates to the specified URL and route (hash).
        /// </summary>

        if (!isNullUndefinedOrEmpty(route)) {
            if (route.startsWith('/')) {
                route = '#' + route;
            }
            else if (!route.startsWith('#/')) {
                route = '#/' + route;
            }
        }

        if (isNullUndefinedOrEmpty(url)) {
            window.location.hash = route;
        }
        else {
            if (isNullUndefinedOrEmpty(route)) {
                window.location = url;
            }
            else {
                window.location = url + route;
            }
        }
    }

    //#region Legacy API (backward compatibility)

    if (DellClarity.Config.apiVersion < 2.5) {
        // Prior to DellClarity 2.5, UI controls were in the root namespace rather than DellClarity.Controls
        DellClarity.Login = retVal;

        // Mimic the old DellClarity 2.5 API functions
        DellClarity.Login.CreateLogin = function (loginUrl, customSuccessHandler, customRender, options) {
            /// <summary>
            /// This method is deprecated.  Use the create() method instead.
            /// </summary>

            return retVal.create({
                loginUrl: loginUrl,
                success: customSuccessHandler,
                loginAccounts: options ? options.loginAccounts : undefined,
                loginAccountsVisibility: options ? options.loginAccountsVisibility : undefined
            });
        };
        DellClarity.Login.DoLogout = retVal.logOut;
        DellClarity.Login.AfterRender = function (elements) { };
    }

    //#endregion

    return retVal;
})();
