/// <summary>
/// A master/detail view, made up of two <section> elements, one that contains master data (usually a table),
/// and one that contains detail data.
/// </summary>
DellClarity.Controls.MasterDetails = (function () {
    var retVal = {
        create: function (options, detailsId) {
            /// <signature>
            ///     <summary>
            ///     Creates a Master/Details control using the specified HTML elements,
            ///     accepting all of the default options.
            ///     &#10; The element syntax should be:
            ///     &#10; &lt;div class="masterDetails"&gt; ... &lt;/section&gt;
            ///     &#10; &lt;section class="master"&gt; ... &lt;/section&gt;
            ///     &#10; &lt;section class="details"&gt; ... &lt;/section&gt;
            ///     </summary>
            ///     <param name="options" type="String">The ID of the master element</param>
            ///     <param name="detailsId" type="String">The ID of the details element</param>
            /// </signature>
            /// <signature>
            ///     <summary>
            ///     Creates a Master/Details control using the specified options.
            ///     &#10; The element syntax should be: 
            ///     &#10; &lt;section class="master"&gt; ... &lt;/section&gt;
            ///     &#10; &lt;section class="details"&gt; ... &lt;/section&gt;
            ///     </summary>
            ///     <param name="options" type="MasterDetailsOptions">
            ///     Determines how the Master/Details control appears and behaves.
            ///     </param>
            /// </signature>

            /// <var type="MasterDetailsOptions">Options for creating a Master/Details control</var>
            var defaultOptions = {
                masterId: '',            // the ID of the <section class="master"> element
                detailsId: '',           // the ID of the <section class="details"> element
                allowUserResizing: true, // display controls to allow the user to resize the details section?
                initialSize: 'normal',   // set the size of the master/details if allowUserResizing is true
            };

            // If two strings are passed, then it's just the IDs of the master and details elements
            if (typeof options === 'string' && typeof detailsId === 'string') {
                options = {
                    masterId: options,
                    detailsId: detailsId
                };
            }

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            // Get the master and details elements
            var $master = $('section#' + options.masterId);
            var $details = $('section#' + options.detailsId);
            //html tag that wraps the master and detail sections
            var $wrapper = $master.parent();

            if ($master.length !== 1 || $details.length !== 1) {
                throw new Error('DellClarity.Controls.MasterDetails was unable to find <section> elements with IDs of "' + options.masterId + '" and "' + options.detailsId + '"');
            }

            // Add the appropriate CSS classes
            $master.addClass('master');
            $details.addClass('details');

            // Progressively enhance the markup as needed
            if (options.allowUserResizing) {
                enableUserResizing($master, $details, $wrapper, options);
            }
        },

        update: function () {

        },
    };

    function enableUserResizing($master, $details, $wrapper, options) {
        /// <summary>
        /// Enhances the markup of the master/details elements with controls that allow
        /// the user to resize each panel.
        /// </summary>

        // TODO: This has not been implemented yet.

        if ($details.find('> header > span.windowStateButtons').length > 0) {
            return;
        }

        $details.find('> header').append(
                '<span class="windowStateButtons">' +
                    '<a id="btnMinimized" title="' + DellClarity.Localization.Translations.genericTerms.minimize + '" class="windowStateButton minimize">' +
                    DellClarity.Localization.Translations.genericTerms.minimize +
                    '</a><a id="btnNormal" title="' + DellClarity.Localization.Translations.genericTerms.normal + '" class="windowStateButton normal">' +
                    DellClarity.Localization.Translations.genericTerms.normal +
                    '</a>' +
                    '<a id="btnMaximized" title="' + DellClarity.Localization.Translations.genericTerms.maximize + '" class="windowStateButton maximize">' +
                    DellClarity.Localization.Translations.genericTerms.maximize +
                    '</a>' +
                    '</span>'
            );

        //$buttons = <span> that contains the button sprite
        var $buttons = $details.find('.windowStateButtons');

        $details.find('.windowStateButton.minimize').off('click.masterDetails').on('click.masterDetails', function () {
            removeSectionAndButtonClasses($master, $details, $wrapper, $buttons);
            setMinimize($master, $details, $wrapper, $buttons);
        });

        $details.find('.windowStateButton.normal').off('click.masterDetails').on('click.masterDetails', function () {
            removeSectionAndButtonClasses($master, $details, $wrapper, $buttons);
            setNormal($master, $details, $wrapper, $buttons);

        });

        $details.find('.windowStateButton.maximize').off('click.masterDetails').on('click.masterDetails', function () {
            removeSectionAndButtonClasses($master, $details, $wrapper, $buttons);
            setMaximize($master, $details, $wrapper, $buttons);
        });

        //upon initial create or bind complete
        if (options.initialSize === 'normal')
            setNormal($master, $details, $wrapper, $buttons);

        else if (options.initialSize === 'minimized')
            setMinimize($master, $details, $wrapper, $buttons);

        else if (options.initialSize === 'maximized')
            setMaximize($master, $details, $wrapper, $buttons);

    }

    function removeSectionAndButtonClasses($master, $details, $wrapper, $buttons) {
        /// <summary>
        /// Removes any .expand or .collapse class which may be attached to the 
        /// master or detail <section> or the buttons of the MasterDetails Control
        /// </summary>
        $wrapper.removeClass('collapsed expanded');
        $buttons.removeClass('minimize normal maximize');


        $master.removeClass('collapsed-1');
        $master.removeClass('collapsed');
        $master.removeClass('expanded');
        $details.removeClass('collapsed-1');
        $details.removeClass('collapsed expanded');

    }

    function setNormal($master, $details, $wrapper, $buttons) {
        $buttons.addClass('normal');
    }

    function setMinimize($master, $details, $wrapper, $buttons) {
        $wrapper.addClass('collapsed');
        $buttons.addClass('minimize');
        $master.addClass('expanded');

        if ($master.find('header').length > 0) //adjust the master window height based on the presence of header or extra details added to master section
        {

            $details.addClass('collapsed');
        }
        else {
            $details.addClass('collapsed-1'); // if section starts from one level lower example section within subtab rather than directly within after header
            // $master.addClass('expanded-1');
        }
    }

    function setMaximize($master, $details, $wrapper, $buttons) {

        $buttons.addClass('maximize');

        $master.addClass('collapsed');

        $master.removeClass('expanded');

        if ($master.find('header').length > 0) {

            $master.addClass('collapsed');

        } else {
            //if section within subtabpanel
            $master.addClass('collapsed-1');
        }
        $details.addClass('expanded');

    }

    return retVal;
})();
