/// <summary>
/// A menu control that supports any level of hierarchy and can be skinned in virtually any way.
/// This control automatically keeps track of the selected/expanded item, and applies the corresponding CSS class(es).
/// </summary>
/// <remarks>
/// NOTE: This control responds to URL hash changes, regardless of whether the hash change
///       was caused by this control or something else.  Clicking a link in the menu just
///       performs a normal browser navigation.  Nothing special.  This ensures that *any*
///       browser navigation event will cause the menu to update accordingly.
/// </remarks>
DellClarity.Controls.Menu = (function () {
    var retVal = {
        /// <field type="String">
        /// The default menu template. Using this template saves you the trouble of hand-rolling your own,
        /// and ensures that your application will automatically benefit from any new features or changes in the future.
        /// </field>
        templateName: 'templates/menutemplate.html',

        create: function (options) {
            /// <signature>
            ///     <summary>
            ///     Creates a Menu control using the specified HTML element,
            ///     accepting all of the default options.
            ///     &#10; The element syntax should be: 
            ///     &#10; &lt;nav class="menu" role="navigation"&gt; 
            ///     </summary>
            ///     <param name="options" type="String">
            ///     The ID of the HTML element that will become the Menu control.
            ///     </param>
            /// </signature>
            /// <signature>
            ///     <summary>
            ///     Creates a Menu control using the specified options.
            ///     &#10; The element syntax should be: 
            ///     &#10; &lt;nav class="menu" role="navigation"&gt; 
            ///     </summary>
            ///     <param name="options" type="MenuOptions">
            ///     Determines how the Menu control appears and behaves.
            ///     </param>
            /// </signature>

            /// <var type="MenuOptions">Options for creating a Menu control</var>
            var defaultOptions = {
                id: '',                 // the ID of the <nav class="menu"> element
                cssClass: '',           // The CSS class to apply to the menu (e.g. 'navigation', 'toolbar', etc.)
                selectOnExpand: false,  // select a menu item when it's expanded?
                expandOnSelect: true,   // expand a menu item when it's selected?
                allowMultipleExpanded: false, // allow more than one menu to be expanded?
            };

            // If a string is passed, then it's just the ID option
            if (typeof options === 'string') {
                options = { id: options };
            }

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            // Get the root menu element and progressively enhance it
            var $rootMenu = $('#' + options.id);
            if (!$rootMenu.exists()) {
                throw new Error('Unable to find a menu control with the ID "' + options.id + '"');
            }

            // Add the appropriate CSS classes to the menu
            $rootMenu.addClass('menu');
            if (!isNullUndefinedOrEmpty(options.cssClass)) {
                $rootMenu.addClass(options.cssClass);
            }

            // Add the .hasSubMenu class to all menus items that have sub-menus
            $rootMenu.find('li>ul').parent().addClass('hasSubMenu');

            // When a menu item is clicked, expand/collapse it
            var onclick = 'click.DellClarity.Controls.Menu';
            $rootMenu.off(onclick).on(onclick, 'li', function (evt) {
                expandCollapse($rootMenu, $(this), options);
                $rootMenu.removeClass('collapsed');
                evt.stopPropagation();
            });

            // When a menu header is clicked, expand/collapse it
            var onheaderclick = 'click.DellClarity.Controls.Menu.Header';
            $rootMenu.off(onheaderclick).on(onheaderclick, 'header', function (evt) {
                if ($rootMenu.hasClass('collapsed'))
                    $rootMenu.removeClass('collapsed');
                else {
                    $rootMenu.addClass('collapsed');
                }
                evt.stopPropagation();
            });

            // Select the menu item for the current page
            selectMenuItemForCurrentUrl($rootMenu, options);

            // If the URL hash changes, select the corresponding menu item
            var onhashchange = 'hashchange.DellClarity.Controls.Menu.' + options.id;
            $(window).off(onhashchange).on(onhashchange, function () {
                if ($rootMenu.parent().exists()) {
                    selectMenuItemForCurrentUrl($rootMenu, options);
                } else {
                    $(window).off(onhashchange);
                }
            });

        },

        __resolveHash: function (hash) {
            /// <summary>
            /// This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!
            /// </summary>

            if (isNullUndefinedOrEmpty(hash)) {
                return undefined;
            }

            if (hash.startsWith('#/')) {
                return hash;
            }

            return '#/' + hash;
        }
    };

    // Load the default menu template and give it a unique ID
    DellClarity.Resources.loadTemplate(retVal.templateName);

    function expandCollapse($rootMenu, $menuItem, options) {
        /// <summary>
        /// Expands or collapses the specified menu item's sub-menu (if any)
        /// </summary>

        // If the menu item has a sub-menu, then expand/collapse it
        if ($menuItem.hasClass('hasSubMenu')) {
            if ($menuItem.hasClass('expanded')) {
                // Collapse the sub-menu
                $menuItem.removeClass('expanded');
            }
            else {
                expand($rootMenu, $menuItem, options);
            }
        }
    }

    function expand($rootMenu, $menuItem, options) {
        /// <summary>
        /// Expands the specified menu item's sub-menu (if any).
        /// </summary>

        if (!options.allowMultipleExpanded) {
            // Collapse all other menus first (except parents of this menu item)
            $rootMenu.find('.expanded').each(function () {
                if (!$.contains(this, $menuItem[0])) {
                    $(this).removeClass('expanded');
                }
            });
        }

        // Expand the sub-menu
        $menuItem.addClass('expanded');

        if (options.selectOnExpand) {
            // Also navigate to the menu item's URL
            // NOTE: We override the expandOnSelect option to prevent infinite loops
            var newOptions = $.extend(options, { expandOnSelect: false });
            navigateTo($rootMenu, $menuItem, newOptions);
        }
    }

    function navigateTo($rootMenu, $menuItem, options) {
        /// <summary>
        /// Navigates to the specified menu item's URL (if any).
        /// </summary>

        // Navigate to the menu item's URL
        var href = $menuItem.children('a').attr('href');
        if (href !== undefined && href != null) {
            if (href.startsWith('#')) {
                window.location.hash = href;
            }
            else {
                window.location = href;
            }
        }
    }

    function selectMenuItemForCurrentUrl($rootMenu, options) {
        /// <summary>
        /// Selects the menu item that matches the current page's URL (if any).
        /// </summary>

        // Examine every menu item URL
        var $matchingItem = $rootMenu.find('li>a[href]').filter(function () {
            var href = $(this).attr('href');
            if (href !== undefined && href != null) {
                if (href.startsWith('#')) {
                    // For hash-based URLs, just make sure they match
                    return href === window.location.hash;
                }
                else {
                    // Non-hash URLs, could be relative or absolute, so
                    // just compare the end of the URL
                    return window.location.toString().endsWith(href);
                }
            }

            // This link does not match the current page
            return false;
        })

        // Multiple menu items could have the same URL, so just get the first one
        .first().parent();

        // If we found one, then select it
        if ($matchingItem.length > 0) {
            // NOTE: We override the selectOnExpand option to prevent navigating
            var newOptions = $.extend(options, { selectOnExpand: false });
            select($rootMenu, $matchingItem, newOptions);
        }
    }

    function select($rootMenu, $menuItem, options) {
        /// <summary>
        /// Selects the specified menu item.
        /// </summary>

        // Select this menu item (and de-select all others)
        $rootMenu.find('.selected').removeClass('selected');
        $menuItem.addClass('selected');

        if (!options.allowMultipleExpanded) {
            // Collapse all menus
            $rootMenu.find('.expanded').removeClass('expanded');
        }

        // Ensure that all parent menus are expanded
        var $parentMenuItems = $menuItem.parentsUntil($rootMenu, 'li');
        for (var i = $parentMenuItems.length - 1; i >= 0; i--) {
            var $parentMenuItem = $($parentMenuItems[i]);
            expand($rootMenu, $parentMenuItem, options);
        }

        if (options.expandOnSelect && !$menuItem.hasClass('expanded')) {
            // Also expand the menu item's sub-menu
            // NOTE: We override the selectOnExpand option to prevent infinite loops
            var newOptions = $.extend(options, { selectOnExpand: false });
            expandCollapse($rootMenu, $menuItem, newOptions);
        }
    }

    //#region Legacy API (backward compatibility)

    if (DellClarity.Config.apiVersion < 2.5) {
        // Prior to DellClarity 2.5, UI controls were in the root namespace rather than DellClarity.Controls
        DellClarity.Menu = retVal;

        // As of DellClarity 2.5, it is the responsibility of the application to set the Menu.model property.
        // The DellClarity.Menu control no longer automatically retrieves the menu data from a pre-defined URL

        /// <summary>
        /// NOTE: This function is deprecated. Use DellClarity.Controls.Menu.create() instead.
        /// </summary>
        retVal.loadMenu = function () {
            /// <summary>
            /// This method is deprecated. Use the create() method instead.
            /// </summary>

            // Make sure the necessary elements exist
            var legacyMenuContainer = document.getElementById('primaryNavigation');
            var legacyMenu = document.getElementById('menuContainer');
            if (legacyMenuContainer === null || legacyMenu === null) {
                throw new Error('The DellClarity.Menu.loadMenu() function requires the page to have elements with IDs of "primaryNavigation" and "menuContainer", but was unable to find any elements with these IDs.');
            }

            // Retrieve the menu data using the DellClarity 1.0 AJAX call
            DellClarity.Ajax.GetJSONSynchronously(DellClarity.CoreUrls.menu.getMenu, null, function (menuObj) {
                // Populate the menu
                // NOTE: As of DellClarity 2.5, it is the responsibility of the application to do this, NOT the framework
                DellClarity.ModelBinding.menuViewModel.items(valueOrDefault(menuObj, []));
                ko.cleanNode(legacyMenuContainer);
                ko.applyBindings({}, legacyMenuContainer);

                // Now that the menu is populated, call the new Menu API
                return DellClarity.Menu.create({
                    id: legacyMenu.id,
                    cssClass: 'navigation'
                });
            });
        };

        retVal.createModel = function (jsonObj) { };

        // The following functions have been replaced by DellClarity.Navigation
        retVal.loadJS = function (js, postLoadFunction) { };
        retVal.updatePage = function (hash) { };
        retVal.changeHash = function (url, postLoadFunction) { };
        retVal.bindModel = function (modelIdentifier, queryProperties) { };
    }

    //#endregion

    return retVal;
})();
