/// <summary>
/// A popup message box that displays simple alerts to the user, 
/// such as prompts, error messages, or warnings.
/// </summary>
DellClarity.Controls.MessageBox = (function () {
    var retVal = {
        /// <field type="String">
        /// The default messagebox template. Using this template saves you the trouble of hand-rolling your own,
        /// and ensures that your application will automatically benefit from any new features or changes in the future.
        /// </field>
        templateName: 'templates/messagebox.html',

        createSuccessMessage: function (id, message) {
            /// <summary>
            /// Displays a success message in the specified element.
            /// &#10; The element syntax should be: 
            /// &#10; &lt;section role="alert"&gt; 
            /// </summary>
            /// <param name="id" type="String">The ID of the element that will contain the message</param>
            /// <param name="message" type="String">The message to display</param>
            /// <returns type="Message"/>

            return DellClarity.Controls.MessageBox.create({
                id: id,
                type: 'Success',
                message: message,
                autoClose: true
            });
        },

        createInfoMessage: function (id, message) {
            /// <summary>
            /// Displays an informational message in the specified element.
            /// &#10; The element syntax should be: &lt;section role="alert"&gt; 
            /// </summary>
            /// <param name="id" type="String">The ID of the element that will contain the message</param>
            /// <param name="message" type="String">The message to display</param>
            /// <returns type="Message"/>

            return DellClarity.Controls.MessageBox.create({
                id: id,
                type: 'Info',
                message: message
            });
        },

        createWarningMessage: function (id, message) {
            /// <summary>
            /// Displays a warning message in the specified element.
            /// &#10; The element syntax should be: &lt;section role="alert"&gt; 
            /// </summary>
            /// <param name="id" type="String">The ID of the element that will contain the message</param>
            /// <param name="message" type="String">The message to display</param>
            /// <returns type="Message"/>

            return DellClarity.Controls.MessageBox.create({
                id: id,
                type: 'Warning',
                message: message
            });
        },

        createErrorMessage: function (id, message, fieldErrors) {
            /// <signature>
            ///     <summary>
            ///     Displays an error message in the specified element.
            ///     &#10; The element syntax should be: &lt;section role="alert"&gt; 
            ///     </summary>
            ///     <param name="id" type="String">The ID of the element that will contain the message</param>
            ///     <param name="message" type="String">The message to display</param>
            ///     <param name="fieldErrors" type="Array" elementType="FieldError" mayBeNull="true" optional="true" value="{field:'',errorMessage:'',errorDetails:''}">
            ///     An optional array of field-specific error messages to display
            ///     </param>
            ///     <returns type="Message"/>
            /// </signature>
            /// <signature>
            ///     <summary>
            ///     Displays an error message in the specified element.
            ///     &#10; The element syntax should be: &lt;section role="alert"&gt; 
            ///     </summary>
            ///     <param name="id" type="String">The ID of the element that will contain the message</param>
            ///     <param name="message" type="ErrorObj" value="{errorMessage:'',errorDetails:'',fldErrors:[]}">
            ///     A JSON error object from an AJAX call
            ///     </param>
            ///     <param name="fieldErrors" type="Array" elementType="FieldError" mayBeNull="true" optional="true" value="{field:'',errorMessage:'',errorDetails:''}">
            ///     An optional array of field-specific error messages to display
            ///     </param>
            ///     <returns type="Message"/>
            /// </signature>

            // Make sure the fieldErrors (if specified) have all the required properties
            fieldErrors = fieldErrors || message.fldErrors;
            if (fieldErrors) {
                for (var i = 0; i < fieldErrors.length; i++) {
                    var fieldError = fieldErrors[i];
                    fieldErrors[i] = {
                        field: fieldError.field || null,
                        message: fieldError.errorMessage || null,
                        details: fieldError.errorDetails || null
                    };
                }
            }

            // If the message is a string
            if (typeof message === 'string' || message instanceof String) {
                return DellClarity.Controls.MessageBox.create({
                    id: id,
                    type: 'Failure',
                    message: message,
                    fieldErrors: fieldErrors
                });
            }

            // If the message is a JSON error object, then map its properties to the 
            // corresponding options for MessageBox.create()
            return DellClarity.Controls.MessageBox.create({
                id: id,
                type: message && message.type ? message.type : 'Failure',
                message: message && message.errorMessage ? message.errorMessage : null,
                details: message && message.errorDetails ? message.errorDetails : null,
                fieldErrors: fieldErrors
            });
        },

        create: function (options) {
            /// <summary>
            /// Creates a MessageBox control using the specified options.
            /// </summary>
            /// <param name="options" type="MessageBoxOptions">Determines the appearance and content of the MessageBox.</param>
            /// <returns type="Message"/>

            /// <var type="MessageBoxOptions">Options for creating a MessageBox control</var>
            var defaultOptions = {
                id: '',                   // the ID of the <section role="alert"> element
                type: 'Info',
                autoShow: true,           // Whether to automatically show the message box
                autoClose: false,         // Whether to automatically close the message box after a few seconds
                autoCloseDelay: 10000,    // The amount of time (in milliseconds) before auto-closing the message box
                message: undefined,       // The message to be displayed
                details: undefined,       // (optional) A more detailed message
                fieldErrors: undefined    // (optional) An array of field-specific errors { field: '', message: '', details: '' }
            };

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            /// <var type="Message">
            /// Provides properties and methods for interacting with a MessageBox control.
            /// </var>
            var message = {
                isErrorModel: true,
                
                /// <field type="String">The type of message (e.g. 'Success', 'Failure', 'Info', or 'Warning')</field>
                type: options.type,

                /// <field type="String">The message to be displayed</field>
                message: options.message,

                /// <field type="String" mayBeNull="true">A more detailed message</field>
                details: options.details,

                /// <field type="Array" elementType="FieldError">An array of field-specific error messages</field>
                fieldErrors: options.fieldErrors,

                /// <field type="Boolean">Indicates whether the MessageBox is currently visible</field>
                visible: ko.observable(false),

                show: function () {
                    /// <summary>
                    /// Shows the MessageBox.  if the autoShow option was specified, then this method is called automatically.
                    /// </summary>

                    var $msgBox = getMessageBox(message, options);

                    // Apply the .messageBox CSS class
                    $msgBox.addClass('messageBox');

                    $msgBox.removeClass('failure success info warning');

                    $msgBox.addClass(message.type.toLowerCase());

                    // Show the messageBox (hide it first, then show it so the animation triggers)
                    $msgBox.removeClass('open').addClass('closed').hide()
                        .removeClass('closed').addClass('open').show();
                    message.visible(true);

                    // Scroll the messagebox into view
                    scrollIntoView($msgBox);

                    // Bind the close() function to the close button
                    var onclick = 'click.DellClarity.Controls.MessageBox.close';
                    $msgBox.find('.msgBoxClose').off(onclick).on(onclick, message.close);

                    // Auto-close messages after a few seconds
                    if (options.autoClose) {
                        window.setTimeout(message.close, options.autoCloseDelay);
                    }
                },

                close: function () {
                    /// <summary>
                    /// Closes the MessageBox.  if the autoClose option was specified, 
                    /// then this method is called automatically after autoCloseDelay milliseconds.
                    /// </summary>

                    try {
                        var $msgBox = getMessageBox(message, options);
                        $msgBox.removeClass('open').addClass('closed');
                        message.visible(false);
                    }
                    catch (x) {
                        // The messageBox has already been closed/destroyed
                    }
                }
            };

            // If an ID was specified, then find the MessageBox control and bind it to the message model object
            if (!isNullUndefinedOrEmpty(options.id)) {
                var $targetMsgBox = getMessageBox(message, options);
                if ($targetMsgBox.exists()) {
                    ko.cleanNode($targetMsgBox[0]);
                    ko.applyBindings(message, $targetMsgBox[0]);
                }
            }

            if (options.autoShow) {
                // Show the message box asynchronously, so knockout bindings can finish being applied
                window.setTimeout(message.show, 0);
            }

            //#region Legacy API (backward compatibility)

            if (DellClarity.Config.apiVersion < 2.5) {
                message.hasMessages = message.visible; // The hasMessages property is obsolete.  Use the visible property instead
            }

            //#endregion

            return message;
        },

        __ensureGlobalMessageBox: function () {
            /// <summary>
            /// This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!
            /// </summary>

            // Determine if there is a MessageBox control in the mainViewContainer
            var $mainViewContainer = $(DellClarity.__mainViewContainer);
            var $msgBox = $mainViewContainer.children('section[id]:not([id=""]).messageBox');
            if ($msgBox.exists()) {
                return $msgBox[0];
            }

            // There isn't a MessageBox control in the mainViewContainer, so create one
            return $(DellClarity.__mainViewContainer)
                .prepend('<section id="DellClarity_Controls_MessageBox_Global" role="alert" class="messageBox" data-bind="template: DellClarity.Controls.MessageBox.templateName"></section>')
                .children('#DellClarity_Controls_MessageBox_Global')[0];
        }
    };

    // Load the messagebox template
    DellClarity.Resources.loadTemplate(retVal.templateName);

    function getMessageBox(message, options) {
        /// <summary>
        /// Returns the <section role="alert"> element that represents the messagebox.
        /// </summary>

        var $msgBox;

        if (options.id) {
            // Find the messageBox by its ID
            $msgBox = $('#' + options.id);
        }
        else if (DellClarity.Config.apiVersion < 2.5) {
            // In older versions of DellClarity, we don't have the ID, so we have to find the message box
            $msgBox = getMessageBoxFromModel(message);
        }
        else {
            // We can't find the MessageBox, so just return an empty jQuery object
            $msgBox = $();
        }

        // If we didn't find a MessageBox, then log a warning, but continue without an error
        if (!$msgBox.exists()) {
            console.warn('Unable to find a MessageBox control with the ID "' + options.id + '"');
        }

        return $msgBox;
    }

    function scrollIntoView($msgBox) {
        /// <summary>
        /// Scrolls the messagebox into view.
        /// </summary>

        var $container;

        // Rather than scrolling the messagebox into view, we actually want
        // to scroll its container into view.  This is more visually appealling.
        if (DellClarity.Config.apiVersion >= 2.5) {
            // Find the <article> that contains the messagebox
            $container = $msgBox.parents('article').first();
        }
        else {
            // In older versions of DellClarity, there are no <article> elements,
            // so we just have to find the nearest offsetParent
            $container = $msgBox.offsetParent();
        }

        if ($container.exists())
            $container[0].scrollIntoView();
        else if ($msgBox.exists())
            $msgBox[0].scrollIntoView();
    }


    //#region Legacy API (backward compatibility)

    if (DellClarity.Config.apiVersion < 2.5) {
        // Prior to DellClarity 2.5, UI controls were in the root namespace rather than DellClarity.Controls
        DellClarity.MessageBox = {
            templateName: retVal.templateName,
            createSuccessMessage: function (message) {
                return DellClarity.Controls.MessageBox.createSuccessMessage(undefined, message);
            },
            createInfoMessage: function (message, type) {
                if (type === 'Success')
                    return DellClarity.Controls.MessageBox.createSuccessMessage(undefined, message);
                else
                    return DellClarity.Controls.MessageBox.createInfoMessage(undefined, message);
            },
            createWarningMessage: function (message) {
                return DellClarity.Controls.MessageBox.createWarningMessage(undefined, message);
            },
            createErrorMessage: function (message, fldErrors) {
                return DellClarity.Controls.MessageBox.createErrorMessage(undefined, message, fldErrors);
            },
            createModel: function (errorObj) {
                return DellClarity.Controls.MessageBox.createErrorMessage(undefined, errorObj);
            }
        };

        var getMessageBoxFromModel = function (model) {
            /// <summary>
            /// Finds the <see cref="DellClarity.MessageBox.template"/> instance that is bound to the specified model object.
            /// </summary>

            var msgBoxes = [];

            $('[data-bind*="MessageBox.templateName"]').each(function (index, element) {
                var msgBoxModel = ko.dataFor(element);
                var childElement = element.firstChild;

                // Determine if the messageBox is bound this model
                if (msgBoxModel === model) {
                    msgBoxes.push(element);
                }
                else if (childElement) {
                    // Determine if the messageBox's children are bound to this model
                    msgBoxModel = ko.dataFor(childElement);
                    if (msgBoxModel === model) {
                        msgBoxes.push(element);
                    }
                }
            });

            return $(msgBoxes);
        };
    }

    //#endregion

    return retVal;
})();
