/// <summary>
/// A modal dialog window that contains a given HTML template.
/// </summary>
DellClarity.Controls.Modal = (function () {
    var retVal = {
        create: function (options) {
            /// <summary>
            /// Creates a modal dialog window using the specified options.
            /// </summary>
            /// <param name="options" type="ModalOptions">
            /// Determines how the Modal control appears and behaves.
            /// </param>
            /// <returns type="Modal"/>

            /// <var type="ModalOptions">Options for creating a Modal control</var>
            var defaultOptions = {
                title: '',
                id: '',
                templateName: '',
                model: null,
                helpUrl: '',
                width: '865px',
                height: '450px',
                closeAction: null,
                allowClose: true,
                fullscreen: false,
                navigateOnClose: true
            };

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            var initialized = false;
            var historyDepth = 0;  // Used to restore the browser history to its previous state when the modal is closed

            /// <var type="Modal">
            /// Provides properties and methods for interacting with a Modal control
            /// </var>
            var retVal = {
                /// <field type="String">The title text of the modal dialog</field>
                title: ko.observable(options.title),

                /// <field type="Number" integer="true">The desired width of the dialog (if no unit is specified, then pixels is assumed)</field>
                width: ko.observable(options.width),

                /// <field type="Number" integer="true">The desired height of the dialog (if no unit is specified, then pixels is assumed)</field>
                height: ko.observable(options.height),

                /// <field type="Boolean">Indicates whether the dialog is in full screen mode</field>
                fullscreen: ko.observable(options.fullscreen),

                /// <field type="Function">A function that is invoked when the user clicks the minimize link in the modal</field>
                minimize: function () {
                    this.fullscreen(false);
                    DellClarity.__updateLayout();
                },

                /// <field type="Function">A function that is invoked when the user clicks the minimize link in the modal</field>
                maximize: function () {
                    this.fullscreen(true);
                    DellClarity.__updateLayout();
                },

                /// <field type="String">The URL of the help page for the dialog</field>
                helpUrl: ko.observable(options.helpUrl),

                /// <field type="Boolean">Indicates whether the dialog includes a close button</field>
                allowClose: ko.observable(options.allowClose),

                /// <field type="Object">The view (HTML) and model (JavaScript) for the dialog</field>
                contents: ko.observable({
                    /// <field type="Object">The model object to which the dialog is bound</field>
                    model: options.model,

                    /// <field type="String">The ID of the HTML template containing the dialog's contents</field>
                    templateName: options.templateName
                }),

                /// <field type="String">Deprecated. DO NOT USE</field>
                modalTemplateUrl: 'templates/modaltemplate.html',

                /// <field type="String">Deprecated. DO NOT USE</field>
                modalTemplateName: 'templates/modaltemplate.html',

                /// <field type="Boolean">Indicates whether the dialog is currently visible</field>
                visible: ko.observable(false),

                /// <field type="Function">A function that is invoked when the user clicks the Help link in the modal</field>
                help: function () {
                    if (this.helpUrl()) {
                        DellClarity.Help.showHelp(this.helpUrl());
                    }
                },

                /// <field type="Function">A function that returns a boolean value indicating whether the Help link is shown in the modal</field>
                isHelpVisible: function () {
                    if (this.helpUrl()) return true;
                    return false;
                },

                /// <field type="Function">Deprecated. DO NOT USE</field>
                isCloseVisible: function () {
                    if (this.allowClose()) return true;
                    return false;
                },

                /// <field type="jQuery">
                /// A jQuery selector that is scoped to the wizard.
                /// You can use wizard.$(selector) to find elements within the scope of this wizard.
                /// </field>
                $: function (selector) {
                    return $('#' + this.getModalId()).find(selector);
                },

                getElement: function () {
                    return document.getElementById(this.getModalId());
                },

                show: function (width, height) {
                    /// <summary>
                    /// Shows the modal dialog.
                    /// </summary>
                    /// <param name="width" type="Number" integer="true" mayBeNull="true" optional="true">
                    /// The desired width of the modal.  Leave null or undefined to use the default width.
                    /// </param>
                    /// <param name="height" type="Number" integer="true" mayBeNull="true" optional="true">
                    /// The desired height of the modal.  Leave null or undefined to use the default height.
                    /// </param>

                    if (!this.visible()) {
                        devLog('DellClarity.Controls.Modal.' + options.id + '.show()');
                        this.initialize();

                        if (width)
                            this.width(width);
                        if (height)
                            this.height(height);

                        this.visible(true);

                        DellClarity.Navigation._modalShown(this);
                    }
                },

                close: function (navigateTo) {
                    /// <summary>
                    /// Closes the modal dialog and invokes the closeAction function, if any.
                    /// </summary>

                    devLog('DellClarity.Controls.Modal.' + options.id + '.close()');
                    this.visible(false);

                    var element = $("#" + this.getModalId());

                    if (element.length !== 0) {
                        ko.cleanNode(element.get(0));

                        element.html('');

                        element.remove();
                    }

                    if (options.closeAction) {
                        options.closeAction();
                    }

                    if (options.navigateOnClose)
                        DellClarity.Navigation._modalClosed(element, navigateTo);

                    // unbind Keyup event
                    $("body").unbind("keyup.coreModal" + options.id);

                    // unbind Keydown event
                    $("body").unbind("keydown.coreModal" + options.id);
                },

                getModalId: function () {
                    /// <summary>
                    /// Returns the ID of the modal in the HTML DOM
                    /// </summary>

                    return options.id;
                },

                initialize: function () {
                    /// <summary>
                    /// This is an internal DellClarity API member. DO NOT CALL IT IN YOUR APPLICATION
                    /// </summary>

                    var self = this;

                    // bind Keyup event
                    $("body").bind("keyup.coreModal" + options.id, function (event) {
                        // 27 == "esc"
                        if (event.which === 27) {
                            //self.close();
                            event.preventDefault();
                            return false;
                        }
                    });

                    // bind Keydown event
                    $("body").bind("keydown.coreModal" + options.id, function (event) {

                        var code;
                        if (!event) var event = window.event; // some browsers don't pass e, so get it from the window
                        if (event.keyCode) code = event.keyCode; // some browsers use e.keyCode
                        else if (event.which) code = event.which;  // others use e.which

                        var inputfield = $(event.target).is("input:not([readonly]):not([type=radio]):not([type=checkbox]), textarea");
                        // 8 == "backspace"
                        if (code === 8 && !inputfield) {
                            return false;
                        }

                        // 112 == "F1", 123 == "F12", 82 == R to block Ctrl+R
                        if ((code >= 112 && code <= 123) || (code === 82 && !inputfield)) {
                            return false;
                        }
                    });

                    if (!initialized) {
                        initialized = true;
                        DellClarity.Resources.loadTemplate(this.modalTemplateName);
                    }

                    var existingElement = $("#" + this.getModalId());

                    if (existingElement.length === 0) {
                        var element = $('<div id="' + this.getModalId() + '" data-bind="template: {name: modalTemplateName, data: $data }" />');
                        $('body').append(element);
                        ko.cleanNode(element.get(0));
                        ko.applyBindings(this, element.get(0));
                    }
                },

                bindComplete: function () {
                    // DellClarity Framework stuff
                    this.$('.modalContentWrapper > section').scroll(DellClarity.__updateLayout);
                    DellClarity.__updateLayout();
                }
            };

            function effectiveSize(size, fullscreen) {
                // In fullscreen mode, we don't set the sizes
                if (fullscreen) {
                    return undefined;
                }

                // If the size is specified as a number, then assume pixels;
                // otherwise, let the developer specify the unit
                if (typeof size === 'number')
                    return size + 'px';
                else
                    return size;
            }

            /// <field type="Number" integer="true">The effective width of the dialog</field>
            retVal.effectiveWidth = ko.computed(function () { return effectiveSize(this.width(), this.fullscreen()); }, retVal);

            /// <field type="Number" integer="true">The effective height of the dialog</field>
            retVal.effectiveHeight = ko.computed(function () { return effectiveSize(this.height(), this.fullscreen()); }, retVal);

            return retVal;
        }
    };


    //#region Legacy API (backward compatibility)

    if (DellClarity.Config.apiVersion < 2.5) {
        // Prior to DellClarity 2.5, UI controls were in the root namespace rather than DellClarity.Controls
        DellClarity.Modal = retVal;
        DellClarity.Modal.create = retVal.create;

        retVal.CreateModal = function (modalId, templateName, model, helpLink, closeAction) {
            /// <summary>
            /// This method is deprecated. Please use the create() method instead.
            /// </summary>

            return DellClarity.Controls.Modal.create({
                id: modalId,
                templateName: templateName,
                model: model,
                helpUrl: helpLink,
                closeAction: closeAction
            });
        };
    }

    //#endregion

    return retVal;
})();
