/// <summary>
/// A multiSelectList control supports any <select multiple> tag with an ID and replaces it and its associated <option> tags
/// with a dropdown select list of checkboxes that the user can use to select multiple options
/// </summary>
/// <remarks>
/// NOTE: This control responds to users select to expand or collapse the dropdown list and adds the 'selected' attribute to the associate
///        <select> tag's <option>s array depending on the checked or unchecked state of each option in the dropdown list.        
/// </remarks>
DellClarity.Controls.MultiSelectList = (function () {

    var retVal = {
        create: function (options) {
            /// <signature>
            ///     <summary>
            ///     Creates a multiSelectList control using the specified HTML element,
            ///     accepting all of the default options.
            ///     &#10; The element syntax should be:
            ///     &#10; &lt;select multiple id="options" &gt;
            ///     </summary>
            ///     <param name="options" type="String">
            ///     The ID of the HTML element that will become the MultiSelectList control.
            ///     </param>
            /// </signature>
            /// <signature>
            ///     <summary>
            ///     Creates a MultiSelectList control using the specified options.
            ///     &#10; The element syntax should be:
            ///     &#10; &lt;select multiple id=options.id &gt;
            ///    </summary>
            ///     <param name="options" type="MultiSelectOptions">
            ///     Determines how the MultiSelectList control appears and behaves
            ///     </param>
            /// </signature>

            /// <var type="MultiSelectOptions">Options for creating a MultiSelectList control</var> 
            var defaultOptions = {
                id: '',                      // the ID of the <select multiple> element
                showSelectAll: false,        // Have an option in the list to Select All Elements?
                selectAllText: 'Select All', // the Select All option text
            };

            // If a string is passed, then it's just the ID option
            if (typeof options === 'string') {
                options = { id: options };
            }

            // Overwrite the default options with any that were explicitly specified
            options = $.extend(defaultOptions, options);

            // Get the root <select multiple> element and progressively enhance it
            $selectList = $('select#' + options.id + '[multiple]');

            if (!$selectList.exists()) {
                throw new Error('Unable to find a <select multiple> element with the ID "' + options.id + '"');
            }

            //Hide the <select> tag and insert the multipleSelectList
            enhanceSelectList(options, $selectList);

            //Attach the event handlers needed for the multiSelectControl
            attachEventHandlers(options, $selectList)
        },
    };

    //#region hideAndInsertMultipleList function
    function enhanceSelectList(options, $selectList) {
        ///<summary>
        ///Creates a new <div> that contains the MultiSelectList, moves the <select> behind the
        ///newly created <div>
        ///</summary>

        var multiSelectListId = getmultiSelectListId(options);
        var selectAllId = getSelectAllId(options);

        // insert a new object after the <select> has been hidden
        $selectList.after('<div id="' + multiSelectListId + '" class="multiSelectList"><span></span><ul class="labelValuePairs"></ul></div>');

        var $multiSelect = $selectList.next();
        var $list = $multiSelect.children('ul');

        //copy any classes from the <select> to the <div>
        $multiSelect.addClass($selectList.attr('class'));

        $selectList.addClass('multiSelectList_hiddenSelect');

        //check the options to see if the selectAll checkbox needs to be added
        if (options.showSelectAll)
            $list.append('<li><input id="' + selectAllId + '" class="value" type="checkbox" value=\'\' /> <label for="' + selectAllId +
                '"><u>' + options.selectAllText + '</u></label></li>');

        //copy options into <li>
        $selectList.children('option').each(function (index, element) {
            var $option = $(element);
            var isSelected = $option.prop('selected');

            var checkboxId = multiSelectListId + '_' + index;

            $list.append('<li><input id="' + checkboxId + '" class="value" value="' + element.value +
            '" type="checkbox"/><label for="' + checkboxId + '"><u>' + $option.text() + '</u></label></li>');

            if (isSelected) {
                $option.attr('selected', 'selected')
                $('#'+checkboxId).prop('checked', true);
                $('#'+checkboxId).attr('checked', 'checked');
            }

        });
    }
    //#endregion

    function attachEventHandlers(options, $selectList) {
        /// <summary>
        /// Attach all the event handlers needed for the multiSelect Control
        /// </summary>

        //the multiselect <div> is always the immediate sibling to the <select> tag
        var $multiSelect = $selectList.next();

        $(document).off('click.DellClarity.Controls.MultiSelect').on('click.DellClarity.Controls.MultiSelect', function (evt) {
            documentClickEvent(evt);
        });
        
        var selectAllId = '#' + getSelectAllId(options);
        if (options.showSelectAll) {           
            $(selectAllId).change(function (evt) {
                selectAllChecked(evt, $multiSelect, options);
            });
        }

        //detects changes to the <ul>'s checkboxes
        $multiSelect.on("change", ':checkbox', function (evt) {
            checkboxChange(evt, $multiSelect, options);

            //the knockout framework does not detect programmatic changes to DOM elements and thus will not 
            //update any changes made to the checkbox/select list, these need to be manually fired, thus the 
            //call to .trigger
            $selectList.trigger('change');
        });

        //make sure the span updates at initialization in case the user is loading from a saved object
        updateSpan($multiSelect, options);
    }

    //#region multiSelectDocClick Event

    
    function documentClickEvent(evt) {
        ///<summary>
        ///Event Handler that changes the multiSelectLists expand and contract behavior depending on the location of the document click
        ///</summary
        //If the target is a span and the parent has a .multiSelect class then toggle the .expanded class
        if ($(evt.target).is('span') && $(evt.target).parent('.multiSelectList').exists()) {
            $(evt.target).parent('.multiSelectList').toggleClass('expanded');

            //Currently we only want one multiSelectList to be open at any given time,
            //this block finds all the multiSelectLists and that are not the currently targeted one and removes
            //the expanded class that might be applied to them
            var targetId = $(evt.target).parent('.multiSelectList').attr('id');
            $('.multiSelectList').not('#' + targetId).removeClass('expanded');
        }

            //if the parent of the event target is expanded then do nothing because
            //this is where the user is clicking around inside of an already expanded list and we don't 
            //want to collapse the list unexpectedly on the user
        else if ($(evt.target).parents('.expanded').exists()) {
            //do nothing
        }

            //if a click event is registered outside of a multiselect list then close all the select lists
        else
            $('.multiSelectList').removeClass('expanded');
    }

    //#endregion

    //#region selectAll Checked Event
    function selectAllChecked(evt, $multiSelect, options)
    {
        ///<summary>
        /// If the Select All option is present and is clicked then all the <option> tags in the <select> 
        /// have the selected option added or removed
        ///</summary>
        var self = evt.target;
        var selectAllId = '#' + getSelectAllId(options);
        $multiSelect.find(':checkbox:not(' + selectAllId + ')').each(function (index, element) {

            //set each checkbox element to whatever the selectAll element is set to
            element.checked = self.checked;

            //the $multiSelect.on("change"... below will handle updating the span with the appropriate text of selected options,
            //however the<select> tag and its <option>s still need to be updated with the appropriate 'selected' attribute
            syncWithSelect(element, $multiSelect, options);

        });
    }
    //#endregion

    //#region checkbox Change Event
    function checkboxChange(evt, $multiSelect, options) {
        ///<summary>
        /// If a checkbox is clicked then the associated <option> within the <select> has the 'selected' attribute
        /// added or removed
        ///</summary
        syncWithSelect(evt.target, $multiSelect, options);
        updateSpan($multiSelect, options);

    }
    //#endregion

    function updateSpan($multiSelect, options) {
        //Add the checked items to the <span> as text
        var $span = $multiSelect.children('span');
        var span_text = '';
        var selectAllId = '#' + getSelectAllId(options);

        $multiSelect.find(':checked:not(' + selectAllId + ') +label').each(function (index, element) {
            //if the span is empty don't put ',' at the beginning of the element value
            if (span_text === '')
                span_text += $(element).text();
            else
                span_text += (', ' + $(element).text());
        });

        $span.text(span_text);
    }

    //#region syncWithSelect
    function syncWithSelect(checkbox, $multiSelect, options) {
        ///<summary>
        ///Sync the <select>'s <option> tags with the multiSelect <div>'s checkboxes
        ///</summary>
        //get the index of that list item
        var $listItem = $(checkbox).parent('li');
        var index = $multiSelect.find('ul > li').index($listItem);

        //check to see if the options.showSelectAll option is true because that means an extra <li> 
        //is present that isn't in the <select>'s <option>s.
        //If true -1 from the index, else leave as is
        if (options.showSelectAll)
            index--;

        //Get the <option> from the <select>
        var $option_tag = $('#' + options.id + '>option').eq(index);
        
        //set the option properties and attributes
        $option_tag.prop('selected', checkbox.checked);
        if (checkbox.checked)
            $option_tag.attr('selected', 'selected');
        else
            $option_tag.removeAttr('selected');
            
    }
    //#endregion

    function getmultiSelectListId(options) {
        ///<summary>
        ///Returns the Multi Select List's <div> id
        ///</summary>
        var multiSelectListId = options.id + '_multiSelectList';

        return multiSelectListId;
    }

    function getSelectAllId(options) {
        ///<summary>
        /// Returns the id of the 'Select All' checkbox
        ///</summary>
        var selectAllId = options.id + '_selectAll';

        return selectAllId;
    }

    return retVal;
})();
