/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    DellClarity.Tasks = {};

    //
    // Represents the number of items in the context to which the task
    // can be applied.
    //
    DellClarity.Tasks.TaskAppliesTo = {
        NONE: 0,
        ONE: 1,
        MANY: 2,
        ALWAYS: 3
    };

    //
    // The importance of the task indicates how will it be rendered.
    // -Main: button
    // -Additional: dropdown list item
    // -External: link
    //
    DellClarity.Tasks.TaskImportance = {
        //Main task, represented as a button
        MAIN: 0,

        //Additional task, represented as an option in a dropdown list.
        ADDITIONAL: 1,

        //External task, does not apply to a specific item in the context and 
        //navigates away from the current page or opens a new window
        EXTERNAL: 2,

        CONTEXT: 3
    };

    //
    // A task that the user can launch. Can apply to one or
    // more items within a context.
    //
    DellClarity.Tasks.Task = Class.extend({
        init: function (taskData, gridId) {
            this._mtgId = gridId;
            this.id = taskData.id;
            this.name = taskData.name;
            this.appliesTo = taskData.appliesTo;
            this.importance = taskData.importance;
            this.handler = taskData.handler;
            this.disabled = taskData.disabled;
            this.styleClass = taskData.styleClass;
        },

        getTargetElement: function () {
            return null;
        },

        disable: function () {
        },

        enable: function () {
        },

        applyBehavior: function () {
        },

        checkStatus: function (selectedRows) {
            switch (this.appliesTo) {
                case DellClarity.Tasks.TaskAppliesTo.ALWAYS:
                    {
                        this.enable();
                        break;
                    }
                case DellClarity.Tasks.TaskAppliesTo.NONE:
                    {
                        if (selectedRows.length == 0)
                            this.enable();
                        else
                            this.disable();
                        break;
                    }
                case DellClarity.Tasks.TaskAppliesTo.ONE:
                    {
                        if (selectedRows.length == 1)
                            this.enable();
                        else
                            this.disable();
                        break;
                    }
                case DellClarity.Tasks.TaskAppliesTo.MANY:
                    {
                        if (selectedRows.length >= 1)
                            this.enable();
                        else
                            this.disable();
                        break;
                    }
            }
        },

        getHTML: function () {
            return '';
        }
    });


    //
    // An important task
    //
    DellClarity.Tasks.ImportantTask = DellClarity.Tasks.Task.extend({
        init: function (taskData, gridId) {
            this._super(taskData, gridId);
            this.elementId = (new DellClarity.StringTemplate('taskBarImportantTask#{id}_#{taskId}')).evaluate({ 'id': this._mtgId, 'taskId': this.id });
        },

        disable: function () {
            this.getTargetElement().addClass("disabled");
            this.disabled = true;
        },

        enable: function () {
            this.getTargetElement().removeClass("disabled");
            this.disabled = false;
        },

        getTargetElement: function () {
            return $(this.elementId).get(0);
        },

        applyBehavior: function () {
            var self = this;
            this.getTargetElement().click(function () {
                if (!self.disabled && self.handler)
                    self.handler();
            });
        },

        getHTML: function () {
            return (new DellClarity.StringTemplate('<div id="taskBarImportantTask#{id}_#{taskId}" class="button disabled #{buttonTypeClass}"><div class="buttonInner">#{taskName}</div></div>')).evaluate({
                'id': this._mtgId,
                'taskId': this.id,
                'taskName': this.name,
                'buttonTypeClass': this.styleClass
            });
        }
    });

    //
    // An additional task, rendered as an item in a dropdown list.
    //
    DellClarity.Tasks.AdditionalTask = DellClarity.Tasks.Task.extend({
        init: function (taskData, gridId) {
            this._super(taskData, gridId);
            this.elementSelector = (new DellClarity.StringTemplate("taskBarSecondaryTasks#{id}_update UL LI[id='#{taskId}']")).evaluate({ 'id': this._mtgId, 'taskId': this.id });
        },

        disable: function () {
            this.disabled = true;
        },

        enable: function () {
            this.disabled = false;
        },


        getTargetElement: function () {
            var foundElements = $(this.elementSelector);
            if (foundElements.length > 0)
                return foundElements[0];
            else
                return null;
        }
    });


    //
    // An external task, that is performed opening a new window or navigating away to another page.
    //
    DellClarity.Tasks.ExternalTask = DellClarity.Tasks.Task.extend({
        init: function (taskData, gridId) {
            this._super(taskData, gridId);
            this.elementId = (new DellClarity.StringTemplate('taskBarExternalTask#{id}_#{taskId}')).evaluate({ 'id': this._mtgId, 'taskId': this.id });
        },

        disable: function () {
            this.getTargetElement().addClass("disabled");
            this.disabled = true;
        },

        enable: function () {
            this.getTargetElement().removeClass("disabled");
            this.disabled = false;
        },

        getTargetElement: function () {
            return $(this.elementId).get(0);
        },

        applyBehavior: function () {
            var self = this;
            this.getTargetElement().click(function () {
                if (!self.disabled && self.handler)
                    self.handler();
            });
        },

        getHTML: function () {
            return (new DellClarity.StringTemplate('<a id="taskBarExternalTask#{id}_#{taskId}" class="link active" href="javascript:void(0);">#{taskName}</a>')).evaluate({
                'id': this._mtgId,
                'taskId': this.id,
                'taskName': this.name
            });
        }
    });


    //
    // A container for tasks
    //
    //DellClarity.Tasks.TaskBar = Class.extend({
    (function ($) {

        var methods = {
            init: function (options) {
                this.id = options.id;
                this.tasks = options.tasks;
                this.taskList = [];
                this.itemsContainer = options.itemsContainer;
            },

            create: function () {
                var taskBarTemplate = new DellClarity.StringTemplate('<div id="taskBar#{id}" class="my-tablegrid-taskbar dellClarityTaskBar">' +
                                                    '   <span id="taskBarTitle#{id}" class="label">#{title}:&nbsp;&nbsp;</span>' +
                                                    '   &nbsp;#{importantTasks}&nbsp;&nbsp;' +
                                                    '   <input id="taskBarSecondaryTasks#{id}" name="taskBarSecondaryTasks#{id}" type="text" class="additionalTasks" style="display: none" />' +
                                                    '   &nbsp;#{externalTasks}&nbsp;' +
                                                    '</div>');

                var dividerTemplate = '<div class="divider"></div>';

                var importantTasks = [];
                var externalTasks = [];

                jQuery.each(this.tasks.items, (function (index, item) {

                    var task;

                    switch (item.importance) {
                        case DellClarity.Tasks.TaskImportance.MAIN:
                            {
                                task = new DellClarity.Tasks.ImportantTask(item, this.id);

                                if (importantTasks.length > 0)
                                    importantTasks.push(dividerTemplate);

                                importantTasks.push(task.getHTML());

                                break;
                            }
                        case DellClarity.Tasks.TaskImportance.ADDITIONAL:
                            {
                                task = new DellClarity.Tasks.AdditionalTask(item, this.id);
                                break;
                            }
                        case DellClarity.Tasks.TaskImportance.EXTERNAL:
                            {
                                task = new DellClarity.Tasks.ExternalTask(item, this.id);

                                externalTasks.push(dividerTemplate);
                                externalTasks.push(task.getHTML());

                                break;
                            }
                    }

                    this.taskList.push(task);

                }).bind(this));

                return taskBarTemplate.evaluate({ 'id': this.id, 'title': this.tasks.title, 'importantTasks': importantTasks.join(''), 'externalTasks': externalTasks.join('') });
            },

            load: function () {

                //Additional tasks
                var additionalTasks = [];

                jQuery.each(this.taskList, (function (index, task) {

                    task.checkStatus([]);

                    task.applyBehavior();

                    switch (task.importance) {
                        case DellClarity.Tasks.TaskImportance.ADDITIONAL:
                            {
                                additionalTasks.push(task);
                                break;
                            }
                    }
                }).bind(this));

                if (additionalTasks.length > 0) {

                    var comboTargetElementId = 'taskBarSecondaryTasks' + this.id;
                    $(comboTargetElementId).show();

                    new DellClarity.ComboBox({
                        input: comboTargetElementId,
                        initialText: 'More Tasks',
                        items: additionalTasks,
                        listTextPropertyName: 'name',
                        listValuePropertyName: 'id',
                        listStatusPropertyName: 'disabled',
                        afterUpdate: (function (sender, selectedItem) {
                            var taskId = selectedItem.id;
                            var task = jQuery.first(this.taskList, function (task) {
                                return task.id == taskId;
                            });
                            if (task && !task.disabled && task.handler)
                                task.handler();
                        }).bind(this)
                    });
                }

                //Subscribe to row selection event
                this.itemsContainer.selectedItemsChanged = this.onSelectedItemsChanged.bind(this);
            },

            onSelectedItemsChanged: function (event) {
                var selectedItems = (event) ? event.memo.selectedItems : [];
                this.checkTasksStatus(selectedItems);
            },

            checkTasksStatus: function (selectedItems) {
                if (!selectedItems) selectedItems = [];
                jQuery.each(this.taskList, function (index, task) {
                    task.checkStatus(selectedItems);
                });
            }
        };

        $.fn.TaskBar = function (method) {

            if (methods[method]) {
                return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
            } else if (typeof method === 'object' || !method) {
                return methods.init.apply(this, arguments);
            } else {
                $.error('Method ' + method + ' does not exist on jQuery.tooltip');
            }

        };

    })(jQuery);

    //});
}
