/******************************************************************************
 * This script loads the DellClarity framework and all of its dependencies.
 *
 * NOTE: The ONLY scripts you need to include in your .html pages are Modernizer.js 
 *       and this file (Global.js). All other scripts will be automatically loaded 
 *       by the DellClarity framework.
 *
 * In your .html page, before referencing this global.js script, you can define a
 * DellClarity object, with a DellClarity.Config property that overrides any of the 
 * DellClarity.Config defaults below.  You can also define a DellClarity.initialize 
 * function that will automatically be called when DellClarity has finished loading 
 * and your custom application logic can safely begin.
 ******************************************************************************/

// Make sure this browser is supported by DellClarity.
// Note: We use feature detection rather than browser sniffing.  It's more reliable and versatile.
if (!document.querySelector || document.querySelector('html[class*="unsupportedBrowser"]')) {
    var message = 'You are using an unsupported or outdated web browser.\n Please upgrade your web browser or use a supported browser.';
    alert(message);
    throw new Error(message);
}

// Define the DellClarity global variable, if it's not already defined
if (!DellClarity) {
    /// <var type="Object">
    /// The DellClarity global object acts as the root namespace for all objects, functions, 
    /// controls, and namespaces in the DellClarity framework.
    /// </var>
    var DellClarity = { Config: {} };
}

DellClarity = {
    /// <field type="Object" static="true">
    /// These configuration options control all aspects of DellClarity, including behavior, appearance, and backward-compatibility.
    /// </field>
    Config: {
        /// <field type="Number" static="true">
        /// The version of the DellClarity API that your application is using.
        /// &#10;&#10;
        /// Setting this option to 2.5 will allow your application to use the latest API features,
        /// but will also break any existing code that relies on deprecated or obsolete features.
        /// </field>
        apiVersion: DellClarity.Config.apiVersion || 1.0,

        /// <field type="String" static="true">
        /// The virtual application path (e.g. "/SkyHawk")
        /// &#10;&#10;
        /// For top-level sites (e.g. http://MyApp.dell.com/), this should be an empty string.
        /// For virtual app sites (e.g. http://dell.com/MyApp/), this should be the virtual app path (e.g. '/MyApp/').
        /// </field>
        virtualAppRoot: DellClarity.Config.virtualAppRoot || '',

        /// <field type="Boolean" static="true">
        /// Indicates whether debug versions of DellClarity script files should be loaded.
        /// Debug versions are useful for development or debugging.
        /// In production, leave this property false to load optimized (minified and combined) scripts instead.
        /// </field>
        debugMode: DellClarity.Config.debugMode === undefined ? true : DellClarity.Config.debugMode,

        /// <field type="Boolean" static="true">
        /// Indicates whether the application is being run in a development environment.
        /// This enables helpful development features, such as markup validation, enhanced console logging, etc.
        /// </field>
        developmentMode: DellClarity.Config.developmentMode === undefined ? true : DellClarity.Config.developmentMode,

        /// <field type="Boolean" static="true">
        /// Indicates whether the application should parse and display fieldErrors collections
        /// </field>
        ignoreFieldErrors: DellClarity.Config.ignoreFieldErrors === undefined ? false : DellClarity.Config.ignoreFieldErrors,


        /// <field type="Number" static="true">
        /// Indicates the number of minutes that the application waits idle before timing out.
        /// If no timeout is set, the value is null.
        /// </field>
        sessionTimeout: DellClarity.Config.sessionTimeout === undefined ? null : DellClarity.Config.sessionTimeout
    },

    /// <field type="Function" static="true" mayBeNull="true">
    /// If set by an application, this function will be called as soon as DellClarity has finished loading
    /// and custom application logic can safely begin.
    /// </field>
    initialize: DellClarity.initialize || undefined,

    /// <field type="Object" static="true">
    /// This namespace contains all DellClarity UI controls
    /// (such as modal dialogs, menus, grids, etc.)
    /// </field>
    Controls: {},

    /// <field type="String" static="true">
    /// The culture locale to use for localization.
    /// </field>
    CurrentLocale: 'en-US',

    /// <field type="Object" static="true">
    /// Global DellClarity enumerations/constants.
    /// </field>
    Constants: {
        LargeIcons: {
            check: 'icon check_48',
            critical: 'cux-icon cux-icon-remove-sign failure-color',
            info: 'cux-icon cux-icon-info-sign info-color',
            nonCritical: 'cux-icon cux-icon-warning-sign warning-color',
            success: 'cux-icon cux-icon-check-sign success-color',
            unknown: 'icon statusUnknown_48'
        },
        DataSortingConstants: {
            //"order" : number // the order to do sort, it must be 0, 1, 2. 0 means no sort, 1 means ascending order and 2 means descending order
            ASCENDING: 1,
            DESCENDING: 2
        },
        WindowStates: {
            Minimized: "minimized",
            Normal: "normal",
            Maximized: "maximized"
        }
    },

    /// <field type="Object" static="true">
    /// This object provides helper functions for loading resources (CSS, JavaScript, images, etc.)
    /// </field>
    Resources: (function () {
        // Keeps track of how many load operations are pending
        var pendingOperations = 0;

        return {
            load: function (urls, completeCallback) {
                /// <signature>
                ///   <summary>
                ///   Loads the specified resource, unless it has already been loaded.
                ///   &#10;&#10;
                ///   NOTE: It's important that developers use this abstraction function rather than using Modernizr.load directly.
                ///         In the future, it is possible that the DellClarity framework will use a different resource loader than Modernizr
                ///   </summary>
                ///   <param name="urls" type="String" mayBeNull="true">
                ///   The absolute or relative URL of the resource (JavaScript or CSS) to load
                ///   </param>
                ///   <param name="completeCallback" type="Function" mayBeNull="true" optional="true">
                ///   A callback function that will be invoked after the resource have been loaded.
                ///   </param>
                /// </signature>
                /// <signature>
                ///   <summary>
                ///   Loads the specified resource(s), in the specified order.
                ///   If a resource has already been loaded, then it is NOT loaded again.
                ///   &#10;&#10;
                ///   NOTE: It's important that developers use this abstraction function rather than using Modernizr.load directly.
                ///         In the future, it is possible that the DellClarity framework will use a different resource loader than Modernizr
                ///   </summary>
                ///   <param name="urls" type="Array" mayBeNull="true" elementType="String">
                ///   An array of the absolute or relative URLs of the resources (JavaScript or CSS) to load
                ///   </param>
                ///   <param name="completeCallback" type="Function" mayBeNull="true" optional="true">
                ///   A callback function that will be invoked after all of the resources have been loaded.
                ///   </param>
                /// </signature>

                // If no URLs are specified, then just call the callback
                if (urls && urls.length > 0) {
                    // Increment the pending-operations counter
                    pendingOperations++;

                    Modernizr.load({
                        load: urls,
                        complete: function () {
                            // Decreement the pending-operations counter
                            pendingOperations--;

                            // Call the callback function (if any)
                            if (completeCallback)
                                completeCallback()
                        }
                    });
                }
                else {
                    // NOTE: no need to increment/decrement the pending-operations counter, since there's nothing to load
                    if (completeCallback)
                        completeCallback();
                }
            },

            loadTemplate: function (templateUrl) {
                /// <summary>
                /// Synchronously loads the specified HTML template, unless it has already been loaded.
                /// </summary>
                /// <param name="url" type="String" mayBeNull="false">
                /// The absolute or relative URL of the HTML template
                /// </param>

                // Use the URL as the unique ID for the template
                var templateId = templateUrl;

                if (DellClarity.Config.apiVersion < 2.5 && arguments.length > 1) {
                    // DellClarity 1.0 allowed custom template IDs
                    templateId = arguments[0];
                    templateUrl = arguments[1];
                }

                if (isNullUndefinedOrEmpty(templateUrl)) {
                    throw new Error('loadTemplate was called with an invalid URL (' + templateUrl + ')');
                }

                // Don't load the template if it already exists
                if (!document.getElementById(templateId)) {
                    // Increment the pending-operations counter
                    pendingOperations++;

                    try {
                        var url = DellClarity.__resolveUrl(templateUrl);
                        DellClarity.Ajax.GetHtmlSychronously(url, function (htmlContents) {
                            if (isNullUndefinedOrEmpty(htmlContents))
                                throw new Error('Unable to load the HTML template "' + url + '".  The HTTP response was empty.');

                            var theScriptTemplate = "<script id='" + templateId + "' type='text/html' >" + htmlContents + "</script>";
                            var body = $("body");
                            body.append(theScriptTemplate);
                        });
                    }
                    finally {
                        // Decreement the pending-operations counter
                        pendingOperations--;
                    }
                }

                // DellClarity 1.0 required the view to be resized after loading a template
                if (DellClarity.Config.apiVersion < 2.5) {
                    DellClarity.Helpers.resize();
                }
            },

            whenFinishedLoading: function (completeCallback) {
                /// <summary>
                /// Waits for all pending resources to finish loading, and then calls the specified callback function.
                /// </summary>
                /// <param name="completeCallback" type="Function">
                /// The callback function that will be invoked after all pending resources have finished loading.
                /// </param>

                // Wait until all pending operations complete before executing the callback.
                // NOTE: We *always* wait at least once, even if the pendingOperations counter is already zero,
                //       this allows code on other "threads" a chance to queue up any resource requests.
                var intervalId = setInterval(function () {
                    if (pendingOperations === 0) {
                        clearInterval(intervalId);
                        completeCallback();
                    }
                }, 200);
            }
        }
    })(),

    //#region Private DellClarity members. DO NOT USE THESE IN YOUR APPLICATION!

    /// <field>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</field>
    __resolveUrl: function (urlWithoutAppRoot) {
        return DellClarity.Config.virtualAppRoot + (urlWithoutAppRoot || '');
    },

    /// <field>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</field>
    __mainViewContainer: null, /* this property gets set after the DOM is loaded */

    /// <field>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</field>
    __domChange: function () {
        // This function is called whenever elements are added/removed from the DOM

        // Apply DellClarity polyfills to any new elements
        DellClarity.__applyPolyfills();

        // Update the page layout
        DellClarity.__updateLayout();
    },

    /// <field>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</field>
    __updateLayout: function () {
        // Position all .sectionEditButtons at the bottom of their offsetParent
        $('.sectionEditButtons').each(function () {
            var $buttons = $(this);
            var $parent = $buttons.offsetParent();

            // Calculate the new top position of the button bar
            var top = ($parent.innerHeight() - $buttons.outerHeight()) + $parent.scrollTop();

            // Apply the new positioning
            $buttons.css({
                position: 'absolute',
                right: '0',
                top: (top - 2) + 'px',
                width: $parent.innerWidth()
            });

            // Adjust the height of the previous sibling. Had to do this percentual since absolute numbers do not work.
            var previous = $buttons.prev();
            var previousNewHeight = previous.outerHeight() - $buttons.outerHeight();
            var previousNewPercentHeight = Math.round((previousNewHeight * 100) / previous.outerHeight());
            if (previousNewPercentHeight > 0 && previousNewPercentHeight <= 100)
                previous.height(previousNewPercentHeight + '%');
        });
    },

    /// <field>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</field>
    __error: (function () {
        // Route any uncaught script errors through DellClarity.__error()
        var existingHandler = window.onerror;
        window.onerror = function (message, url, lineNumber) {
            // Call any existing window.onerror handler
            if (existingHandler) existingHandler();

            // Convert window.onerror params to an ErrorObj
            DellClarity.__error({
                errorMessage: message,
                url: url,
                lineNumber: lineNumber,
                fldErrors: [
                    { field: 'URL', errorMessage: url },
                    { field: 'Line Number', errorMessage: lineNumber }
                ]
            }, true);

            // In development mode, allow the browser's error prompt to show.
            // In runtime mode, prevent the browser's error prompt from showing.
            return !DellClarity.Config.developmentMode;
        };

        // The actual DellClarity.error function
        return function (ex, alreadyLogged) {
            try {
                // In development mode, also log the error to the console (unless it has already been logged)
                if (DellClarity.Config.developmentMode && !alreadyLogged) {
                    console.error(ex);
                }

                // Show the error in the global MessageBox
                if (DellClarity.Config.apiVersion >= 2.5) {
                    // Create the MessageBox if necessary (and possible), then show the message
                    var msgBox = DellClarity.Controls.MessageBox.__ensureGlobalMessageBox();
                    if (msgBox) {
                        DellClarity.Controls.MessageBox.createErrorMessage(msgBox.id, ex)
                    }
                }
                else {
                    // In older versions of DellClarity, the global MessageBox is bound to a global observable
                    var message = DellClarity.Controls.MessageBox.createErrorMessage(ex);
                    DellClarity.ModelBinding.mainModelErrors(message);
                }
            }
            catch (ex2) { /* suppress */ }
        }
    })()

    //#endregion
};

// Function enclosure to control variable scope
(function () {
    //#region Check for legacy config settings

    if (DellClarity.Config.apiVersion < 2.5) {
        if (typeof VIRTUAL_APP_ROOT != 'undefined')
            DellClarity.Config.virtualAppRoot = VIRTUAL_APP_ROOT;
        if (((typeof debugMode != 'undefined') && (debugMode === true))
            || (typeof libraries != 'undefined' && libraries.indexOf("debug") >= 0))
            DellClarity.Config.debugMode = true;
    }

    //#endregion

    // Load dependencies and core resources
    if (!DellClarity.Config.debugMode) {
        // Load optimized (minified, combined) scripts
        var coreScripts = [
            'dellclarity.core/scripts/dellclarity.core.min.js',
            'dellclarity.core/localization/coretranslations.js'
        ];
    }
    else if (DellClarity.Config.apiVersion >= 2.5) {
        // DellClarity 2.5 scripts
        var coreScripts = [
            // Libraries that DellClarity depends upon
            //'dellclarity.core/scripts/libraries/knockout-2.2.1.debug.js',
            //'dellclarity.core/scripts/libraries/knockout.mapping-2.3.5.debug.js',
            'dellclarity.core/scripts/libraries/jquery-2.2.4.min.js',
            'dellclarity.core/scripts/libraries/knockout-3.4.0.js',
            'dellclarity.core/scripts/libraries/knockout.mapping-2.4.1.min.js',

            'dellclarity.core/scripts/libraries/moment.js',

            // Need to examine these scripts to see if we really need them, or if there are better alternatives
            'dellclarity.core/scripts/libraries/globalize.js',
            'dellclarity.core/scripts/libraries/globalize.cultures.js',

            // Charting components
            'dellclarity.core/scripts/libraries/highcharts.js',
            'dellclarity.core/scripts/libraries/highcharts-more.js',

             // JQGrid Scripts
           'dellclarity.core/scripts/libraries/jquery.jqgrid.js',
           'dellclarity.core/scripts/libraries/i18n/grid.locale-en.js',
           'dellclarity.core/scripts/controls/jqgridtable.js',


            // DellClarity core scripts
            'dellclarity.core/scripts/includes/javascript.polyfills.js',
            'dellclarity.core/scripts/includes/javascript.extensions.js',
            'dellclarity.core/scripts/includes/ko.extensions.js',
            'dellclarity.core/scripts/includes/deprecated.js',
            'dellclarity.core/scripts/includes/oop.js',
            'dellclarity.core/scripts/includes/helpers.js',
            'dellclarity.core/scripts/includes/ajax.js',
            'dellclarity.core/scripts/includes/localization.js',
            'dellclarity.core/localization/coretranslations.js',
            'dellclarity.core/scripts/includes/help.js',
            'dellclarity.core/scripts/includes/session.js',
            'dellclarity.core/scripts/includes/navigation.js',
            'dellclarity.core/scripts/includes/re_weburl.js',

            // DellClarity controls
            'dellclarity.core/scripts/controls/advancedwizard.js',
            'dellclarity.core/scripts/controls/login.js',
            'dellclarity.core/scripts/controls/modal.js',
            'dellclarity.core/scripts/controls/confirmationModal.js',
            'dellclarity.core/scripts/controls/messagebox.js',
            'dellclarity.core/scripts/controls/menu.js',
            'dellclarity.core/scripts/controls/masterDetails.js',
            'dellclarity.core/scripts/controls/multiSelectList.js',
            'dellclarity.core/scripts/controls/multiselectview.js',
            'dellclarity.core/scripts/controls/jquery.fixedheadertable.min.js',
            'dellclarity.core/scripts/controls/scrollabletable.js',
            'dellclarity.core/scripts/controls/statusIndicatorModal.js',
            'dellclarity.core/scripts/controls/tooltip.js',
            'dellclarity.core/scripts/controls/progress-button.js'
        ];
    }
    else {
        // DellClarity 1.0 scripts
        var coreScripts = [
            // Libraries that DellClarity depends upon
            'dellclarity.core/scripts/libraries/jquery-1.8.2.js',  // Some of the jQuery plug-ins that are used by DellClarity 1.0 apps are incompatible with jQuery 1.9
            'dellclarity.core/scripts/libraries/knockout-2.2.1.debug.js',
            'dellclarity.core/scripts/libraries/knockout.mapping-2.3.5.debug.js',
            'dellclarity.core/scripts/libraries/moment.js',


            // Need to examine these scripts to see if we really need them, or if there are better alternatives
            'dellclarity.core/scripts/libraries/globalize.js',
            'dellclarity.core/scripts/libraries/globalize.cultures.js',

            // Charting components
            'dellclarity.core/scripts/libraries/highcharts.js',
            'dellclarity.core/scripts/libraries/highcharts-more.js',

            // These scripts are still needed FOR NOW, but are being phased out in favor of HTML 5 polyfills
            'dellclarity.core/scripts/libraries/knockout.validation.js',
            'dellclarity.core/scripts/libraries/jquery-ui-1.8.18.custom.min.js',

            // JQGrid Scripts
            'dellclarity.core/scripts/libraries/jquery.jqgrid.js',
            'dellclarity.core/scripts/libraries/i18n/grid.locale-en.js',
             'dellclarity.core/scripts/controls/jqgridtable.js',

            // DellClarity core scripts
            'dellclarity.core/scripts/includes/javascript.extensions.js',
            'dellclarity.core/scripts/includes/ko.extensions.js',
            'dellclarity.core/scripts/includes/deprecated.js',
            'dellclarity.core/scripts/includes/oop.js',
            'dellclarity.core/scripts/includes/helpers.js',
            'dellclarity.core/scripts/includes/ajax.js',
            'dellclarity.core/scripts/includes/localization.js',
            'dellclarity.core/localization/coretranslations.js',
            'dellclarity.core/scripts/includes/help.js',
            'dellclarity.core/scripts/includes/session.js',
            'dellclarity.core/scripts/includes/navigation.js',

            // DellClarity controls
            'dellclarity.core/scripts/controls/advancedwizard.js',
            'dellclarity.core/scripts/controls/login.js',
            'dellclarity.core/scripts/controls/modal.js',
            'dellclarity.core/scripts/controls/confirmationModal.js',
            'dellclarity.core/scripts/controls/messagebox.js',
            'dellclarity.core/scripts/controls/menu.js',
            'dellclarity.core/scripts/controls/masterDetails.js',
            'dellclarity.core/scripts/controls/multiSelectList.js',
            'dellclarity.core/scripts/controls/multiselectview.js',
            'dellclarity.core/scripts/controls/tooltips.js',
            // jqgrid deprecated
            //'dellclarity.core/scripts/libraries/jquery.jqgrid.src.custom.js',
            //'dellclarity.core/scripts/controls/grid.js',
            'dellclarity.core/scripts/controls/wizard.js',
            'dellclarity.core/scripts/controls/datetimepicker.js',
            'dellclarity.core/scripts/controls/treecontrol.js',
            'dellclarity.core/scripts/controls/quickmenu.js',
            'dellclarity.core/scripts/controls/healthindicator.js',
            'dellclarity.core/scripts/controls/jquery.fixedheadertable.min.js',
            'dellclarity.core/scripts/controls/scrollabletable.js',
            'dellclarity.core/scripts/controls/progress-button.js',
           

            // Not sure if we even need these anymore
            'dellclarity.core/scripts/libraries/jquery.delegate.js',
            'dellclarity.core/scripts/libraries/jquery.tooltip.js',
            'dellclarity.core/scripts/libraries/jquery.poshytip.js',
            'dellclarity.core/scripts/libraries/jquery.form.js',
            'dellclarity.core/scripts/libraries/jquery.blockUI.js',
            'dellclarity.core/scripts/libraries/splitter.js',
            'dellclarity.core/scripts/libraries/jquery.contextmenu.js',

            // Chassis view
            'dellclarity.core/scripts/controls/chassisStatus.js',
            'dellclarity.core/scripts/controls/topologyView.js',
            'dellclarity.core/scripts/controls/chassisfront.js',

            // Topology view
            'dellclarity.core/scripts/controls/topologyGraph.js',
            'dellclarity.core/scripts/includes/excanvas.js',
            'dellclarity.core/scripts/includes/jit.js'
        ];
    }

    // In development mode, also load markupValidation.js
    /*if (DellClarity.Config.developmentMode) {
        coreScripts.push('dellclarity.core/scripts/includes/dev.markupValidation.js');
    }*/

    DellClarity.Resources.load(coreScripts, function () {
        // Get the main view container element
        if (DellClarity.Config.apiVersion >= 2.5) {
            // The main view container must be an <article> element with an ARIA role of "main".
            // This ensures proper semantics and accessibility.
            DellClarity.__mainViewContainer = document.querySelector('article[role="main"]');
        }
        else {
            // Older DellClarity applications used an ID of "mainContent" to identify the main view container.
            // This is not allowed anymore, because it allows for improper semantics and accessibility
            DellClarity.__mainViewContainer = document.getElementById('mainContent');
        }

        // If we get here and still haven't found a matching element, then throw an exception
        if (!DellClarity.__mainViewContainer) {
            throw new Error('DellClarity was unable to find the main view container for this page. Expected <article role="main">');
        }

        //sets default language for moment.js
        if (moment)
            moment.locale('en');

        // Call updateLayout whenever the browser window is resized or scrolled
        $(window).resize(DellClarity.__updateLayout);
        $(DellClarity.__mainViewContainer).scroll(DellClarity.__updateLayout);

        //Configure user session
        DellClarity.Session.configure(DellClarity.Config.sessionTimeout);

        // Everything has been loaded, so call the initialize handler(s)
        if (DellClarity.initialize)
            DellClarity.initialize();
    });
})();
