/// <field type="Object" static="true">
/// This object provides helper methods for retrieving JSON data, XML data, and dynamic page content
/// via AJAX.
/// </field>
DellClarity.Ajax = (function () {

    var redirect = null;
    
    // private methods
    var doAjax = function (url, successHandler, failureHandler, customOptions) {
        var options = {
            cache: false,
            dataType: 'html',
            type: 'GET',
            data: null,
            async: true,
            contentType: 'application/x-www-form-urlencoded',
            redirect: redirect
        };

        var combinedOptions = $.extend(options, customOptions);


        return $.ajax({
            cache: combinedOptions.cache,
            type: combinedOptions.type,
            url: url,
            //set timeout to 30 seconds for ajax calls, will then generate a network exception error.
            timeout: 120000,
            data: combinedOptions.data,
            async: combinedOptions.async,
            dataType: combinedOptions.dataType,
            contentType: combinedOptions.contentType,
            tryCount: 0,
            retryLimit: 1,
            success: function (responseText) {
                
                //custom error code for "retry"
                if (responseText.responseCode == -3000) {
                    this.tryCount++;
                    if (this.tryCount <= this.retryLimit) {
                        //try again
                        $.ajax(this);
                        return;
                    }
                    return;
                }
                
                if (successHandler)
                    successHandler(responseText);
            },
            error: function (transport, textStatus) {

                if (transport && transport.status == 403) {
                    if (combinedOptions.redirect)
                        window.location = combinedOptions.redirect;
                    else
                        window.location = "";
                    return;
                }

                if (failureHandler) {

                    var errorCode;
                    var errorType;
                    var message;

                    //NOTE: all network errors are now redirected to client code error handlers.
                    if (textStatus == "abort") {
                        errorCode = 999;
                        errorType = "Abort";
                        message = "Aborted";
                    } else if (transport.status == 0) {
                        errorCode = transport.status;
                        errorType = transport.statusText;
                        message = DellClarity.Localization.Translations.core.networkexception;
                    } else if (transport.status == 500) {
                        message = DellClarity.Localization.Translations.core.serverexception;
                    } else {
                        errorCode = transport.status;
                        errorType = transport.statusText;
                        message = url;
                    }

                    if (transport.status == 500) {
                        failureHandler(
                        {
                            errorMessage: message,
                            fldErrors: [
                            ]
                        });
                    } else {
                        failureHandler(
                        {
                            errorMessage: message,
                            fldErrors: [
                                { field: 'Error Code', errorMessage: errorCode },
                                { field: 'Error Type', errorMessage: errorType }
                            ]
                        });
                    }
                } else if (transport && transport.status == 0 || transport.status == 503) {
                    // Simulate an ErrorObj object
                    DellClarity.__error({
                        errorMessage: DellClarity.Localization.Translations.core.notresponding,
                        fldErrors: [
                            { errorMessage: DellClarity.Localization.Translations.core.relaunch }
                        ]
                    });
                } else if (transport && transport.status != 200) {
                    // Simulate an ErrorObj object
                    DellClarity.__error({
                        errorMessage: DellClarity.Localization.Translations.core.genericexception,
                        fldErrors: [
                            {
                                field: DellClarity.Localization.Translations.genericTerms.url,
                                errorMessage: url
                            },
                            {
                                field: DellClarity.Localization.Translations.genericTerms.response,
                                errorMessage: transport.status + " " + transport.statusText
                            }
                        ]
                    });
                } else {
                    if (transport)
                        throw new Error(transport.responseText);
                    else
                        throw new Error(textStatus);
                }

            },
            isModified: true
        });
    };

    var checkErrorCodeAndDisplayErrorIfNecessary = function (responseText, myFailureHandler) {
        var jobResponse = responseText;
        if (jobResponse && !isNullUndefinedOrEmpty(jobResponse.responseCode)) {
            if (jobResponse.responseCode == "0" || jobResponse.responseCode == "1") {
                // A response code of zero or 1 indicates success
                return true;
            }
            else {
                // Any other response code indicates an error,
                // so call the error handler (if there is one), or throw the error
                if (myFailureHandler) {
                    if (DellClarity.Config.apiVersion < 2.5) {
                        myFailureHandler(DellClarity.Controls.MessageBox.createErrorMessage(undefined, jobResponse.errorObj));
                    } else {
                        myFailureHandler(jobResponse.errorObj);
                    }
                    return false;
                } else if (jobResponse.errorObj) {
                    DellClarity.__error(jobResponse.errorObj);
                    return false;
                } else {
                    throw new Error(DellClarity.Localization.Translations.core.genericexception);
                }
            }
        } else {
            // There's no response code at all, which is an error
            throw new Error(DellClarity.Localization.Translations.core.genericexception);
        }

        return true;
    };

    // public / privileged methods
    return {
        requestOptions: [],

        Set403Redirect: function (page) {
            redirect = page;
        },
        
        GetHtmlSychronously: function (myDataUrl, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Synchronously retrieves HTML content via AJAX.
            /// </summary>
            /// <param name="myDataUrl" type="String">The URL that returns the HTML content</param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler('')" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the content has been retrieved.
            /// The function will be passed a string containing the HTML.
            /// </param>

            var options = {
                async: false
            };

            return doAjax(myDataUrl, mySuccessHandler, myFailureHandler, options);
        },

        GetView: function (myUrl, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Asynchronously retrieves HTML content via AJAX.
            /// </summary>
            /// <param name="myUrl" type="String">The URL that returns the HTML content</param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler('')" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the content has been retrieved.
            /// The function will be passed a string containing the HTML.
            /// </param>

            return doAjax(myUrl, mySuccessHandler, myFailureHandler);
        },

        UpdateWithHtml: function (myUrl, element, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Asynchronously retrieves HTML content via AJAX, and inserts it into the specified DOM element.
            /// </summary>
            /// <param name="myUrl" type="String">The URL that returns the HTML content</param>
            /// <param name="element" type="HTMLElement">The DOM element (or ID) that will contain the HTML</param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler('')" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the content has been retrieved.
            /// The function will be passed a string containing the HTML.
            /// </param>

            if (typeof element === 'string') {
                element = document.getElementById(element);
            }
            var successHandler = function (responseHtml) {
                element.innerHTML = responseHtml;
                if (mySuccessHandler)
                    mySuccessHandler(responseHtml);
            };

            return doAjax(myUrl, successHandler, myFailureHandler);
        },

        GetJSON: function (myUrl, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Asynchronously retrieves JSON data via AJAX.
            /// </summary>
            /// <param name="myUrl" type="String">The URL that returns the JSON data</param>
            /// <param name="objectToPass" type="Object" mayBeNull="true" optional="true">
            /// The data to pass to the AJAX call
            /// </param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the JSON data has been retrieved.
            /// The function will be passed the parsed JavaScript object (NOT the JSON string!).
            /// </param>
            /// <param name="myFailureHandler" type="Function" value="myFailureHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked if the AJAX request fails or the returned JSON data
            /// is invalid. The function will be passed an error object containing information about the failure.
            /// </param>

            var jobRequest = this.__buildJobRequest(objectToPass);

            var data = JSON.stringify(jobRequest);

            var options = {
                dataType: 'json',
                type: 'GET',
                data: data,
                contentType: 'application/json; charset=utf-8'
            };

            var successHandler = function (responseText) {
                var noErrors = checkErrorCodeAndDisplayErrorIfNecessary(responseText, myFailureHandler);
                if (!noErrors) {
                    return;
                }

                var shouldDoSuccessHandler = responseText.responseCode == "0";
                if (mySuccessHandler && shouldDoSuccessHandler)
                    mySuccessHandler(responseText.responseObj);
                else if (myFailureHandler && !shouldDoSuccessHandler)
                    myFailureHandler(responseText.errorObj);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        },

        GetXMLSynchronously: function (myUrl, mySuccessHandler, myFailureHandler) {

            var options = {
                dataType: 'xml',
                type: 'GET',
                async: false,
                contentType: 'text/xml; charset=utf-8'
            };

            var successHandler = function (responseText) {
                if (mySuccessHandler)
                    mySuccessHandler(responseText);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        },

        GetJSONSynchronously: function (myUrl, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Synchronously retrieves JSON data via AJAX.
            /// </summary>
            /// <param name="myUrl" type="String">The URL that returns the JSON data</param>
            /// <param name="objectToPass" type="Object" mayBeNull="true" optional="true">
            /// The data to pass to the AJAX call
            /// </param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the JSON data has been retrieved.
            /// The function will be passed the parsed JavaScript object (NOT the JSON string!).
            /// </param>
            /// <param name="myFailureHandler" type="Function" value="myFailureHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked if the AJAX request fails or the returned JSON data
            /// is invalid. The function will be passed an error object containing information about the failure.
            /// </param>

            var jobRequest = this.__buildJobRequest(objectToPass);

            var data = JSON.stringify(jobRequest);

            var options = {
                dataType: 'json',
                type: 'GET',
                data: data,
                async: false,
                contentType: 'application/json; charset=utf-8'
            };

            var successHandler = function (responseText) {
                var noErrors = checkErrorCodeAndDisplayErrorIfNecessary(responseText, myFailureHandler);
                if (!noErrors) {
                    return;
                }

                var shouldDoSuccessHandler = responseText.responseCode == "0";
                if (mySuccessHandler && shouldDoSuccessHandler)
                    mySuccessHandler(responseText.responseObj);
                else if (myFailureHandler && !shouldDoSuccessHandler)
                    myFailureHandler(responseText.errorObj);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        },

        PostJSON: function (myUrl, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Asynchronously posts data as JSON via AJAX.
            /// </summary>
            /// <param name="myUrl" type="String">The URL that accepts the JSON data</param>
            /// <param name="objectToPass" type="Object" mayBeNull="true" optional="true">
            /// The data to pass to the AJAX call
            /// </param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the AJAX request has completed.
            /// If the AJAX response contained JSON data, then the function will be passed the parsed 
            /// JavaScript object (NOT the JSON string!).
            /// </param>
            /// <param name="myFailureHandler" type="Function" value="myFailureHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked if the AJAX request fails or the returned JSON data
            /// is invalid. The function will be passed an error object containing information about the failure.
            /// </param>
            var jobRequest = this.__buildJobRequest(objectToPass);

            var data = JSON.stringify(jobRequest);

            var options = {
                dataType: 'json',
                type: 'POST',
                data: data,
                contentType: 'application/json; charset=utf-8'
            };

            var successHandler = function (response) {
                var noErrors = checkErrorCodeAndDisplayErrorIfNecessary(response, myFailureHandler);
                if (!noErrors)
                    return;

                var shouldDoSuccessHandler = response.responseCode == "0";
                if (mySuccessHandler && shouldDoSuccessHandler)
                    mySuccessHandler(response.responseObj, response);
                else if (myFailureHandler && !shouldDoSuccessHandler)
                    myFailureHandler(response.errorObj);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        },

        PostJSONSynchronously: function (myUrl, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Synchronously posts data as JSON via AJAX.
            /// </summary>
            /// <param name="myUrl" type="String">The URL that accepts the JSON data</param>
            /// <param name="objectToPass" type="Object" mayBeNull="true" optional="true">
            /// The data to pass to the AJAX call
            /// </param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the AJAX request has completed.
            /// If the AJAX response contained JSON data, then the function will be passed the parsed 
            /// JavaScript object (NOT the JSON string!).
            /// </param>
            /// <param name="myFailureHandler" type="Function" value="myFailureHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked if the AJAX request fails or the returned JSON data
            /// is invalid. The function will be passed an error object containing information about the failure.
            /// </param>

            var jobRequest = this.__buildJobRequest(objectToPass);

            var data = JSON.stringify(jobRequest);

            var options = {
                dataType: 'json',
                type: 'POST',
                data: data,
                async: false,
                contentType: 'application/json; charset=utf-8'
            };

            var successHandler = function (responseText) {
                var noErrors = checkErrorCodeAndDisplayErrorIfNecessary(responseText, myFailureHandler);
                if (!noErrors) {
                    return;
                }

                var shouldDoSuccessHandler = responseText.responseCode == "0";
                if (mySuccessHandler && shouldDoSuccessHandler)
                    mySuccessHandler(responseText.responseObj);
                else if (myFailureHandler && !shouldDoSuccessHandler)
                    myFailureHandler(responseText.errorObj);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        },

        PostJSONFormStyle: function (myUrl, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// Asynchronously posts data as a URL-encoded string via AJAX.
            /// </summary>
            /// <param name="myUrl" type="String">The URL that accepts the JSON data</param>
            /// <param name="objectToPass" type="Object" mayBeNull="true" optional="true">
            /// The data to pass to the AJAX call
            /// </param>
            /// <param name="mySuccessHandler" type="Function" value="mySuccessHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked after the AJAX request has completed.
            /// If the AJAX response contained JSON data, then the function will be passed the parsed 
            /// JavaScript object (NOT the JSON string!).
            /// </param>
            /// <param name="myFailureHandler" type="Function" value="myFailureHandler({})" mayBeNull="true" optional="true">
            /// A callback function that will be invoked if the AJAX request fails or the returned JSON data
            /// is invalid. The function will be passed an error object containing information about the failure.
            /// </param>

            var jobRequest = this.__buildJobRequest(objectToPass);

            var data = jobRequest;

            var options = {
                dataType: 'json',
                type: 'POST',
                data: data
            };

            var successHandler = function (responseText) {
                var noErrors = checkErrorCodeAndDisplayErrorIfNecessary(responseText, myFailureHandler);
                if (!noErrors)
                    return;

                var shouldDoSuccessHandler = responseText.responseCode == "0";
                if (mySuccessHandler && shouldDoSuccessHandler)
                    mySuccessHandler(responseText.responseObj);
                else if (myFailureHandler && !shouldDoSuccessHandler)
                    myFailureHandler(responseText.errorObj);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        },

        //#region REST operations (Deprecated)

        GetRestfulJSON: function (myUrl, id, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// This method is deprecated. DellClarity does not currently support REST.
            /// Support may be added in a future release.
            /// </summary>

            var jobRequest = this.__buildJobRequest(objectToPass);
            var data = jobRequest;
            
            if (id) {
                if (myUrl && !myUrl.endsWith("/"))
                    myUrl += "/";
                myUrl += id;
            }

            var options = {
                dataType: 'json',
                type: 'GET',
                data: (data ? data : null),
                contentType: 'application/json; charset=utf-8'
            };

            var successHandler = function (response) {
                var noErrors = checkErrorCodeAndDisplayErrorIfNecessary(response, myFailureHandler);
                if (!noErrors) {
                    return;
                }

                var shouldDoSuccessHandler = response.responseCode == "0";
                if (mySuccessHandler && shouldDoSuccessHandler)
                    mySuccessHandler(response.responseObj, response);
                else if (myFailureHandler && !shouldDoSuccessHandler)
                    myFailureHandler(response.errorObj);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        },

        PutJSON: function (myUrl, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// This method is deprecated. DellClarity does not currently support REST.
            /// Support may be added in a future release.
            /// </summary>

            return this.__SendJSONInternal(myUrl, 'PUT', objectToPass, mySuccessHandler, myFailureHandler);
        },

        DeleteJSON: function (myUrl, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>
            /// This method is deprecated. DellClarity does not currently support REST.
            /// Support may be added in a future release.
            /// </summary>

            return this.__SendJSONInternal(myUrl, 'DELETE', objectToPass, mySuccessHandler, myFailureHandler);
        },

        //#endregion

        //#region PRIVATE members

        __appendRequestOptions: function (jobRequest) {
            /// <summary>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</summary>
            $(DellClarity.Ajax.requestOptions).each(function(index, item) {
                if ($.isFunction(item.value))
                    jobRequest[item.name] = item.value();
                else
                    jobRequest[item.name] = item.value;
            });
        },
        
        __buildJobRequest: function(objectToPass) {
            var jobRequest;

            if (objectToPass) {
                if (objectToPass.requestObj || objectToPass.filterObj || objectToPass.sortObj || objectToPass.schedule) {
                    jobRequest = objectToPass;
                } else {
                    jobRequest = { requestObj: objectToPass };
                }
            } else {
                jobRequest = { requestObj: null };
            }

            this.__appendRequestOptions(jobRequest);

            return jobRequest;
        },

        __SendJSONInternal: function (myUrl, verb, objectToPass, mySuccessHandler, myFailureHandler) {
            /// <summary>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</summary>

            var jobRequest = {};

            if (objectToPass) {
                if (objectToPass.requestObj || objectToPass.filterObj || objectToPass.sortObj) {
                    jobRequest = objectToPass;
                } else {
                    jobRequest = { requestObj: objectToPass };
                }
            } else {
                jobRequest = { requestObj: null };
            }

            this.appendRequestOptions(jobRequest);

            var data = JSON.stringify(jobRequest);

            var options = {
                dataType: 'json',
                type: verb,
                data: data,
                contentType: 'application/json; charset=utf-8'
            };

            var successHandler = function (response) {
                var noErrors = checkErrorCodeAndDisplayErrorIfNecessary(response, myFailureHandler);
                if (!noErrors)
                    return;

                var shouldDoSuccessHandler = response.responseCode == "0";
                if (mySuccessHandler && shouldDoSuccessHandler)
                    mySuccessHandler(response.responseObj, response);
                else if (myFailureHandler && !shouldDoSuccessHandler)
                    myFailureHandler(response.errorObj);
            };

            return doAjax(myUrl, successHandler, myFailureHandler, options);
        }

        //#endregion
    };
})();
