/*
 * This file contains global helper functions that simplify, enhance, or extend JavaScript functionality.
 * These functions are nonstandard and not based on any API or spec.
 * 
 * See javascript.polyfills.js for implementations of standards-based APIs and specs.
 *
 */
(function () {
    window.devLog = function (message, optionalParams) {
        /// <summary>
        /// Logs a message to the console, only if DellClarity is running in development mode.
        /// </summary>
        /// <param name="message" type="String" mayBeNull="true"/>
        /// <param name="optionalParams" type="Object" optional="true" mayBeNull="true"/>

        if (DellClarity.Config.developmentMode) {
            if (optionalParams)
                console.log(message, optionalParams);
            else
                console.log(message);
        }
    };

    window.isNullOrEmpty = function (value) {
        /// <summary>
        /// Tests whether a value is null or an empty string/array
        /// </summary>
        /// <param name="value" mayBeNull="true" optional="true">The value to inspect</param>
        /// <returns type="Boolean"/>

        return value === null || value === ''
            || (value && value.length === 0); // Support checking for empty arrays
    };

    window.isNullOrUndefined = function (value) {
        /// <summary>
        /// Tests whether a value is null or undefined
        /// </summary>
        /// <param name="value" mayBeNull="true" optional="true">The value to inspect</param>
        /// <returns type="Boolean"/>
        return value === undefined || value === null;
    };

    window.isNullUndefinedOrEmpty = function (value) {
        /// <summary>
        /// Tests whether a value is null, undefined, or an empty string/array
        /// </summary>
        /// <param name="value" mayBeNull="true" optional="true">The value to inspect</param>
        /// <returns type="Boolean"/>
        return value === undefined || value === null || value === ''
            || value.length === 0; // Support checking for empty arrays
    };

    window.valueOrDefault = function (value, defaultValue) {
        /// <summary>
        /// Returns the specified value as long as it's not null, undefined, or an empty string/array,
        /// in which case the specified default value is returned instead.
        /// </summary>
        /// <param name="value" mayBeNull="true">
        /// The value to be returned (as long as it's not null, undefined, or an empty string/array
        /// </param>
        /// <param name="defaultValue" mayBeNull="true">
        /// The default value, which will be returned if value is null, undefined, or an empty string/array
        /// </param>

        if (isNullUndefinedOrEmpty(value))
            return defaultValue;
        else
            return value;
    };

    Number.prototype.isInt32 = function () {
        /// <summary>
        /// Returns a boolean value indicating whether the current value is an Integer of 32 bits.
        /// </summary>

        return this >= -2147483648 && this <= 2147483647 && this % 1 === 0;
    }

    String.prototype.isNumber = function () {
        /// <summary>
        /// Tests whether a string is a valid numeric value.
        /// </summary>
        /// <returns type="Boolean">true if the string is a valid real number; otherwise, false</returns>
        return !isNaN(parseFloat(this)) && isFinite(this);
    };

    String.prototype.isDate = function () {
        /// <summary>
        /// Tests whether a string is a valid date/time string.
        /// </summary>
        /// <returns type="Boolean">true if the string contains a valid date/time; otherwise, false</returns>

        var date = moment(this);
        return date && date.isValid();
    };
    
    String.prototype.isIsoDate = function () {
        /// <summary>
        /// Tests whether a string is a valid ISO 8601 date/time string.
        /// </summary>
        /// <returns type="Boolean">true if the string contains an ISO 8601 date/time; otherwise, false</returns>

        // A simple regular expression for detecting ISO 8601 dates
        // Note: This expression does NOT ensure that the date is valid, just that it's in the right format
        if (!String.prototype.__iso8601DateFormat) {
            var iso8601DateFormat = /^\d{4}-\d{2}-\d{2}(T\d{2}:\d{2}:\d{2}(\.\d+)?(Z|[\-\+]\d{2}:\d{2}))?$/;
            if (iso8601DateFormat.compile)
                iso8601DateFormat.compile(iso8601DateFormat);
            String.prototype.__iso8601DateFormat = iso8601DateFormat;
        }

        return String.prototype.__iso8601DateFormat.test(this);
    };

    Date.parseDate = function (value) {
        /// <summary>
        /// Parses the specified date string and returns a Date object.
        /// &#10; NOTE: This differs from the Date.parse() function, which returns a tick count (Number) rather than a Date.
        /// </summary>
        /// <param name="value" type="String">The string to be parsed</param>
        /// <returns type="Date">The parsed Date</returns>
        
        var date = moment(value);
        if (date && date.isValid())
            return date.toDate();
        else
            return null;
    };

    Date.parseUTCDate = function (value) {
        /// <summary>
        /// Parses the specified UTC date string and returns a Date object.
        /// &#10; NOTE: This differs from the Date.UTC() function, which returns a tick count (Number) rather than a Date.
        /// </summary>
        /// <param name="value" type="String">The UTC string to be parsed</param>
        /// <returns type="Date">The parsed Date</returns>
        
        var date = moment(value);
        if (date && date.isValid())
            return new Date(Date.UTC(date.year(), date.month(), date.date(), date.hour(), date.minute(), date.second(), date.millisecond()));
        else
            return null;
    };

    Date.prototype.format = function (format) {
        /// <summary>
        /// Returns a date string in the specified format.
        /// </summary>
        /// <param name="format" type="String">
        /// A date format string, such as the following:
        /// &#10; L = 2/14/2010
        /// &#10; LL = Feb 14 2010
        /// &#10; LLL = Feb 14 2010 3:25 PM
        /// &#10; LLLL = Sunday, February 14 2010 3:25 PM
        /// &#10; M/D/YYYY = 2/14/2010
        /// &#10; MM/DD/YYYY = 02/14/2010
        /// &#10; dddd, MMMM Do YYYY, h:mm:ss a = Sunday, February 14th 2010, 3:25:50 pm
        /// &#10; ddd, hA = Sun, 3PM
        /// &#10;A full list of format string tokens can be found at http://momentjs.com/docs/#/displaying/format/
        /// </param>
        return moment(this).format(format);
    };

    Date.prototype.humanize = function (/* optional */ fromDate) {
        /// <signature>
        ///     <summary>
        ///     Humanizes the date to a string such as "5 days ago", "Next Wednesday", "Last month", etc.
        ///     </summary>
        ///     <returns type="String">The humanized date</returns>
        /// </signature>
        /// <signature>
        ///     <summary>
        ///     Humanizes the difference between this date and the specified date.
        ///     For example, "5 days", "2 weeks", "6 months", etc.
        ///     </summary>
        ///     <param name="fromDate" type="Date" mayBeNull="false" optional="false">
        ///     The date to compare to this date.  The returned string will be the difference between the two.
        ///     </param>
        ///     <returns type="String">The humanized difference between the two dates</returns>
        /// </signature>

        if (fromDate)
            return moment(this).from(fromDate, true);
        else
            return moment(this).fromNow();
    };

    Number.prototype.format = function (format) {
        /// <summary>
        /// Formats a number according to the current Globalization settings.
        /// </summary>
        /// <param name="format" type="String">
        /// A numeric format string, such as the following:
        /// &#10; Number: n = 123.45, n0 = 123, n1 = 123.5
        /// &#10; Decimal: d = 123, d4 = 0123
        /// &#10; Currency: c = $123.45, c0 = $123
        /// &#10; Percentage: p = 12.34%, p0 = 12%, p4 = 12.3450%
        /// </param>
        /// <returns type="String">The formatted number</returns>
        if (window.Globalize) {
            if (format) {
                return Globalize.format(parseFloat(this), format);
            }
            else {
                return Globalize.format(parseFloat(this), 'n');
            }
        }

        // Unable to format, so just return the unformatted value
        return value.toString();
    };

    Object.getNestedPropertyValue = function (obj, property) {
        /// <summary>
        /// Returns the value of the specified property of the object,
        /// even if the property is multiple levels down the object map.
        /// &#10; For example, if you have the following object:
        /// &#10;
        /// &#10; a = { b: { c: 'hello world' } };
        /// &#10;
        /// &#10; Then calling Object.getNestedPropertyValue(a, 'b.c')
        /// will return the value 'hello world'.
        /// </summary>
        /// <param name="obj" type="Object">The object whose nested property is returned</param>
        /// <param name="property" type="String">The string path to the nested property</param>
        /// <returns type="Object" mayBeNull="true">The value of the nested property</returns>
        
        if (typeof property !== 'string') return null;

        var s = property;
        var o = obj;
        s = s.replace(/\[(\w+)\]/g, '.$1'); // convert indexes to properties
        s = s.replace(/^\./, ''); // strip a leading dot
        var a = s.split('.');
        while (a.length) {
            var n = a.shift();
            if (n in o) {
                o = o[n];
            } else {
                return null;
            }
        }
        return o;
    };

    jQuery.fn.shiftSelectableKO = function () {
        var lastChecked,
            $boxes = this;

        $boxes.click(function (evt) {
            if (!lastChecked) {
                lastChecked = this;
                return;
            }

            if (evt.shiftKey) {
                var start = $boxes.index(this),
                    end = $boxes.index(lastChecked);

                //$boxes.slice(Math.min(start, end), Math.max(start, end) + 1)
                //    .attr('checked', lastChecked.checked)
                //    .trigger('change');

                var boxes = $boxes.slice(Math.min(start, end), Math.max(start, end) + 1);

                $.each(boxes, function (i, box) {
                    var data = ko.dataFor(box);
                    data.checked(lastChecked.checked);
                });

            }

            lastChecked = this;
        });
    };
    
    jQuery.fn.exists = function () {
        /// <summary>
        /// Indicates whether the jQuery selector matched any HTML elements.
        /// </summary>
        /// <returns type="Boolean">
        /// true if at least one HTML element matched the selector(s); otherwise, false
        /// </returns>

        return this.length > 0;
    };

    jQuery.fn.onUnique = function (events, selector, data, handler) {
        /// <summary>
        /// This function directly mirrors the jQuery.on function, 
        /// except that it will not bind the same event to the same element more than once.
        /// This is useful if you are not sure whether you are already handling
        /// a certain event for an element.
        /// </summary>

        var eventKey = 'onUnique_' + events;
        if (!this.data(eventKey)) {
            this.on.apply(this, arguments);
            this.data(eventKey, true);
        }

        return this;
    };

    //adding extra browser detection function for jQuery 1.8.3
    // Limit scope pollution from any deprecated API
    (function () {

        var matched, browser;

        // Use of jQuery.browser is frowned upon.
        // More details: http://api.jquery.com/jQuery.browser
        // jQuery.uaMatch maintained for back-compat
        jQuery.uaMatch = function (ua) {
            ua = ua.toLowerCase();

            var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
                /(webkit)[ \/]([\w.]+)/.exec(ua) ||
                /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
                /(msie) ([\w.]+)/.exec(ua) ||
                ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
                [];

            return {
                browser: match[1] || "",
                version: match[2] || "0"
            };
        };

        matched = jQuery.uaMatch(navigator.userAgent);
        browser = {};

        if (matched.browser) {
            browser[matched.browser] = true;
            browser.version = matched.version;
        }

        // Chrome is Webkit, but Webkit is also Safari.
        if (browser.chrome) {
            browser.webkit = true;
        } else if (browser.webkit) {
            browser.safari = true;
        }

        jQuery.browser = browser;
    })();
    
    window.getCookie = function (cookieName) {
        if (document.cookie.length > 0) {
            var cookieStart = document.cookie.indexOf(cookieName + "=");
            if (cookieStart != -1) {
                cookieStart = cookieStart + cookieName.length + 1;
                var cookieEnd = document.cookie.indexOf(";", cookieStart);
                if (cookieEnd == -1) cookieEnd = document.cookie.length;
                return unescape(document.cookie.substring(cookieStart, cookieEnd));
            }
        }
        return "";
    };

    window.setCookie = function(cookieName, value, expiredays) {
        var exdate = new Date();
        exdate.setDate(exdate.getDate() + expiredays);
        document.cookie = cookieName + "=" + escape(value) +
            ((expiredays == null) ? "" : "; expires=" + exdate.toGMTString());
    };

    window.htmlEscape = function (str) {
        return String(str)
            .replace(/&/g, '&amp;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;');
    };
    
    window.to
    
})();
