/*
 * This file contains polyfills, which are JavaScript functions that reproduce standards-based APIs 
 * and specs that are not supported by the browser.  This allows developers to write standards-based
 * code without having to worry about whether it is supported by all browsers.  Any browser that doesn't
 * support a certain feature will gain support via a polyfill.
 * 
 * Non-standards-based code (such as helper methods or JavaScript enhancements) belongs in
 * javascript.extensions.js, not in this file.
 *
 */
(function () {

    //#region Simple Polyfills

    // https://developer.mozilla.org/en-US/docs/DOM/console
    if (!window.console) window.console = {};
    if (!console.log) console.log = function () { };
    if (!console.info) console.info = function () { };
    if (!console.warn) console.warn = function () { };
    if (!console.error) console.error = function () { };
    if (!console.group) console.group = function () { };
    if (!console.groupEnd) console.groupEnd = function () { };

    // https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Function/bind
    if (!Function.prototype.bind) {
        Function.prototype.bind = function (target) {
            if (typeof this !== "function") {
                // closest thing possible to the ECMAScript 5 internal IsCallable function
                throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");
            }

            var slice = Array.prototype.slice,
            args = slice.call(arguments, 1),
            functionToBind = this,
            emptyFunction = function () { },
            boundFunction = function () {
                return functionToBind.apply(this instanceof emptyFunction ? this : target || window,
                                            args.concat(slice.call(arguments)));
            };

            emptyFunction.prototype = this.prototype;
            boundFunction.prototype = new emptyFunction();

            return boundFunction;
        };
    }

    // https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Object/keys
    if (!Object.keys) {
        Object.keys = function (o) {
            if (o !== Object(o))
                throw new TypeError('Object.keys called on non-object');
            var ret = [], p;
            for (p in o) {
                if (Object.prototype.hasOwnProperty.call(o, p))
                    ret.push(p);
            }
            return ret;
        };
    }

    // https://developer.mozilla.org/en-US/docs/DOM/Element.nextElementSibling
    // NOTE: We can't directly pollyfill nextElementSibling because it's an instance property,
    //       so instead, we add a prototype method that pollyfills the instance property
    Element.prototype.getNextElementSibling = function () {
        if (this.nextElementSibling) return this.nextElementSibling;

        var el = this;
        do { el = el.nextSibling; } while (el && el.nodeType !== 1);
        return el;
    };

    // https://developer.mozilla.org/en-US/docs/DOM/Element.firstElementChild
    // NOTE: We can't directly pollyfill firstElementChild because it's an instance property,
    //       so instead, we add a prototype method that pollyfills the instance property
    Element.prototype.getFirstElementChild = function () {
        if (this.firstElementChild) return this.firstElementChild;

        var el = this.firstChild;
        if (el && el.nodeType === 1) {
            return el;
        } else {
            return el.nextElementSibling;
        }
    };

    // https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/String/Trim
    if (typeof String.prototype.trim !== 'function') {
        String.prototype.trim = function () {
            return this.replace(/^\s+|\s+$/g, '');
        };
    }

    // https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/String/startsWith
    if (typeof String.prototype.startsWith !== 'function') {
        String.prototype.startsWith = function (startingText) {
            if (!startingText)
                return false;

            return this.slice(0, startingText.length) === startingText;
        };
    }

    // https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/String/endsWith
    if (typeof String.prototype.endsWith !== 'function') {
        String.prototype.endsWith = function (endingText) {
            if (!endingText)
                return false;

            return this.slice(-endingText.length) === endingText;
        };
    }

    // https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Date/toJSON
    // NOTE: We ALWAYS overwrite the Date.toJSON() function so that all browsers create json dates with the appropriate ISO date format
    Date.prototype.toJSON = function (key) {
        function f(n) {
            // Format integers to have at least two digits.
            return n < 10 ? '0' + n : n;
        }

        function f3(n) {
            // Format integers to have at least three digits.
            return n < 10 ? '00' + n : n < 100 ? '0' + n : n;
        }

        return isFinite(this.valueOf()) ?
            this.getUTCFullYear() + '-' +
            f(this.getUTCMonth() + 1) + '-' +
            f(this.getUTCDate()) + 'T' +
            f(this.getUTCHours()) + ':' +
            f(this.getUTCMinutes()) + ':' +
            f(this.getUTCSeconds()) + '.' +
            f3(this.getUTCMilliseconds()) + 'Z' : null;
    };

    // https://developer.mozilla.org/en-US/docs/JavaScript/Reference/Global_Objects/Date/toJSON
    if (typeof String.prototype.toJSON !== 'function') {
        String.prototype.toJSON =
            Number.prototype.toJSON =
                Boolean.prototype.toJSON = function (key) {
                    return this.valueOf();
                };
    }

    //#endregion

    //#region HTML 5 form validation polyfills

    //#region Property and Method Definitions

    /*****************************************************************************
     * ValidityState type
     *****************************************************************************/
    var ValidityState = {
        customError: false,
        patternMismatch: false,
        rangeOverflow: false,
        rangeUnderflow: false,
        stepMismatch: false,
        tooLong: false,
        typeMismatch: false,
        valueMissing: false,
        valid: true
    };

    /*****************************************************************************
     * willValidate Property
     *****************************************************************************/
    var willValidatePropertyDefinition = {
        configurable: true,
        get: function () {
            // If the field is read-only or disabled, then it does not participate in validation
            if (this.readOnly || this.disabled)
                return false;

            var $input = $(this);

            // If the field is hidden, then it does not participate in validation
            if ($input.css('display') === 'none') return false;

            // If any of the field's parent elements are hidden, then the field does not participate in validation
            var parentIsHidden = false;
            $input.parents().each(function () {
                if ($(this).css('display') === 'none') {
                    parentIsHidden = true;
                    return false;
                }
            });
            return !parentIsHidden;
        }
    };

    /*****************************************************************************
     * setCustomValidity Method
     *****************************************************************************/
    function setCustomValidity(validationMessage) {
        this.__setValidity({ customError: validationMessage });
    }

    /*****************************************************************************
     * checkValidity Method
     *****************************************************************************/
    function checkValidity() {
        if (!this.willValidate) {
            // This element isn't participating in validation right now
            return this.__setValidity({ valid: true });
        }

        var $input = $(this);

        // Validate the `required` attribute
        if ($input.attr('required')) {
            // For checkboxes and radio buttons, inspect the `checked` property
            if ((this.type === 'checkbox' && !this.checked)
            || (this.type === 'radio' && !$('input[name="' + this.name + '"]:checked').exists())) {
                return this.__setValidity({ valueMissing: DellClarity.Localization.Translations.validation.required });
            }

            // For everything else, just check the `value` property
            if (!this.value) {
                return this.__setValidity({ valueMissing: DellClarity.Localization.Translations.validation.required });
            }
        }

        // Validate the `pattern` attribute
        var pattern = $input.attr('pattern');
        if (pattern && this.value) {
            pattern = new RegExp('^' + pattern + '$');
            if (!pattern.test(this.value)) {
                return this.__setValidity({ patternMismatch: DellClarity.Localization.Translations.validation.pattern.replace('{title}', this.title) });
            }
        }

        // If we get here, then the field is valid
        return this.__setValidity({ valid: true });
    }

    /*****************************************************************************
     * checkValidity Method (for forms)
     *****************************************************************************/
    function checkFormValidity() {
        // If validation is disabled, then do nothing
        if (this.noValidate) return true;

        // Call checkValidity on each input element within the form
        var $form = $(this);
        var isValid = true;

        //Check group of checkboxes (particular case)
        $form.find('.checkbox-group-required').each(function () {
            var checkboxName = $($(this).find(':input[type="checkbox"]').get(0)).attr('name');
            var checkGroupValid = false;
            $(this).find(':input[name="' + checkboxName + '"]').each(function () {
                if (checkGroupValid)
                    return;

                if ($(this).get(0).checked)
                    checkGroupValid = true;
            });

            isValid = isValid && checkGroupValid;
        });

        $form.find(':input').each(function () {
            if (this.checkValidity)
                isValid = isValid && this.checkValidity();

            if (!isValid) {
                this.focus();
                return false;
            }
        });

        return isValid;
    }

    /*****************************************************************************
     * Helper method to set the `validity` property
     *****************************************************************************/
    function setValidity(validityState) {
        // If the browser natively supports the validation API, 
        // then we always call setCustomValidity rather than using our polyfill logic
        if (!formValidationPolyfillEnabled) {
            var customValidationMessage =
                validityState.customError ||
                validityState.patternMismatch ||
                validityState.rangeOverflow ||
                validityState.rangeUnderflow ||
                validityState.stepMismatch ||
                validityState.tooLong ||
                validityState.typeMismatch ||
                validityState.valueMissing ||
                '';
            this.setCustomValidity(customValidationMessage);
            return !customValidationMessage;
        }

        // If there is a custom error, then don't allow `valid` to be set to true
        if (validityState.valid) {
            validityState.customError = this.__validity.customError ? this.__validationMessage : undefined;
            validityState.valid = !validityState.customError;
        }

        // Update the `validity` property
        this.__validity = {
            customError: validityState.customError ? true : false,
            patternMismatch: validityState.patternMismatch ? true : false,
            rangeOverflow: validityState.rangeOverflow ? true : false,
            rangeUnderflow: validityState.rangeUnderflow ? true : false,
            stepMismatch: validityState.stepMismatch ? true : false,
            tooLong: validityState.tooLong ? true : false,
            typeMismatch: validityState.typeMismatch ? true : false,
            valueMissing: validityState.valueMissing ? true : false,
            valid: validityState.valid ? true : false
        };

        // Update the `validationMessage` property
        if (this.__validity.valid) {
            // The field is valid, so clear the validation message
            this.__validationMessage = '';
        }
        else {
            // The field is invalid, so set the validation message
            // to the first error message
            this.__validationMessage =
                validityState.customError ||
                validityState.patternMismatch ||
                validityState.rangeOverflow ||
                validityState.rangeUnderflow ||
                validityState.stepMismatch ||
                validityState.tooLong ||
                validityState.typeMismatch ||
                validityState.valueMissing;
        }

        // Return a validity indicator
        return this.__validity.valid;
    }

    function showValidityErrors() {
        // Show/Hide the validation message
        $(":input").each(function () {
            if (this.__validity && this.__validity.valid)
                return;

            var validationMessageId = this.id + '_validationMessage';
            var $validationMessage = $('#' + validationMessageId);
            if (this.__validationMessage) {
                // Create a message <span> if necessary
                if (!$validationMessage.exists()) {
                    $validationMessage = $('<span id="' + validationMessageId + '" class="validationMessage"></span>').insertAfter($(this)).hide();
                }

                $validationMessage.text(this.__validationMessage).fadeIn();
                $(this).addClass("validationError");
            }
        });
    }

    function hideValidityErrors() {
        $(":input").each(function () {
            if (this.__validity && this.__validity.valid)
                return;

            var validationMessageId = this.id + '_validationMessage';
            var $validationMessage = $('#' + validationMessageId);
            if (this.__validationMessage) {
                $validationMessage.fadeOut().text('');
                $(this).removeClass("validationError");
            }
        });
    }

    /*****************************************************************************
     * Applies the standard HTML 5 DOM polyfills to an input element
     *****************************************************************************/
    function polyfill(input) {
        // Standard HTML 5 DOM properties & methods
        // https://developer.mozilla.org/en-US/docs/HTML/Forms_in_HTML
        input.setCustomValidity = setCustomValidity;
        input.checkValidity = checkValidity;
        Object.defineProperty(input, 'willValidate', willValidatePropertyDefinition);
        Object.defineProperty(input, 'validity', { configurable: true, get: function () { return this.__validity; } });
        Object.defineProperty(input, 'validationMessage', { configurable: true, get: function () { return this.__validationMessage; } });

        // Non-standard helper properties & methods that are required by our polyfills
        input.__validity = ValidityState;
        input.__validationMessage = '';
        input.__setValidity = setValidity;
    }

    /*****************************************************************************
     * <input type="email">
     *****************************************************************************/
    function polyfill_email(input) {
        if (!Modernizr.inputtypes.email) {
            // Use the `pattern` attribute to ensure a valid email is entered
            $(input).attr({
                pattern: '[^@]+@[^\\.]+\\.[a-zA-Z]+',
                title: DellClarity.Localization.Translations.validation.email.example,
                placeholder: DellClarity.Localization.Translations.validation.email.hint
            });
        }
    }

    /*****************************************************************************
     * <input type="url">
     *****************************************************************************/
    function polyfill_url(input) {
        if (!Modernizr.inputtypes.url) {
            // Use the `pattern` attribute to ensure a valid URL is entered
            $(input).attr({
                pattern: '(([a-zA-Z]+\\://)|\\\\).+',
                title: DellClarity.Localization.Translations.validation.url.example,
                placeholder: DellClarity.Localization.Translations.validation.url.hint
            });
        }
    }

    /*****************************************************************************
     * <input type="number">
     *****************************************************************************/
    function polyfill_number(input) {
        if (!Modernizr.inputtypes.number) {
            // Add hints for the user
            $(input).attr('title', DellClarity.Localization.Translations.validation.number.example.replace('{example}', 1234));

            // Also polyfill the valueAsNumber property
            Object.defineProperty(input, 'valueAsNumber', {
                configurable: true,
                get: function () {
                    return parseFloat(this.value);
                },
                set: function (newValue) {
                    this.value = newValue;
                }
            });

            // Override the default checkValidity function
            input.checkValidity = function () {
                if (!this.willValidate) {
                    // This element isn't participating in validation right now
                    return setValidity.call(this, { valid: true });
                }

                if (!this.value) {
                    if ($(this).attr('required')) {
                        // No value was entered, and the field is required
                        return setValidity.call(this, { valueMissing: DellClarity.Localization.Translations.validation.required });
                    }
                    else {
                        // No value was entered, but the field isn't required
                        return setValidity.call(this, { valid: true });
                    }
                }

                var value = this.valueAsNumber;
                if (isNaN(value)) {
                    // The value is not a valid number, so display the input example that's in the title attribute
                    return setValidity.call(this, { typeMismatch: this.title });
                }
                else {
                    // Validate the min/max range
                    var min = parseFloat(this.min);
                    var max = parseFloat(this.max);

                    if (!isNaN(min) && value < min) {
                        if (!isNaN(max))
                            return setValidity.call(this, {
                                rangeUnderflow: DellClarity.Localization.Translations.validation.number.invalidRange
                                    .replace('{min}', min)
                                    .replace('{max}', max)
                            });
                        else
                            return setValidity.call(this, {
                                rangeUnderflow: DellClarity.Localization.Translations.validation.number.invalidMin
                                    .replace('{min}', min)
                            });
                    }

                    if (!isNaN(max) && value > max) {
                        if (!isNaN(min))
                            return setValidity.call(this, {
                                rangeOverflow: DellClarity.Localization.Translations.validation.number.invalidRange
                                    .replace('{min}', min)
                                    .replace('{max}', max)
                            });
                        else
                            return setValidity.call(this, {
                                rangeOverflow: DellClarity.Localization.Translations.validation.number.invalidMax
                                    .replace('{max}', max)
                            });
                    }

                    // Validate the step
                    if (!isNaN(min)) {
                        var step = parseFloat(this.step) || 1;
                        var difference = value - min;

                        if (difference % step > 0) {
                            return setValidity.call(this, {
                                stepMismatch: DellClarity.Localization.Translations.validation.number.invalidStep
                                    .replace('{step}', step)
                                    .replace('{min}', min)
                            });
                        }
                    }

                    // Re-set the value to ensure proper formatting
                    this.valueAsNumber = value;
                    $(this).trigger('change', true /* skipValidation */);

                    // The value is a valid number, so clear any errors
                    return setValidity.call(this, { valid: true });
                }
            };
        }
    }

    var ONE_DAY = 86400000; // One day, in milliseconds
    var ONE_HOUR = 3600000; // One hour, in milliseconds
    var ONE_MINUTE = 60000; // One minute, in milliseconds

    /*****************************************************************************
     * <input type="date">
     *****************************************************************************/
    function polyfill_date(input) {
        if (!Modernizr.inputtypes.date) {
            // Add hints for the user
            $(input).attr({
                title: DellClarity.Localization.Translations.validation.date.example.replace('{example}', (new Date()).format('L')),
                placeholder: DellClarity.Localization.Translations.validation.date.hint
            });

            // Apply the date/time polyfill
            polyfill_datetime_checkValidity(input, 'L', ONE_DAY);
            polyfill_datetime_valueAs(input, 'L', false);

        }
        else if (!Modernizr.input['date-valueAs']) {
            // Polyfill the `valueAsDate` and `valueAsNumber` properties
            polyfill_datetime_valueAs(input, 'YYYY-MM-DD', false);
        }
    }

    /*****************************************************************************
     * <input type="datetime">
     *****************************************************************************/
    function polyfill_datetime(input) {
        if (!Modernizr.inputtypes.datetime) {
            // Add hints for the user
            $(input).attr({
                title: DellClarity.Localization.Translations.validation.datetime.example.replace('{example}', (new Date()).format('L LT')),
                placeholder: DellClarity.Localization.Translations.validation.datetime.hint
            });

            // Apply the shared date/time polyfill
            polyfill_datetime_checkValidity(input, 'L LT', ONE_MINUTE);
            polyfill_datetime_valueAs(input, 'L LT', false);
        }
        else if (!Modernizr.input['datetime-valueAs']) {
            // Polyfill the `valueAsDate` and `valueAsNumber` properties
            polyfill_datetime_valueAs(input, 'YYYY-MM-DDTHH:mm:ssZ', false);
        }
    }

    /*****************************************************************************
     * <input type="datetime-local">
     *****************************************************************************/
    function polyfill_datetime_local(input) {
        if (!Modernizr.inputtypes['datetime-local']) {
            // Add hints for the user
            $(input).attr({
                title: DellClarity.Localization.Translations.validation.datetime.example.replace('{example}', (new Date()).format('L LT')),
                placeholder: DellClarity.Localization.Translations.validation.datetime.hint
            });

            // Apply the shared date/time polyfill
            polyfill_datetime_checkValidity(input, 'L LT', ONE_MINUTE);
            polyfill_datetime_valueAs(input, 'L LT', true);
        }
        else if (!Modernizr.input['datetime-local-valueAs']) {
            // Polyfill the `valueAsDate` and `valueAsNumber` properties
            polyfill_datetime_valueAs(input, 'YYYY-MM-DDTHH:mm:ss', true);
        }
    }

    /*****************************************************************************
     * Helper methods for <input type="date"> and <input type="datetime-local">
     *****************************************************************************/
    function polyfill_datetime_valueAs(input, formatString, isLocal) {
        // Polyfill the valueAsDate property
        Object.defineProperty(input, 'valueAsDate', {
            configurable: true,
            get: function () {
                if (isLocal)
                    return Date.parseDate(this.value);
                else
                    return Date.parseUTCDate(this.value);
            },
            set: function (newValue) {
                if (!newValue)
                    this.value = '';
                else if (isLocal)
                    this.value = newValue.format(formatString);
                else
                    this.value = moment(newValue).utc().format(formatString);
            }
        });

        // Polyfill the valueAsNumber property
        Object.defineProperty(input, 'valueAsNumber', {
            configurable: true,
            get: function () {
                var date = this.valueAsDate;
                return date ? date.getTime() : NaN;
            },
            set: function (newValue) {
                this.valueAsDate = new Date(newValue);
            }
        });
    }

    function polyfill_datetime_checkValidity(input, formatString, precision) {
        // Override the default checkValidity function
        input.checkValidity = function () {
            if (!this.willValidate) {
                // This element isn't participating in validation right now
                return setValidity.call(this, { valid: true });
            }

            if (!this.value) {
                if ($(this).attr('required')) {
                    // No value was entered, and the field is required
                    return setValidity.call(this, { valueMissing: DellClarity.Localization.Translations.validation.required });
                }
                else {
                    // No value was entered, but the field isn't required
                    return setValidity.call(this, { valid: true });
                }
            }

            var value = this.valueAsDate;
            if (!value) {
                // The value is not a valid date/time, so display the input example that's in the title attribute
                return setValidity.call(this, { typeMismatch: this.title });
            }
            else {
                // Validate the min/max date range
                var min = this.min ? Date.parseDate(this.min) : undefined;
                var max = this.max ? Date.parseDate(this.max) : undefined;

                if (min && value < min) {
                    if (max)
                        return setValidity.call(this, {
                            rangeUnderflow: DellClarity.Localization.Translations.validation.date.invalidRange
                                .replace('{min}', min.format(formatString))
                                .replace('{max}', max.format(formatString))
                        });
                    else
                        return setValidity.call(this, {
                            rangeUnderflow: DellClarity.Localization.Translations.validation.date.invalidMin
                                .replace('{min}', min.format(formatString))
                        });
                }

                if (max && value > max) {
                    if (min)
                        return setValidity.call(this, {
                            rangeOverflow: DellClarity.Localization.Translations.validation.date.invalidRange
                                .replace('{min}', min.format(formatString))
                                .replace('{max}', max.format(formatString))
                        });
                    else
                        return setValidity.call(this, {
                            rangeOverflow: DellClarity.Localization.Translations.validation.date.invalidMax
                                .replace('{max}', max.format(formatString))
                        });
                }

                // Validate the step
                if (min && this.step) {
                    var step = this.step * precision;
                    var difference = value - min;

                    if (difference % step > 0) {
                        if (step >= ONE_DAY)
                            return setValidity.call(this, {
                                stepMismatch: DellClarity.Localization.Translations.validation.date.invalidStep.days
                                    .replace('{step}', step / ONE_DAY)
                                    .replace('{min}', min.format(formatString))
                            });
                        else if (step >= ONE_HOUR)
                            return setValidity.call(this, {
                                stepMismatch: DellClarity.Localization.Translations.validation.date.invalidStep.hours
                                    .replace('{step}', step / ONE_HOUR)
                                    .replace('{min}', min.format(formatString))
                            });
                        else
                            return setValidity.call(this, {
                                stepMismatch: DellClarity.Localization.Translations.validation.date.invalidStep.minutes
                                    .replace('{step}', step / ONE_MINUTE)
                                    .replace('{min}', min.format(formatString))
                            });
                    }
                }

                // Re-set the value to ensure proper formatting
                this.valueAsDate = value;
                $(this).trigger('change', true /* skipValidation */);

                // The value is a valid date/time, so clear any errors
                return setValidity.call(this, { valid: true });
            }
        };
    }

    //#endregion

    //#region Feature detection and applying the polyfills

    /*****************************************************************************
     * One-time polyfills (this code only runs once)
     *****************************************************************************/
    var formValidationPolyfillEnabled =
        !HTMLInputElement.prototype.setCustomValidity ||
        !HTMLInputElement.prototype.checkValidity ||
        !HTMLFormElement.prototype.checkValidity;

    if (formValidationPolyfillEnabled) {
        // Polyfill all <input>, <select>, and <textarea> elements
        polyfill(HTMLInputElement.prototype);
        polyfill(HTMLSelectElement.prototype);
        polyfill(HTMLTextAreaElement.prototype);

        // Polyfill all <form> elements
        HTMLFormElement.prototype.checkValidity = checkFormValidity;

        // Validate all fields on change
        $(document).on('change.DellClarity.polyfill', ':input', function (evt, skipValidation) {
            if (!skipValidation) {
                hideValidityErrors();
                this.checkValidity();
            }
        });
    }
    else {
        // Validate any fields that have been polyfilled (i.e. input types that aren't natively supported)
        $(document).on('change.DellClarity.polyfill', ':input', function (evt, skipValidation) {
            if (!skipValidation && $(this).data('DellClarity.polyfilled')) {
                hideValidityErrors();
                this.checkValidity();
            }
        });
    }

    // Determine if the browser supports the `valueAsDate` property for each of the date/time input types
    Modernizr.input['date-valueAs'] = !!$('<input type="date" value="2000-01-01">')[0].valueAsDate;
    Modernizr.input['datetime-valueAs'] = !!$('<input type="datetime" value="2000-01-01T00:00:00Z">')[0].valueAsDate;
    Modernizr.input['datetime-local-valueAs'] = !!$('<input type="datetime-local" value="2000-01-01T00:00:00">')[0].valueAsDate;


    /*****************************************************************************
     * DOM-Change Polyfills (this code needs to execute whenever the DOM changes)
     *****************************************************************************/
    DellClarity.__applyPolyfills = function () {
        /// <summary>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</summary>

        if (formValidationPolyfillEnabled) {
            // We need to trigger our custom validation logic whenever a form is submitted, so add an OnSubmit handler.
            // But here's the catch!  This handlers has to be the *first* handler of the submit event, so that we can
            // cancel the event if validation fails (and not trigger any other OnSubmit logic).
            // That's why we attach the event here, immediately after the <form> is instantiated, hopefully before any
            // other event handlers are attached.
            $('form').each(function () {
                var $form = $(this);

                // If it's already been polyfilled, then do nothing
                if ($form.data('DellClarity.polyfilled')) return;

                // Call checkValidity on submit (if the flag is set)
                $(this).on('submit.DellClarity.polyfill', function (evt) {
                    if (!this.checkValidity()) {
                        evt.stopImmediatePropagation();
                        evt.preventDefault();
                        showValidityErrors();
                        return false;
                    }
                });

                // Flag this element as polyfilled, so we don't do it again
                $form.data('DellClarity.polyfilled', true);
            });
        }

        // Polyfill all <input>, <select>, and <textarea> elements
        $(':input').each(function () {
            var $input = $(this);

            // If it's already been polyfilled, then do nothing
            if ($input.data('DellClarity.polyfilled')) return;

            var polyfilled = false;

            switch ($input.attr('type')) {
                case 'email':
                    polyfill_email(this);
                    polyfilled = true;
                    break;
                case 'url':
                    polyfill_url(this);
                    polyfilled = true;
                    break;
                case 'date':
                    polyfill_date(this);
                    polyfilled = true;
                    break;
                case 'datetime':
                    polyfill_datetime(this);
                    polyfilled = true;
                    break;
                case 'datetime-local':
                    polyfill_datetime_local(this);
                    polyfilled = true;
                    break;
                case 'number':
                    polyfill_number(this);
                    polyfilled = true;
                    break;
                case 'text':
                    // IE8 and IE9 report type="text" instead of type="number",
                    // so we check for the min, max, and/or step attributes instead
                    if ($input.attr('min') || $input.attr('max') || $input.attr('step')) {
                        polyfill_number(this);
                        polyfilled = true;
                    }
                    break;
            }

            // Flag this element as polyfilled, so we don't do it again
            if (polyfilled) $input.data('DellClarity.polyfilled', true);
        });
    };

    //#endregion

    //#endregion

})();
