/// <field type="Object" static="true">
/// Disables an element and all of its children when the parameter value is true.
/// </field>
ko.bindingHandlers.styledDisabled = {

    disable: function (element) {
        if ($(element).attr('type') == 'button')
            $(element).removeClass('cux-button-primary');

        $(element).addClass('disabled').attr('disabled', 'disabled')
        .find(':input').attr('disabled', 'disabled');
    
        //DellClarity 1.0 apps required the disabled class on tables and rows
        if (DellClarity.Config.apiVersion < 2.5)
            $(element).find("table, tr, :input").addClass('disabled');
    },
    enable: function (element) {
        if ($(element).attr('type') == 'button')
            $(element).addClass('cux-button-primary');

        $(element).removeClass('disabled').removeAttr('disabled')
        .find(':input').removeAttr('disabled');

        //DellClarity 1.0 apps required the disabled class on tables and rows
        if (DellClarity.Config.apiVersion < 2.5)
            $(element).find("table, tr, :input").removeClass('disabled');
    },

    init: function (element, valueAccessor) {
        var disabled = valueAccessor();
        if (disabled) {
            ko.bindingHandlers.styledDisabled.disable(element);
        }
    },
    update: function (element, valueAccessor) {
        var disabled = valueAccessor();
        if (disabled) {
            ko.bindingHandlers.styledDisabled.disable(element);
        } else {
            ko.bindingHandlers.styledDisabled.enable(element);
        }
    }
};

ko.bindingHandlers.title = {
    init: function (element, valueAccessor, allBindingsAccessor) {
        ko.bindingHandlers.title.update(element, valueAccessor, allBindingsAccessor);
    },

    update: function (element, valueAccessor, allBindingsAccessor) {
        var value = valueAccessor(), allBindings = allBindingsAccessor();
        var unwrapped = ko.utils.unwrapObservable(value);
        var val = unwrapped;
        if (!isNullOrEmpty(unwrapped) && val.length > length) {
            $(element).attr('title', unwrapped);
        }
    }
};


/// <field type="Object" static="true">
/// Binds a JavaScript Date object to an HTML <time> element, automatically setting the "datetime" attribute, 
/// the "title" attribute, and the inner text of the element to the appropriate values.
/// &#10;&#10;
/// By default, the inner text of the element is set to a humanized date string (e.g. "5 minutes ago", "18 days ago", "last month", etc.).
/// To display the date in a different format, use the timeFormat parameter (e.g. data-bind="time: myDateVar, timeFormat: 'MMMM DD YYYY'").
/// For the full list of supported date/time format strings, go to http://momentjs.com/docs/#/displaying/format/
/// &#10;
/// &#10; Localized date format  
/// &#10; LT 8:30 PM
/// &#10; L 07/10/1986 
/// &#10; LL July 10 1986 
/// &#10; LLL July 10 1986 8:30 PM 
/// &#10; LLLL Saturday, July 10 1986 8:30 PM 
/// </field>
ko.bindingHandlers.time = {
    update: function (element, valueAccessor, allBindingsAccessor, viewModel) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();
        var format = allBindings.timeFormat || null;

        var $element = $(element);

        // If the value is a date string, then convert it to a Date
        if (typeof value === 'string' && value.isDate()) {
            value = Date.parseDate(value);
        }

        // If the date is valid, then set the <time> element's properties
        if (value !== null && value instanceof Date && !isNaN(value)) {
            var formattedDate;

            if (format) {
                // Format the date using the specified format
                formattedDate = value.format(format);
            }
            else {
                // No format was specified, so humanize the date (e.g. "5 days ago")
                formattedDate = value.humanize();
            }

            $element.attr('datetime', value.toJSON());
            $element.attr('title', value.format('LLLL'));
            $element.text(formattedDate);
        }
        else {
            // Not a date or a valid date convertable string, clear out the text
            $element.removeAttr('datetime');
            $element.removeAttr('title');
            $element.text('');
        }
    }
};

/// <field type="Object" static="true">
/// Binds a JavaScript property to a localized number format
/// </field>
ko.bindingHandlers.number = {
    update: function (element, valueAccessor, allBindingsAccessor, viewModel) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();
        var format = allBindings.numberFormat || null;
        var hideZero = allBindings.hideZero || false;

        if (hideZero == true && (value == 0 || value == "0")) {
            ko.utils.setHtml($(element), '&nbsp;');
        }
        else if (value != null) {
            var formattedNumber;
            if (typeof value === 'number') 
                formattedNumber = value.format(format);
            else
                formattedNumber = parseFloat(value).format(format);
            ko.utils.setHtml($(element), formattedNumber);
        }

    }
};

/// <field type="Object" static="true">
/// Binds a JavaScript property to the valueAsDate property of an <input> field
/// </field>
ko.bindingHandlers.valueAsDate = {
    init: function (element, valueAccessor, allBindingsAccessor) {
        //error handling
        //make sure element has a valueAsDate property

        if (element.valueAsDate === undefined)
            throw new Error("valueAsDate binding must be bound to <input> of type date, datetime, or datetime-local");

        ko.utils.registerEventHandler(element, 'change', function () {
            //when the input box changes then update the observable
            valueAccessor()(element.valueAsDate);
        });
    },

    update: function (element, valueAccessor, allBindingsAccessor) {

        element.valueAsDate = ko.utils.unwrapObservable(valueAccessor());
    }
};

/// <field type="Object" static="true">
/// Binds a JavaScript property to a localized number format
/// </field>
ko.bindingHandlers.lookup = {
    update: function (element, valueAccessor, allBindingsAccessor, viewModel) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();
        var lookuplist = allBindings.lookuplist || null;
        var lookuppropertyname = allBindings.lookuppropertyname || 'name';

        if (!lookuplist){
            ko.utils.setHtml($(element), '&nbsp;');
        }
        else if (value != null) {
            var lookupvalue;

                var idValue = value;
                var listValue = lookuplist;
                if (typeof id === 'function')
                    idValue = id();

                if (typeof lookuplist === 'function')
                    listValue = lookuplist();

            if (listValue && listValue.length > 0) {
                for (var i = 0; i < listValue.length; i++) {
                    var currentListItem = listValue[i];

                    var listItemId = currentListItem.id;
                    if (typeof listItemId === 'function')
                        listItemId = listItemId();

                    if (listItemId == idValue)
                        lookupvalue = currentListItem[lookuppropertyname];
                }
            }

            if (lookupvalue == null) lookupvalue = '&nbsp;';
            ko.utils.setHtml($(element), lookupvalue);
        }

    }
};

//This is responsbile for providing regex patterns to html tags through knockout databinding.
//Ex. <input id="test1" type="text" data-bind="pattern: 'ipv4'"/> --> This would insert a pattern regex.
ko.bindingHandlers.pattern = {
    
    update: function (element, valueAccessor) {
        var patternName = ko.utils.unwrapObservable(valueAccessor());

        switch (patternName) {
            case 'ip':
            case 'ipv4':
                element.pattern = '\\b((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\\.|$)){4}\\b';
                break;
            case 'ipv4cidr':
                element.pattern = '\\b^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\\/(\\d|[0-9]\\d|9[0-9]))?$\\b';
                break;
            case 'ipv6':
                element.pattern = '\\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\\.(25[0-5]|2[0-4]\d|1\\d\\d|[1-9]?\\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)(\.(25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]?\\d)){3}))|:)))(%.+)?\\s*';
                break;
            default:
                throw new Error('Invalid Pattern Name' + patternName);
        }
    }
};

/// <field type="Object" static="true">
/// Binds the object's id to an indexed version
/// </field>
ko.bindingHandlers.indexedid = {
    update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();
        var index = bindingContext.$index();
        var prependHash = allBindings.prependHash || null;

        if (!isNullOrEmpty(value)) {
            if (prependHash && !value.startsWith('#/')) {
                value = '#/' + value;
            }
            element.id = value + '_' + index;
        }

    }
};

/// <field type="Object" static="true">
/// Binds the object's for attribute to an indexed version
/// </field>
ko.bindingHandlers.indexedfor = {
    update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();
        var index = bindingContext.$index();
        var prependHash = allBindings.prependHash || null;

        if (!isNullOrEmpty(value)) {
            if (prependHash) value = prependHash + value;
            $(element).attr("for", value + '_' + index);
        }

    }
};

/// <field type="Object" static="true">
/// Binds the object's name attribute to an indexed version
/// </field>
ko.bindingHandlers.indexedname = {
    update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();
        var index = bindingContext.$index();
        var prependHash = allBindings.prependHash || null;

        if (!isNullOrEmpty(value)) {
            if (prependHash) value = prependHash + value;
            $(element).attr("name", value + '_' + index);
        }

    }
};

/// <field type="Object" static="true">
/// Binds the object's click even to a sort method
/// </field>
ko.bindingHandlers.sort = {
    update: function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {

        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();

        if (!isNullOrEmpty(value)) {

            $(element).addClass('sortable');
        
            $(element).removeClass('sorted');
            $(element).removeClass('ascending');
            $(element).removeClass('descending');

            if (viewModel.sortObj && viewModel.sortObj()) {
                var currentSortObj = viewModel.sortObj()[0];

                if (currentSortObj && currentSortObj.field == value) {
                    if (currentSortObj.order == DellClarity.Constants.DataSortingConstants.DESCENDING) {
                        $(element).addClass('sort sorted descending');
                    } else {
                        $(element).addClass('sort sorted ascending');
                    }
                }
            }

            $(element).off('click').on('click', function () {
                viewModel.updateSort(value);
            });
        }

    }
};

/// <field type="Object" static="true">
/// Binds the object's class name
/// </field>
ko.bindingHandlers['class'] = {
    'update': function (element, valueAccessor) {
        if (element['__ko__previousClassValue__']) {
            $(element).removeClass(element['__ko__previousClassValue__']);
        }
        var value = ko.utils.unwrapObservable(valueAccessor());
        $(element).addClass(value);
        element['__ko__previousClassValue__'] = value;
    }
};

/// <field type="Object" static="true">
/// Binds the object's href attribue
/// </field>
ko.bindingHandlers.href = {
    update: function (element, valueAccessor, allBindingsAccessor) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        var allBindings = allBindingsAccessor();
        var prependHash = allBindings.prependHash || null;

        if (!isNullOrEmpty(value)) {
            if (prependHash && !value.startsWith('#/')) {
                value = '#/' + value;
            }
            $(element).attr("href", value);
        }

    }
};

/// <field type="Object" static="true">
/// Adds a change event to the object
/// </field>
ko.bindingHandlers.changeOnEnter = {
    init: function (element, valueAccessor) {
        var value = ko.utils.unwrapObservable(valueAccessor());
        $(element).keypress(function (event) {
            if (value == true) {
                var keyCode = (event.which ? event.which : event.keyCode);
                if (keyCode === 13) {
                    event.preventDefault();
                    $(element).change();
                    return false;
                }
            }
        });
    }
};

/// <field type="Object" static="true">
/// Displays Enabled/Disabled instead of true/false
/// </field>
ko.bindingHandlers.enabledText = {
    init: function (element, valueAccessor) {
        ko.bindingHandlers.enabledText.update(element, valueAccessor);
    },

    update: function (element, valueAccessor) {
        var value = valueAccessor();
        var unwrapped = ko.utils.unwrapObservable(value);
        ko.utils.setHtml($(element), unwrapped === true ? DellClarity.Localization.Translations.core.enabled : DellClarity.Localization.Translations.core.disabled);
    }
};

if (DellClarity.Config.apiVersion >= 2.5) {
    (function () {
        // Knockout.js is constantly adding and removing DOM elements via its
        // control flow bindings (if, ifnot, with, foreach, template), and DellClarity needs
        // to be notified when these DOM changes occur.  Knockout provides an afterRender event,
        // but it requires a handler each time a binding is used, which is too cumbersome.
        // So instead, we override each of the control flow bindings to trigger DellClarity events.
        function bindingWithEvents() {
            // Call the original Knockout binding
            var retVal = this.apply(window, arguments);

            // Raise DellClarity's domChange event
            DellClarity.__domChange.apply(window, arguments);

            return retVal;
        }

        function wrapBinding(bindingName) {
            var binding = ko.bindingHandlers[bindingName];

            if (binding.init) {
                var init = binding.init;
                binding.init = function () { return bindingWithEvents.apply(init, arguments); };
            }

            if (binding.update) {
                var update = binding.update;
                binding.update = function () { return bindingWithEvents.apply(update, arguments); };
            }
        }

        wrapBinding('with');
        wrapBinding('foreach');
        wrapBinding('if');
        wrapBinding('ifnot');
        wrapBinding('template');
        wrapBinding('visible');
    }());


    ko.mapping.fromJS = (function() {
        /// <summary>
        /// Enhances the default ko.mapping.fromJS() function.
        /// &#10;&#10;
        /// The second parameter of ko.mapping.fromJS is used to specify mapping options.
        /// In addition to the standard KO mapping options (http://knockoutjs.com/documentation/plugins-mapping.html),
        /// You can also specify the following custom DellClarity options:
        /// &#10;&#10;
        /// mapDateStrings:  Enables/disables automatic parsing and mapping of ISO 8601 formatted date strings.
        ///                  If enabled, these strings will automatically be converted to observable Date() objects.
        ///                  The default is true (enabled).
        /// &#10;&#10;
        /// mapObjects:      Enables/disables mapping of objects to observable objects.
        ///                  By default, only the properties of objects are mapped, not the objects themselves.
        ///                  Enabling this option will map the object AND its properties.
        ///                  This can sometimes be very useful, but can cause problems if used incorrectly, which is why
        ///                  The default is false (disabled).
        /// &#10;&#10;
        /// allowArray:      Enables/disables support for mapping the contents of an array (instead of an object).
        ///                  The original ko.mapping.fromJS() function requires the first parameter to be an object.
        ///                  Enabling this option allows it to be an array or an object.
        ///                  The default is true (enabled).
        /// </summary>

        var defaultOptions = {
            mapDateStrings: true,
            mapObjects: false,
            allowArray: true
        };

        // Save off the original mapping function
        var originalMappingFunc = ko.mapping.fromJS;

        // This function recursively converts all date-string properties
        // on an object to JavaScript Date properties

        function convertDateStrings(obj) {
            for (var prop in obj) {
                var propValue = obj[prop];

                if (typeof propValue === 'string' || propValue instanceof String) {
                    // We have a string!  So check if it's a date
                    if (propValue.isIsoDate()) {
                        // This property is a date string, so convert it to a Date object
                        obj[prop] = Date.parseDate(propValue);
                    }
                } else if (propValue instanceof Date || typeof propValue === 'function' || propValue instanceof Function) {
                    // Do nothing
                } else if (propValue instanceof Object) {
                    // Recursively convert this nested object's properties
                    // NOTE: This will also catch arrays, since they are objects
                    convertDateStrings(propValue);
                }
            }
        }

        ;

        // This function recursively converts all object properties to observable objects

        function convertObjects(obj) {
            for (var prop in obj) {
                var propValue = obj[prop];

                if (prop !== '__ko_mapping__' && !ko.isObservable(propValue)) {
                    if (typeof propValue === 'function' || propValue instanceof Function) {
                        // Do nothing
                    } else if (propValue instanceof Object) {
                        // First, recursively convert any nested objects
                        convertObjects(propValue);

                        // Now make this object observable
                        obj[prop] = ko.observable(propValue);
                    }
                }
            }
        }

        ;

        // Our new override of the ko.mapping.fromJS() function.
        // http://knockoutjs.com/documentation/plugins-mapping.html
        return function(model, options, target) {
            var unwrapArray, retVal;

            options = $.extend(true, $.extend(true, {}, defaultOptions), options);

            if (options.mapDateStrings) {
                // Convert any date-string properties
                convertDateStrings(model);
            }

            if (options.allowArray && $.isArray(model)) {
                // Wrap the model array in an object, so the original fromJS() 
                // function can process it.
                unwrapArray = true;
                model = { array: model };
            }

            // Perform the standard KO mapping and return the result
            if (options && target)
                retVal = originalMappingFunc(model, options, target);
            else if (options)
                retVal = originalMappingFunc(model, options);
            else
                retVal = originalMappingFunc(model);

            // If we wrapped the original model array in an object,
            // then we need to unwrap it now
            if (unwrapArray) {
                retVal = retVal.array();
            }

            if (options.mapObjects) {
                // Convert any objects into observables
                convertObjects(retVal);
            }

            // Return the final value
            return retVal;
        };
    })();
}


//#region Legacy API Support (backward compatibility)

if (DellClarity.Config.apiVersion < 2.5) {
    ko.bindingHandlers.slideUpVisible = {
        init: function (element, valueAccessor) {
            var value = valueAccessor();
            $(element).toggle(ko.utils.unwrapObservable(value));
        },
        update: function (element, valueAccessor) {
            var value = valueAccessor();
            ko.utils.unwrapObservable(value) ? $(element).show() : $(element).slideUp(200);
        }
    };

    ko.bindingHandlers.slideDownVisible = {
        init: function (element, valueAccessor) {
            var value = valueAccessor();
            $(element).toggle(ko.utils.unwrapObservable(value));
        },
        update: function (element, valueAccessor) {
            var value = valueAccessor();
            !ko.utils.unwrapObservable(value) ? $(element).hide() : $(element).slideDown(200);
        }
    };

    ko.bindingHandlers.slideVisible = {
        init: function (element, valueAccessor) {
            var value = valueAccessor();
            $(element).hide();
        },
        update: function (element, valueAccessor) {
            var value = ko.utils.unwrapObservable(valueAccessor());
            var $element = $(element);
            value ? $element.show() : $element.slideUp();
        }
    };

    ko.bindingHandlers.dynamicClass = {
        init: function (element, valueAccessor) {
            var value = valueAccessor();
            if (!isNullOrEmpty(value))
                $(element).addClass(ko.utils.unwrapObservable(value));
        },
        update: function (element, valueAccessor) {
            var value = valueAccessor();
            $(element).removeClass();
            if (!isNullOrEmpty(value))
                $(element).addClass(ko.utils.unwrapObservable(value));
        }
    };

    ko.bindingHandlers.expandIconLink = {
        setIconClass: function (element, theObservable) {
            $(element).removeClass('expand_normal_12');
            $(element).removeClass('collapse_normal_12');

            if (theObservable() === true)
                $(element).addClass('collapse_normal_12');
            else
                $(element).addClass('expand_normal_12');
        },

        init: function (element, valueAccessor) {
            var theObservable = valueAccessor();
            $(element).click(function () {
                theObservable(theObservable() === true ? false : true);
            });

            $(element).addClass('ui-icon');
            $(element).addClass('icon');
            ko.bindingHandlers.expandIconLink.setIconClass(element, theObservable);
        },
        update: function (element, valueAccessor) {
            var theObservable = valueAccessor();
            ko.bindingHandlers.expandIconLink.setIconClass(element, theObservable);
        }
    };


    ko.bindingHandlers.loadingText = {
        init: function (element, valueAccessor) {
            ko.bindingHandlers.loadingText.update(element, valueAccessor);
        },

        update: function (element, valueAccessor) {
            var requiredOptions = {
                loading: null,
                value: null
            };
            var options = ko.utils.unwrapObservable(valueAccessor() || requiredOptions);
            var loadingWrapped = options['loading'];
            var valueWrapped = options['value'];

            var loading = ko.utils.unwrapObservable(loadingWrapped);
            var value = ko.utils.unwrapObservable(valueWrapped);
            if (loading === false) {
                ko.utils.setHtml($(element), value !== null ? value : DellClarity.Localization.Translations.loadingText.noData);
            } else {
                $(element).html(DellClarity.Localization.Translations.loadingText.loading);
            }
        }
    };

    ko.bindingHandlers.hideZero = {
        init: function (element, valueAccessor, allBindingsAccessor) {
            ko.bindingHandlers.hideZero.update(element, valueAccessor, allBindingsAccessor);
        },

        update: function (element, valueAccessor, allBindingsAccessor) {
            var value = valueAccessor();
            var unwrapped = ko.utils.unwrapObservable(value);
            var allBindings = allBindingsAccessor();
            var isBoundToNumber = allBindings.number || false;
            if (!isBoundToNumber) {
                ko.utils.setHtml($(element), unwrapped == "0" ? '&nbsp;' : unwrapped);
            }
        }
    };

    ko.bindingHandlers.passwordText = {
        init: function (element, valueAccessor) {
            ko.bindingHandlers.passwordText.update(element, valueAccessor);
        },

        update: function (element, valueAccessor) {
            var value = valueAccessor();
            var unwrapped = ko.utils.unwrapObservable(value);
            ko.utils.setHtml($(element), isNullOrEmpty(unwrapped) ? DellClarity.Localization.Translations.core.nopassword : DellClarity.Localization.Translations.core.stars);
        }
    };


    ko.bindingHandlers.ellipsisText = {
        init: function (element, valueAccessor, allBindingsAccessor) {
            ko.bindingHandlers.ellipsisText.update(element, valueAccessor, allBindingsAccessor);
        },

        update: function (element, valueAccessor, allBindingsAccessor) {
            var value = valueAccessor(), allBindings = allBindingsAccessor();
            var length = allBindings.ellipsisLength || 10; // 10 is default length unless otherwise specified
            var unwrapped = ko.utils.unwrapObservable(value);
            var val = unwrapped;
            if (!isNullOrEmpty(unwrapped) && val.length > length) {
                $(element).addClass('toolTip-Dynamic');
                $(element).addClass('noicon');
                $(element).attr('title', unwrapped);
                val = unwrapped.substring(0, length - 1) + '&hellip;';
            }
            ko.utils.setHtml($(element), val);
        }
    };

    ko.validation.rules['mustEqual'] = {
        validator: function (val, mustEqualVal) {
            if (typeof (mustEqualVal) === 'function') return val === mustEqualVal();
            else return val === mustEqualVal;
        },
        message: 'This field must equal {0}'
    };

    ko.validation.rules['requiredIf'] = {
        validator: function (val, isRequired) {

            var required;

            if (typeof (isRequired) === 'function') {
                required = isRequired();
            }
            else {
                required = isRequired;
            }

            return !required || (val && (val + '').length > 0);
        },
        message: 'This field is required.'
    };

    ko.validation.rules['rule'] = {
        validator: function (val, rule) {

            var isTrue;

            if (typeof (rule) === 'function') {
                isTrue = rule();
            }
            else {
                isTrue = rule;
            }

            return isTrue;
        },
        message: 'This rule has not passed validation.'
    };

    ko.validation.rules['ipaddress'] = {
        validator: function (val, validate) {
            if (isNullOrEmpty(val)) return true;
            var ipaddressregex = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/;
            return validate && ipaddressregex.test(val);
        },
        message: '{0} is not a valid IP address'
    };

    ko.validation.rules['ipaddressWithOptionalPortOrHostname'] = {
        validator: function (val, validate) {
            if (isNullOrEmpty(val)) return true;
            return DellClarity.Helpers.Validate.ipAddressWithOptionalPortOrHostname(val);
        },
        message: 'Field must be a valid IP address or hostname.'
    };

    ko.validation.rules['emailOptional'] = {
        validator: function (val, validate) {
            return isNullOrEmpty(val) || /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))$/i.test(val);
        },
        message: '{0} is not a proper email address'
    };

    ko.validation.rules['nullableNumber'] = {
        validator: function (val, validate) {
            if (isNullOrEmpty(val)) return true;
            return ko.validation.rules['number'].validator(val, validate);
        },
        message: 'Must be a number.'
    };

    ko.validation.rules['date'] = {
        validator: function (val, validate) {
            if (isNullOrEmpty(val)) return true;
            var dateregex = /^(0?[1-9]|1[012])[\/\-](0?[1-9]|[12][0-9]|3[01])[\/\-]\d{4}$/;
            return validate && dateregex.test(val);
        },
        message: 'Must be a valid date in the format of MM/DD/YYYY.'
    };

    ko.validation.rules['time'] = {
        validator: function (val, validate) {
            if (isNullOrEmpty(val)) return true;
            var dateregex = /^(20|21|22|23|[0-1]?\d{1}):([0-5]?\d{1}):([0-5]?\d{1})$/;
            return validate && dateregex.test(val);
        },
        message: 'Must be a time in the format of HH:MM:SS.'
    };


    //  observable used for forms -- only saves actual value with a call to "commit()"
    //  functions and props:
    //  commit(): sets the actual value from the temp value
    //  reset(): clears out the temp value so that future calls to commit won't have the current temp data
    //  isCommittable: set to true, so you can see if this observable is 
    ko.formObservable = function (initialValue) {
        //private variables
        var _temp = initialValue;
        var _actual = ko.observable(initialValue);

        var result = ko.dependentObservable({
            read: function () {
                return _actual();
            },
            write: function (newValue) {
                _temp = newValue;
            }
        });

        //commit the temporary value to our observable, if it is different
        result.commit = function () {
            if (_temp !== _actual()) {
                _actual(_temp);
            }
        };

        //notify subscribers to update their value with the original
        result.reset = function () {
            _actual.valueHasMutated();
            _temp = _actual();
        };

        result.isCommittable = true;

        return result;
    };


    DellClarity.Knockout = {};
    DellClarity.Knockout.Helpers = {
        commitAll: function (model) {
            for (var propertyName in model) {
                var property = model[propertyName];

                if (property != null && property.isCommittable === true) {
                    property.commit();
                }
            }
        }
    }
}



//#endregion

