/// <field type="Object" static="true">
/// Automatically loads views/viewmodels based on URL hash changes.
/// &#10;&#10;
/// DellClarity.Navigation relies on a JSON object map (found in DellClarity.Urls.dependencyMap) that maps URL hashes to views.
/// This JSON object map also specifies all dependencies (scripts, viewmodels, views, etc.), and specifies the model to use.
/// Whenever the URL hash changes, DellClarity.Navigation finds the JSON object that corresponds to the new hash, and automatically
/// loads the appropriate viewmodels and dependencies.
/// </field>
DellClarity.Navigation = (function () {
    var retVal = {
        /// <field type="String" static="true">
        /// This is the default view that will be loaded if the URL does not contain a URL hash
        /// &#10;
        /// NOTE: Leave this blank if your .html pages have default content, or if your application is not an SPA (single page application).
        /// </field>
        defaultView: '',

        /// <field type="Object" static="true">
        /// A JSON object map that specifies all views in the application, along with their corresponding viewmodel, model, and dependencies (scripts, stylesheets, images, etc.)
        /// &#10;
        /// Applications must populate this object in order to take advantage of the automatic SPA hash-based navigation of DellClarity.Navigation.
        /// </field>
        viewMap: {

            /*
                EXAMPLE VIEW MAPS:

                // This is the simplest possible view.  It simply consists of HTML content
                home: 'views/homepage.html,

                // This view has CSS, JavaScript, and HTML template dependencies, which will automatically be loaded by DellClarity.Navigation (if they're not already loaded)
                somePage: {
                    url: 'views/somepage.html',
                    html: 'views/sometemplate.html',
                    css: 'styles/somepage.css',
                    js: 'scripts/somepage.js'
                },

                // This is a view that has a viewmodel (someotherpage.js). 
                // DellClarity.Navigation will automatially call the initialize() or update() method of the model object (DellClarity.MyApp.ViewModels.SomeOtherPage)
                someOtherPage: {
                    url: 'views/someotherpage.html',
                    js: 'scripts/viewmodels/someotherpage.js'
                    model: 'DellClarity.MyApp.ViewModels.SomeOtherPage'
                },

                // This is a view that has multiple viewmodels and HTML/CSS/JavaScript dependencies.
                // DellClarity.Navigation will automatically load the CSS and JavaScript dependencies, and will call initialize() or update() on all viewmodels
                anotherPage: {
                    url: 'views/anotherpage.html',
                    html: ['views/toolbartemplate.html', 'views/gridtemplate.html'],
                    css: ['styles/anotherpage.css', 'styles/toolbar.css', 'styles/grid.css'],
                    js: ['scripts/viewmodels/anotherpage.js', 'scripts/viewmodels/dependentpage.js', 'scripts/toolbar.js', 'scripts/grid.js'],
                    model: ['DellClarity.MyApp.ViewModels.AnotherPage', 'DellClarity.MyApp.ViewModels.DependentPage']
                },
            */
        },

        /// <field type="Object" static="false">
        /// A Knockout observable that notifies observers when the navigation cycle of a screen has been completed.
        /// </field>
        navigationComplete: ko.observable().extend({ notify: 'always' }),

        //#region Private DellClarity members. DO NOT USE THESE IN YOUR APPLICATION!

        

        /// <field>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</field>
        _modalShown: function (modal) {

            //hash already exists as target so don't add new hash again 
            //(this prevents multilevel modals from triggering history when you're just trying to close the modal and have no desire to go back to it)
            var existinghash = $.grep(modalViewHistory, function (e) { return e.currenthash == window.location.hash; });
            if (existinghash.length == 0)
                // Store the previous view hash, so we can restore it later when the modal is closed
                modalViewHistory.push({ hash: lastViewHash, modal: modal, currenthash: window.location.hash });
        },

        /// <field>This is a private DellClarity API function.  DO NOT USE IT IN YOUR APPLICATION!</field>
        _modalClosed: function (modal, navigateTo) {
            // Restore the view hash that was visible before the modal was shown

            var previousViewHistory = modalViewHistory.pop();

            //alert(previousViewHistory.hash);
            if (typeof (previousViewHistory) == "undefined")
                return;

            if (typeof navigateTo === 'string')
                window.location.hash = navigateTo;
            else if (previousViewHistory.hash)
                window.location.hash = previousViewHistory.hash;
            else
                window.location.hash = '';
        },

        //#endregion
    };

    // Compile the URL-hash-parsing RegExp object for improved performance
    var hashRegExp = /#\/([^\/]+)\/?(?:([^\/]+)\/?)?(?:([^\/]+)\/?)?(?:([^\/]+)\/?)?(?:([^\/]+)\/?)?(?:([^\/]+)\/?)?(?:([^\/]+)\/?)?(?:([^\/]+)\/?)?/;
    if (hashRegExp.compile) hashRegExp.compile(hashRegExp);

    // Keeps track of the previous view hash, so we know when it changes
    var lastViewHash = '';

    // Keeps track of the view hashes *before* each modal was shown
    var modalViewHistory = [];


    // Keeps track of the models that have already been initialized
    var initializedModels = [];

    // On DellClarity.initialize, load the view for the current URL hash
    var existingHandler = DellClarity.initialize;
    DellClarity.initialize = function () {
        // Call any previously-bound initialize function first
        if (existingHandler)
            existingHandler();

        // Make sure all pending resources are finished loading before we continue
        DellClarity.Resources.whenFinishedLoading(function () {
            // Load the view for the current URL hash
            navigateTo(location.hash);

            // Wire-up the hash-change listener
            $(window).on("hashchange", function () {
                // Navigate to the new view
                navigateTo(location.hash);
            });
        });
    };

    function navigateTo(viewHash) {
        /// <summary>
        /// Navigates to the specifeid view WITHOUT CHANGING THE ACTUAL PAGE URL.
        /// This includes loading all dependencies and initializing the view.
        /// </summary>

        retVal.navigationComplete(false);

        // Check to see if the hash is empty
        if (isNullUndefinedOrEmpty(viewHash) || viewHash === '#' || viewHash === '#/') {
            // Normalize the viewHash ('', '#', and '#/' are all the same)
            viewHash = '';

            // The hash is empty, so navigate to the default view instead
            if (!isNullUndefinedOrEmpty(DellClarity.Navigation.defaultView)) {
                // Make sure the default view starts with #/
                if (!DellClarity.Navigation.defaultView.startsWith('#/')) {
                    DellClarity.Navigation.defaultView = '#/' + DellClarity.Navigation.defaultView;
                }

                window.location.hash = DellClarity.Navigation.defaultView;

                //Notify observers that navigation is complete
                retVal.navigationComplete(true);

                return;
            }
            else {
                // There is no default view, and we're already on the homepage.
                // BUT, we may not be showing the original homepage content...
                if (!isNullUndefinedOrEmpty(lastViewHash)) {
                    // We had previously loaded a view hash, so just to make sure we're
                    // showing the original hompeage content, reload the page
                    window.location.reload();
                }

                //Notify observers that navigation is complete
                retVal.navigationComplete(true);

                return;
            }
        }

        // Parse the URL hash into a view name and arguments.
        var viewArgs = parseHash(viewHash);


        // If the hash couldn't be parsed, then just allow the browser to load the page as usual.
        // This allows DellClarity apps to use traditional hash URLs to navigate directly to parts of a page
        if (viewArgs === null) {
            return;
        }
		
		
		//JIT-201758: When DellClarity.Navigation.viewMap(urls.js) is not loaded in login view, Reloading the page based on navigation default view(login or #/login)
		if (Object.keys(DellClarity.Navigation.viewMap).length == 0 && (viewArgs.viewName == DellClarity.Navigation.defaultView || viewHash == DellClarity.Navigation.defaultView)) {
			window.location.reload();
			retVal.navigationComplete(true);
			
			return;
		}
		
        // Get the view map for this view
        var viewMap = getViewMap(viewArgs.viewName);

        // If we get this far, then the URL hash was successfully mapped to a view,
        // so load and initialize the view, starting with its dependencies
        console.group();

        // First, load the HTML dependencies (templates)
        loadTemplates(viewMap.html, function () {
            // Then, load the CSS dependencies
            DellClarity.Resources.load(viewMap.css, function () {
                // Then load the JavaScript dependencies
                DellClarity.Resources.load(viewMap.js, function () {
                    // Then load thew view (html content)
                    loadView(viewMap, function () {
                        // Instantiate the model(s)
                        instantiateModel(viewMap, viewArgs);

                        // Initialize the model(s)
                        initializeModel(viewMap, viewArgs);

                        // Bind the view to the model(s)
                        bindViewToModel(viewMap, viewArgs);

                        if (DellClarity.Config.apiVersion < 2.5) {
                            // DellClarity 1.0 required JavaScript resizing logic
                            DellClarity.Helpers.resize();
                        }

                        // Save this new viewHash, so we can detect changes later
                        lastViewHash = viewHash;

                        console.groupEnd();

                        //Notify observers that navigation is complete
                        retVal.navigationComplete(true);
                    });
                });
            });
        });
    }

    function parseHash(viewHash) {
        /// <summary>
        /// Parses the specified URL hash into separate parts for the view and its arguments.
        /// </summary>
        /// <remarks>
        /// Only seven arguments are supported.  Any additional arguments will be ignored.
        /// </remarks>

        // Parse the view hash using RegExp
        var hashParts = viewHash.match(hashRegExp);

        // Return null if the hash could not be parsed
        if (hashParts === null) {
            return null;
        }

        // Return an object containing the view and its arguments
        return {
            viewName: hashParts[1],
            arg1: hashParts[2],
            arg2: hashParts[3],
            arg3: hashParts[4],
            arg4: hashParts[5],
            arg5: hashParts[6],
            arg6: hashParts[7],
            arg7: hashParts[8],
        };
    }

    function getViewMap(viewName) {
        /// <summary>
        /// Returns the URL view map for the specified view name.
        /// The view map specifies the viewmodel, model, and dependencies (scripts, stylesheets, etc.) for the view.
        /// </summary>

        // Load the view map for this view
        var viewMap = Object.getNestedPropertyValue(DellClarity.Navigation.viewMap, viewName);

        // Throw and error if the view name could not be mapped to a view
        if (viewMap === null) {           
            throw new Error('DellClarity.Navigation.viewMap does not contain a view named "' + viewName + '"');
        }

        if (typeof viewMap === 'string') {
            // This is just a plain HTML view, no model or dependencies
            return {
                url: viewMap,
                html: undefined,
                css: undefined,
                js: undefined,
                model: undefined
            };
        }

        // All views must have a URL or a model
        if (isNullUndefinedOrEmpty(viewMap.url) && isNullUndefinedOrEmpty(viewMap.model)) {
            throw new Error('The "' + viewName + '" view in DellClarity.Navigation.viewMap does not have a url or model specified.  One or both are required.');
        }

        return viewMap;
    }

    function loadTemplates(urls, completeCallback) {
        /// <summary>
        /// Loads the specified HTML template(s).
        /// </summary>

        if (!isNullUndefinedOrEmpty(urls)) {
            // If urls is a string, convert it to an array
            if (typeof urls === 'string' || urls instanceof String) {
                urls = [urls];
            }

            // Load each HTML template in the array
            for (var i = 0; i < urls.length; i++) {
                DellClarity.Resources.loadTemplate(urls[i]);
            }
        }

        // Invoke the callback funciton
        if (completeCallback)
            completeCallback();
    }

    function loadView(viewMap, completeCallback) {
        /// <summary>
        /// Loads the specified view (html content) into the main content area of the page.
        /// </summary>

        var self = this;

        // If there is no view URL, then there's nothing to load.
        // Note: This is usually because the viewmodel loads its own view (e.g. a modal dialog, wizard, etc.)
        if (isNullUndefinedOrEmpty(viewMap.url)) {

            if (completeCallback)
                completeCallback();
            return;
        }

        // Cancel any current requests
        if (self.viewAjaxRequest) {
            self.viewAjaxRequest.abort();
            self.viewAjaxRequest = null;
        }

        // Fully-qualify the template URL
        var fullUrl = DellClarity.__resolveUrl(viewMap.url);

        // Download the view template and update the main content area
        self.viewAjaxRequest = DellClarity.Ajax.UpdateWithHtml(fullUrl, DellClarity.__mainViewContainer, function () {
            // We've finished loading the template, so clear the AJAX object
            self.viewAjaxRequest = null;

            // Execute the callback function 
            if (completeCallback)
                completeCallback();
        });
    }

    function instantiateModel(viewMap, viewArgs) {
        /// <summary>
        /// Instantiates the model object(s) for the specified view, or re-uses the existing instances, if they already exist.
        /// </summary>

        // If the view doesn't have a model, then there's nothing to do
        if (isNullUndefinedOrEmpty(viewMap.model)) {
            return;
        }

        // if viewMap.model is a string, then convert it to a single-item array
        if (typeof viewMap.model === 'string') {
            viewMap.model = [viewMap.model];
        }

        // Instantiate each model object
        for (var i = 0; i < viewMap.model.length; i++) {
            if (typeof viewMap.model[i] === 'string') {
                var modelName = viewMap.model[i];
                var model = null;

                try {
                    // The model has not been instantiated yet, so eval() the string to instantiate it
                    // (this will throw an error if invalid)
                    model = eval(modelName);

                    // If the model evals to a null or undefined object, then throw an error anyway
                    if (isNullOrUndefined(model)) {
                        throw new Error();
                    }
                }
                catch (e) {
                    throw new Error('The ' + viewArgs.viewName + ' view relies on the "' + modelName + '" model, which could not be found.');
                }

                // Overwrite the model name with the actual model object,
                // so we don't have to re-instantiate it next time
                viewMap.model[i] = { name: modelName, instance: model };
            }
        }
    }

    function initializeModel(viewMap, viewArgs) {
        /// <summary>
        /// Initializes the specified view's models, passing the specified arguments.
        /// </summary>

        // If the view doesn't have a model, then there's nothing to do
        if (isNullUndefinedOrEmpty(viewMap.model)) {
            return;
        }

        // Initialize/Update each model
        for (var i = 0; i < viewMap.model.length; i++) {
            var modelName = viewMap.model[i].name;
            var model = viewMap.model[i].instance;

            // Prior to DellClarity 2.5, there was a public API property called currentViewModel.  This property is now deprecated.
            // Future versions of DellClarity.Navigation may expose a similar property, but with support for multiple concurrent models
            if (DellClarity.Config.apiVersion < 2.5) {
                DellClarity.Navigation.currentViewModel = model;
            }

            // Determine whether the model has already been initialized
            var alreadyInitialized = false;
            for (var j = 0; j < initializedModels.length; j++) {
                if (initializedModels[j] === model) {
                    // This model has already been initialized
                    alreadyInitialized = true;
                    break;
                }
            }

            if (!alreadyInitialized) {
                // Call initialize(), passing it the view arguments
                if (typeof model.initialize === 'function') {
                    devLog(modelName + '.initialize()');
                    model.initialize.call(model, viewArgs.arg1, viewArgs.arg2, viewArgs.arg3, viewArgs.arg4, viewArgs.arg5, viewArgs.arg6, viewArgs.arg7);
                }
                else {
                    devLog(modelName + '.initialize() does not exist. Skipping it.');
                }

                // Add it to the list of initialized models, so we'll call update() instead next time
                initializedModels.push(model);
            }
            else {
                // Call update(), passing it the view arguments
                if (typeof model.update === 'function') {
                    devLog(modelName + '.update()');
                    model.update.call(model, viewArgs.arg1, viewArgs.arg2, viewArgs.arg3, viewArgs.arg4, viewArgs.arg5, viewArgs.arg6, viewArgs.arg7);
                }
                else if (DellClarity.Config.apiVersion < 2.5) {
                    // Prior to DellClarity 2.5, the initialize() function was called every time, if there was no update() function
                    if (typeof model.initialize === 'function') {
                        devLog(modelName + 'initialize()');
                        model.initialize.call(model, viewArgs.arg1, viewArgs.arg2, viewArgs.arg3, viewArgs.arg4, viewArgs.arg5, viewArgs.arg6, viewArgs.arg7);
                    }
                    else {
                        devLog(modelName + '.initialize() does not exist. Skipping it.');
                    }
                }
                else {
                    devLog(modelName + '.update() does not exist. Skipping it.');
                }
            }
        }
    }

    function cleanNode(element)
    {
        /*ko.utils.domNodeDisposal.cleanExternalData = function () {
            // Do nothing. Now any jQuery data associated with elements will
            // not be cleaned up when the elements are removed from the DOM.
        };*/

        var original = ko.utils.domNodeDisposal['cleanExternalData'];
        ko.utils.domNodeDisposal['cleanExternalData'] = function () { };
        ko.cleanNode(element);
        ko.utils.domNodeDisposal['cleanExternalData'] = original;
    }

    function bindViewToModel(viewMap, viewArgs) {
        /// <summary>
        /// Binds the view to the model(s).
        /// </summary>

        // Determine what to bind to
        var mainContentData = ko.dataFor(DellClarity.__mainViewContainer);
        if (DellClarity.Config.apiVersion < 2.5) {
            // In DellClarity 1.0, the mainViewContainer had to be bound to DellClarity.ModelBinding.baseViewModel
            mainContentData = DellClarity.ModelBinding.baseViewModel || mainContentData;
        }

        // Apply the bindings
        if (modalViewHistory.length > 0) {
            var modalHistory = modalViewHistory[modalViewHistory.length - 1];
			cleanNode(modalHistory.modal.getElement());
            ko.applyBindings(modalHistory.modal, modalHistory.modal.getElement());
            modalHistory.modal.bindComplete();
        }
        else{
            if (window.location.hash != "#/logout") {
                cleanNode(DellClarity.__mainViewContainer);
                ko.applyBindings(mainContentData, DellClarity.__mainViewContainer);
            }
        }

        // If the model(s) supports bindComplete(), then call it now
        if (!isNullUndefinedOrEmpty(viewMap.model)) {
            for (var i = 0; i < viewMap.model.length; i++) {
                var modelName = viewMap.model[i].name;
                var model = viewMap.model[i].instance;
                if (typeof model.bindComplete === 'function') {
                    devLog(modelName + '.bindComplete()');
                    model.bindComplete();
                }
                else {
                    devLog(modelName + '.bindComplete() does not exist. Skipping it.');
                }
            }
        }
    }

    //#region Legacy API Support (backward compatibility)

    if (DellClarity.Config.apiVersion < 2.5) {
        if (typeof DEFAULT_ROUTE !== 'undefined' && DEFAULT_ROUTE !== '')
            retVal.defaultView = '#/' + DEFAULT_ROUTE;

        jQuery.observeHashChange = function (options) { };
        jQuery.fn.hashchange = function (callback) { };

        window.navigationConfigLoader = {
            initialize: function (menuConfig) { }
        };

        retVal.updateMainContent = function (templateUrl) {
            loadView({
                url: templateUrl
            });
        };

        retVal.Router = {
            initialize: function () {
            },
            setDefaultRoute: function (defaultRoute) {
                if (isNullUndefinedOrEmpty(defaultRoute))
                    defaultRoute = '';

                if (!defaultRoute.startsWith('#/'))
                    defaultRoute = '#/' + defaultRoute;

                DellClarity.Navigation.defaultView = defaultRoute;
            },
            gotoRoute: function (route) {
                if (route.indexOf('/') === -1) route = "/" + route;
                window.location.hash = route;
            },
            gotoRouteAndReload: function (url, route) {
                if (route === '') window.location = url;
                else window.location = url + '#' + route;
            },
            redirect: function (url, route) {
                gotoRouteAndReload(url, route);
            }
        };
    }

    //#endregion

    // Returnt the public API
    return retVal;
})();
